@@ -13,9 +13,6 @@ export CC='clang'
 # Change to detect_leaks=1 to detect memory leaks (slower).
 export ASAN_OPTIONS="detect_leaks=0:log_path=$log_path/asan"
 
-# Show backtraces in the logs.
-export UBSAN_OPTIONS="print_stacktrace=1"
-
 make -C "$root_path" CMAKE_EXTRA_FLAGS="-DENABLE_ASAN_UBSAN=ON"
 VIMRUNTIME="$root_path"/runtime "$root_path"/build/bin/nvim
 
