@@ -1,74 +1,112 @@
-// This is an open source non-commercial project. Dear PVS-Studio, please check
-// it. PVS-Studio Static Code Analyzer for C, C++ and C#: http://www.viva64.com
-
 #include <assert.h>
-#include <limits.h>
+#include <stdbool.h>
+#include <stddef.h>
+#include <stdint.h>
 #include <stdlib.h>
+#include <string.h>
 
+#include "klib/kvec.h"
+#include "nvim/api/keysets_defs.h"
 #include "nvim/api/private/converter.h"
 #include "nvim/api/private/defs.h"
 #include "nvim/api/private/helpers.h"
 #include "nvim/api/vimscript.h"
-#include "nvim/ascii.h"
+#include "nvim/ascii_defs.h"
+#include "nvim/buffer_defs.h"
 #include "nvim/eval.h"
 #include "nvim/eval/typval.h"
+#include "nvim/eval/typval_defs.h"
 #include "nvim/eval/userfunc.h"
-#include "nvim/ex_cmds2.h"
+#include "nvim/ex_docmd.h"
+#include "nvim/garray.h"
+#include "nvim/garray_defs.h"
+#include "nvim/globals.h"
+#include "nvim/memory.h"
+#include "nvim/memory_defs.h"
+#include "nvim/runtime.h"
+#include "nvim/vim_defs.h"
 #include "nvim/viml/parser/expressions.h"
 #include "nvim/viml/parser/parser.h"
+#include "nvim/viml/parser/parser_defs.h"
 
 #ifdef INCLUDE_GENERATED_DECLARATIONS
 # include "api/vimscript.c.generated.h"
 #endif
 
-/// Executes Vimscript (multiline block of Ex-commands), like anonymous
+/// Executes Vimscript (multiline block of Ex commands), like anonymous
 /// |:source|.
 ///
 /// Unlike |nvim_command()| this function supports heredocs, script-scope (s:),
 /// etc.
 ///
-/// On execution error: fails with VimL error, does not update v:errmsg.
+/// On execution error: fails with Vimscript error, updates v:errmsg.
 ///
 /// @see |execute()|
 /// @see |nvim_command()|
+/// @see |nvim_cmd()|
 ///
 /// @param src      Vimscript code
-/// @param output   Capture and return all (non-error, non-shell |:!|) output
+/// @param opts  Optional parameters.
+///           - output: (boolean, default false) Whether to capture and return
+///                     all (non-error, non-shell |:!|) output.
 /// @param[out] err Error details (Vim error), if any
-/// @return Output (non-error, non-shell |:!|) if `output` is true,
-///         else empty string.
-String nvim_exec(uint64_t channel_id, String src, Boolean output, Error *err)
-  FUNC_API_SINCE(7)
+/// @return Dict containing information about execution, with these keys:
+///       - output: (string|nil) Output if `opts.output` is true.
+Dict nvim_exec2(uint64_t channel_id, String src, Dict(exec_opts) *opts, Error *err)
+  FUNC_API_SINCE(11) FUNC_API_RET_ALLOC
+{
+  Dict result = ARRAY_DICT_INIT;
+
+  String output = exec_impl(channel_id, src, opts, err);
+  if (ERROR_SET(err)) {
+    return result;
+  }
+
+  if (opts->output) {
+    PUT(result, "output", STRING_OBJ(output));
+  }
+
+  return result;
+}
+
+String exec_impl(uint64_t channel_id, String src, Dict(exec_opts) *opts, Error *err)
 {
   const int save_msg_silent = msg_silent;
+  const bool save_redir_off = redir_off;
   garray_T *const save_capture_ga = capture_ga;
+  const int save_msg_col = msg_col;
   garray_T capture_local;
-  if (output) {
+  if (opts->output) {
     ga_init(&capture_local, 1, 80);
     capture_ga = &capture_local;
   }
 
-  try_start();
-  if (output) {
-    msg_silent++;
-  }
+  TRY_WRAP(err, {
+    if (opts->output) {
+      msg_silent++;
+      redir_off = false;
+      msg_col = 0;  // prevent leading spaces
+    }
 
-  const sctx_T save_current_sctx = api_set_sctx(channel_id);
+    const sctx_T save_current_sctx = api_set_sctx(channel_id);
 
-  do_source_str(src.data, "nvim_exec()");
-  if (output) {
-    capture_ga = save_capture_ga;
-    msg_silent = save_msg_silent;
-  }
+    do_source_str(src.data, "nvim_exec2()");
+    if (opts->output) {
+      capture_ga = save_capture_ga;
+      msg_silent = save_msg_silent;
+      redir_off = save_redir_off;
+      // Put msg_col back where it was, since nothing should have been written.
+      msg_col = save_msg_col;
+    }
 
-  current_sctx = save_current_sctx;
-  try_end(err);
+    current_sctx = save_current_sctx;
+  });
 
   if (ERROR_SET(err)) {
     goto theend;
   }
 
-  if (output && capture_local.ga_len > 1) {
+  if (opts->output && capture_local.ga_len > 1) {
     String s = (String){
       .data = capture_local.ga_data,
       .size = (size_t)capture_local.ga_len,
@@ -76,88 +114,91 @@ String nvim_exec(uint64_t channel_id, String src, Boolean output, Error *err)
     // redir usually (except :echon) prepends a newline.
     if (s.data[0] == '\n') {
       memmove(s.data, s.data + 1, s.size - 1);
-      s.data[s.size - 1] = '\0';
+      s.data[s.size - 1] = NUL;
       s.size = s.size - 1;
     }
     return s;  // Caller will free the memory.
   }
 theend:
-  if (output) {
+  if (opts->output) {
     ga_clear(&capture_local);
   }
   return (String)STRING_INIT;
 }
 
-/// Executes an ex-command.
+/// Executes an Ex command.
 ///
-/// On execution error: fails with VimL error, does not update v:errmsg.
+/// On execution error: fails with Vimscript error, updates v:errmsg.
 ///
-/// @see |nvim_exec()|
+/// Prefer |nvim_cmd()| or |nvim_exec2()| instead. To modify an Ex command in a structured way
+/// before executing it, modify the result of |nvim_parse_cmd()| then pass it to |nvim_cmd()|.
 ///
-/// @param command  Ex-command string
+/// @param command  Ex command string
 /// @param[out] err Error details (Vim error), if any
 void nvim_command(String command, Error *err)
   FUNC_API_SINCE(1)
 {
-  try_start();
-  do_cmdline_cmd(command.data);
-  try_end(err);
+  TRY_WRAP(err, {
+    do_cmdline_cmd(command.data);
+  });
 }
 
-/// Evaluates a VimL |expression|.
-/// Dictionaries and Lists are recursively expanded.
+/// Evaluates a Vimscript |expression|. Dicts and Lists are recursively expanded.
 ///
-/// On execution error: fails with VimL error, does not update v:errmsg.
+/// On execution error: fails with Vimscript error, updates v:errmsg.
 ///
-/// @param expr     VimL expression string
+/// @param expr     Vimscript expression string
 /// @param[out] err Error details, if any
 /// @return         Evaluation result or expanded object
-Object nvim_eval(String expr, Error *err)
+Object nvim_eval(String expr, Arena *arena, Error *err)
   FUNC_API_SINCE(1)
 {
   static int recursive = 0;  // recursion depth
   Object rv = OBJECT_INIT;
 
-  TRY_WRAP({
-    // Initialize `force_abort`  and `suppress_errthrow` at the top level.
-    if (!recursive) {
-      force_abort = false;
-      suppress_errthrow = false;
-      current_exception = NULL;
-      // `did_emsg` is set by emsg(), which cancels execution.
-      did_emsg = false;
-    }
-    recursive++;
-    try_start();
+  // Initialize `force_abort`  and `suppress_errthrow` at the top level.
+  if (!recursive) {
+    force_abort = false;
+    suppress_errthrow = false;
+    did_throw = false;
+    // `did_emsg` is set by emsg(), which cancels execution.
+    did_emsg = false;
+  }
 
-    typval_T rettv;
-    int ok = eval0((char_u *)expr.data, &rettv, NULL, true);
+  recursive++;
 
-    if (!try_end(err)) {
-      if (ok == FAIL) {
-        // Should never happen, try_end() should get the error. #8371
-        api_set_error(err, kErrorTypeException,
-                      "Failed to evaluate expression: '%.*s'", 256, expr.data);
-      } else {
-        rv = vim_to_object(&rettv);
-      }
-    }
+  typval_T rettv;
+  int ok;
 
-    tv_clear(&rettv);
-    recursive--;
+  TRY_WRAP(err, {
+    ok = eval0(expr.data, &rettv, NULL, &EVALARG_EVALUATE);
+    clear_evalarg(&EVALARG_EVALUATE, NULL);
   });
 
+  if (!ERROR_SET(err)) {
+    if (ok == FAIL) {
+      // Should never happen, try_end() (in TRY_WRAP) should get the error. #8371
+      api_set_error(err, kErrorTypeException,
+                    "Failed to evaluate expression: '%.*s'", 256, expr.data);
+    } else {
+      rv = vim_to_object(&rettv, arena, false);
+    }
+  }
+
+  tv_clear(&rettv);
+  recursive--;
+
   return rv;
 }
 
-/// Calls a VimL function.
+/// Calls a Vimscript function.
 ///
 /// @param fn Function name
 /// @param args Function arguments
 /// @param self `self` dict, or NULL for non-dict functions
 /// @param[out] err Error details, if any
 /// @return Result of the function call
-static Object _call_function(String fn, Array args, dict_T *self, Error *err)
+static Object _call_function(String fn, Array args, dict_T *self, Arena *arena, Error *err)
 {
   static int recursive = 0;  // recursion depth
   Object rv = OBJECT_INIT;
@@ -172,40 +213,39 @@ static Object _call_function(String fn, Array args, dict_T *self, Error *err)
   typval_T vim_args[MAX_FUNC_ARGS + 1];
   size_t i = 0;  // also used for freeing the variables
   for (; i < args.size; i++) {
-    if (!object_to_vim(args.items[i], &vim_args[i], err)) {
-      goto free_vim_args;
-    }
+    object_to_vim(args.items[i], &vim_args[i], err);
   }
 
-  TRY_WRAP({
-    // Initialize `force_abort`  and `suppress_errthrow` at the top level.
-    if (!recursive) {
-      force_abort = false;
-      suppress_errthrow = false;
-      current_exception = NULL;
-      // `did_emsg` is set by emsg(), which cancels execution.
-      did_emsg = false;
-    }
-    recursive++;
-    try_start();
-    typval_T rettv;
-    funcexe_T funcexe = FUNCEXE_INIT;
-    funcexe.firstline = curwin->w_cursor.lnum;
-    funcexe.lastline = curwin->w_cursor.lnum;
-    funcexe.evaluate = true;
-    funcexe.selfdict = self;
-    // call_func() retval is deceptive, ignore it.  Instead we set `msg_list`
-    // (see above) to capture abort-causing non-exception errors.
-    (void)call_func((char_u *)fn.data, (int)fn.size, &rettv, (int)args.size,
-                    vim_args, &funcexe);
-    if (!try_end(err)) {
-      rv = vim_to_object(&rettv);
-    }
-    tv_clear(&rettv);
-    recursive--;
+  // Initialize `force_abort`  and `suppress_errthrow` at the top level.
+  if (!recursive) {
+    force_abort = false;
+    suppress_errthrow = false;
+    did_throw = false;
+    // `did_emsg` is set by emsg(), which cancels execution.
+    did_emsg = false;
+  }
+  recursive++;
+
+  typval_T rettv;
+  funcexe_T funcexe = FUNCEXE_INIT;
+  funcexe.fe_firstline = curwin->w_cursor.lnum;
+  funcexe.fe_lastline = curwin->w_cursor.lnum;
+  funcexe.fe_evaluate = true;
+  funcexe.fe_selfdict = self;
+
+  TRY_WRAP(err, {
+    // call_func() retval is deceptive, ignore it.  Instead TRY_WRAP sets `msg_list` to capture
+    // abort-causing non-exception errors.
+    (void)call_func(fn.data, (int)fn.size, &rettv, (int)args.size, vim_args, &funcexe);
   });
 
-free_vim_args:
+  if (!ERROR_SET(err)) {
+    rv = vim_to_object(&rettv, arena, false);
+  }
+
+  tv_clear(&rettv);
+  recursive--;
+
   while (i > 0) {
     tv_clear(&vim_args[--i]);
   }
@@ -213,30 +253,30 @@ free_vim_args:
   return rv;
 }
 
-/// Calls a VimL function with the given arguments.
+/// Calls a Vimscript function with the given arguments.
 ///
-/// On execution error: fails with VimL error, does not update v:errmsg.
+/// On execution error: fails with Vimscript error, updates v:errmsg.
 ///
 /// @param fn       Function to call
 /// @param args     Function arguments packed in an Array
 /// @param[out] err Error details, if any
 /// @return Result of the function call
-Object nvim_call_function(String fn, Array args, Error *err)
+Object nvim_call_function(String fn, Array args, Arena *arena, Error *err)
   FUNC_API_SINCE(1)
 {
-  return _call_function(fn, args, NULL, err);
+  return _call_function(fn, args, NULL, arena, err);
 }
 
-/// Calls a VimL |Dictionary-function| with the given arguments.
+/// Calls a Vimscript |Dictionary-function| with the given arguments.
 ///
-/// On execution error: fails with VimL error, does not update v:errmsg.
+/// On execution error: fails with Vimscript error, updates v:errmsg.
 ///
-/// @param dict Dictionary, or String evaluating to a VimL |self| dict
-/// @param fn Name of the function defined on the VimL dict
+/// @param dict Dict, or String evaluating to a Vimscript |self| dict
+/// @param fn Name of the function defined on the Vimscript dict
 /// @param args Function arguments packed in an Array
 /// @param[out] err Error details, if any
 /// @return Result of the function call
-Object nvim_call_dict_function(Object dict, String fn, Array args, Error *err)
+Object nvim_call_dict_function(Object dict, String fn, Array args, Arena *arena, Error *err)
   FUNC_API_SINCE(4)
 {
   Object rv = OBJECT_INIT;
@@ -244,27 +284,28 @@ Object nvim_call_dict_function(Object dict, String fn, Array args, Error *err)
   typval_T rettv;
   bool mustfree = false;
   switch (dict.type) {
-  case kObjectTypeString:
-    try_start();
-    if (eval0((char_u *)dict.data.string.data, &rettv, NULL, true) == FAIL) {
-      api_set_error(err, kErrorTypeException,
-                    "Failed to evaluate dict expression");
-    }
-    if (try_end(err)) {
+  case kObjectTypeString: {
+    int eval_ret;
+    TRY_WRAP(err, {
+        eval_ret = eval0(dict.data.string.data, &rettv, NULL, &EVALARG_EVALUATE);
+        clear_evalarg(&EVALARG_EVALUATE, NULL);
+      });
+    if (ERROR_SET(err)) {
       return rv;
     }
+    if (eval_ret != OK) {
+      abort();  // Should not happen.
+    }
     // Evaluation of the string arg created a new dict or increased the
     // refcount of a dict. Not necessary for a RPC dict.
     mustfree = true;
     break;
-  case kObjectTypeDictionary:
-    if (!object_to_vim(dict, &rettv, err)) {
-      goto end;
-    }
+  }
+  case kObjectTypeDict:
+    object_to_vim(dict, &rettv, err);
     break;
   default:
-    api_set_error(err, kErrorTypeValidation,
-                  "dict argument type must be String or Dictionary");
+    api_set_error(err, kErrorTypeValidation, "dict argument type must be String or Dict");
     return rv;
   }
   dict_T *self_dict = rettv.vval.v_dict;
@@ -273,7 +314,7 @@ Object nvim_call_dict_function(Object dict, String fn, Array args, Error *err)
     goto end;
   }
 
-  if (fn.data && fn.size > 0 && dict.type != kObjectTypeDictionary) {
+  if (fn.data && fn.size > 0 && dict.type != kObjectTypeDict) {
     dictitem_T *const di = tv_dict_find(self_dict, fn.data, (ptrdiff_t)fn.size);
     if (di == NULL) {
       api_set_error(err, kErrorTypeValidation, "Not found: %s", fn.data);
@@ -289,8 +330,8 @@ Object nvim_call_dict_function(Object dict, String fn, Array args, Error *err)
       goto end;
     }
     fn = (String) {
-      .data = (char *)di->di_tv.vval.v_string,
-      .size = STRLEN(di->di_tv.vval.v_string),
+      .data = di->di_tv.vval.v_string,
+      .size = strlen(di->di_tv.vval.v_string),
     };
   }
 
@@ -299,7 +340,7 @@ Object nvim_call_dict_function(Object dict, String fn, Array args, Error *err)
     goto end;
   }
 
-  rv = _call_function(fn, args, self_dict, err);
+  rv = _call_function(fn, args, self_dict, arena, err);
 end:
   if (mustfree) {
     tv_clear(&rettv);
@@ -313,11 +354,9 @@ typedef struct {
   Object *ret_node_p;
 } ExprASTConvStackItem;
 
-/// @cond DOXYGEN_NOT_A_FUNCTION
 typedef kvec_withinit_t(ExprASTConvStackItem, 16) ExprASTConvStack;
-/// @endcond
 
-/// Parse a VimL expression.
+/// Parse a Vimscript expression.
 ///
 /// @param[in]  expr  Expression to parse. Always treated as a single line.
 /// @param[in]  flags Flags:
@@ -329,8 +368,8 @@ typedef kvec_withinit_t(ExprASTConvStackItem, 16) ExprASTConvStack;
 ///                    - "l" when needing to start parsing with lvalues for
 ///                      ":let" or ":for".
 ///                    Common flag sets:
-///                    - "m" to parse like for ":echo".
-///                    - "E" to parse like for "<C-r>=".
+///                    - "m" to parse like for `":echo"`.
+///                    - "E" to parse like for `"<C-r>="`.
 ///                    - empty string for ":call".
 ///                    - "lm" to parse for ":let".
 /// @param[in]  highlight  If true, return value will also include "highlight"
@@ -341,22 +380,22 @@ typedef kvec_withinit_t(ExprASTConvStackItem, 16) ExprASTConvStack;
 ///                        one should highlight region [start_col, end_col)).
 ///
 /// @return
-///      - AST: top-level dictionary with these keys:
-///        - "error": Dictionary with error, present only if parser saw some
+///      - AST: top-level dict with these keys:
+///        - "error": Dict with error, present only if parser saw some
 ///                 error. Contains the following keys:
 ///          - "message": String, error message in printf format, translated.
 ///                       Must contain exactly one "%.*s".
 ///          - "arg": String, error message argument.
 ///        - "len": Amount of bytes successfully parsed. With flags equal to ""
 ///                 that should be equal to the length of expr string.
-///                 (“Successfully parsed” here means “participated in AST
-///                  creation”, not “till the first error”.)
-///        - "ast": AST, either nil or a dictionary with these keys:
+///                 ("Successfully parsed" here means "participated in AST
+///                  creation", not "till the first error".)
+///        - "ast": AST, either nil or a dict with these keys:
 ///          - "type": node type, one of the value names from ExprASTNodeType
 ///                    stringified without "kExprNode" prefix.
-///          - "start": a pair [line, column] describing where node is "started"
+///          - "start": a pair `[line, column]` describing where node is "started"
 ///                     where "line" is always 0 (will not be 0 if you will be
-///                     using nvim_parse_viml() on e.g. ":let", but that is not
+///                     using this API on e.g. ":let", but that is not
 ///                     present yet). Both elements are Integers.
 ///          - "len": “length” of the node. This and "start" are there for
 ///                   debugging purposes primary (debugging parser and providing
@@ -391,7 +430,7 @@ typedef kvec_withinit_t(ExprASTConvStackItem, 16) ExprASTConvStack;
 ///        - "svalue": String, value for "SingleQuotedString" and
 ///                    "DoubleQuotedString" nodes.
 /// @param[out] err Error details, if any
-Dictionary nvim_parse_expression(String expr, String flags, Boolean highlight, Error *err)
+Dict nvim_parse_expression(String expr, String flags, Boolean highlight, Arena *arena, Error *err)
   FUNC_API_SINCE(4) FUNC_API_FAST
 {
   int pflags = 0;
@@ -406,11 +445,11 @@ Dictionary nvim_parse_expression(String expr, String flags, Boolean highlight, E
     case NUL:
       api_set_error(err, kErrorTypeValidation, "Invalid flag: '\\0' (%u)",
                     (unsigned)flags.data[i]);
-      return (Dictionary)ARRAY_DICT_INIT;
+      return (Dict)ARRAY_DICT_INIT;
     default:
       api_set_error(err, kErrorTypeValidation, "Invalid flag: '%c' (%u)",
                     flags.data[i], (unsigned)flags.data[i]);
-      return (Dictionary)ARRAY_DICT_INIT;
+      return (Dict)ARRAY_DICT_INIT;
     }
   }
   ParserLine parser_lines[] = {
@@ -433,82 +472,40 @@ Dictionary nvim_parse_expression(String expr, String flags, Boolean highlight, E
                            + (size_t)(east.err.msg != NULL)  // "error"
                            + (size_t)highlight  // "highlight"
                            + 0);
-  Dictionary ret = {
-    .items = xmalloc(ret_size * sizeof(ret.items[0])),
-    .size = 0,
-    .capacity = ret_size,
-  };
-  ret.items[ret.size++] = (KeyValuePair) {
-    .key = STATIC_CSTR_TO_STRING("ast"),
-    .value = NIL,
-  };
-  ret.items[ret.size++] = (KeyValuePair) {
-    .key = STATIC_CSTR_TO_STRING("len"),
-    .value = INTEGER_OBJ((Integer)(pstate.pos.line == 1
-                                   ? parser_lines[0].size
-                                   : pstate.pos.col)),
-  };
+
+  Dict ret = arena_dict(arena, ret_size);
+  PUT_C(ret, "len", INTEGER_OBJ((Integer)(pstate.pos.line == 1
+                                          ? parser_lines[0].size
+                                          : pstate.pos.col)));
   if (east.err.msg != NULL) {
-    Dictionary err_dict = {
-      .items = xmalloc(2 * sizeof(err_dict.items[0])),
-      .size = 2,
-      .capacity = 2,
-    };
-    err_dict.items[0] = (KeyValuePair) {
-      .key = STATIC_CSTR_TO_STRING("message"),
-      .value = STRING_OBJ(cstr_to_string(east.err.msg)),
-    };
-    if (east.err.arg == NULL) {
-      err_dict.items[1] = (KeyValuePair) {
-        .key = STATIC_CSTR_TO_STRING("arg"),
-        .value = STRING_OBJ(STRING_INIT),
-      };
-    } else {
-      err_dict.items[1] = (KeyValuePair) {
-        .key = STATIC_CSTR_TO_STRING("arg"),
-        .value = STRING_OBJ(((String) {
-          .data = xmemdupz(east.err.arg, (size_t)east.err.arg_len),
-          .size = (size_t)east.err.arg_len,
-        })),
-      };
-    }
-    ret.items[ret.size++] = (KeyValuePair) {
-      .key = STATIC_CSTR_TO_STRING("error"),
-      .value = DICTIONARY_OBJ(err_dict),
-    };
+    Dict err_dict = arena_dict(arena, 2);
+    PUT_C(err_dict, "message", CSTR_TO_ARENA_OBJ(arena, east.err.msg));
+    PUT_C(err_dict, "arg", CBUF_TO_ARENA_OBJ(arena, east.err.arg, (size_t)east.err.arg_len));
+    PUT_C(ret, "error", DICT_OBJ(err_dict));
   }
   if (highlight) {
-    Array hl = (Array) {
-      .items = xmalloc(kv_size(colors) * sizeof(hl.items[0])),
-      .capacity = kv_size(colors),
-      .size = kv_size(colors),
-    };
+    Array hl = arena_array(arena, kv_size(colors));
     for (size_t i = 0; i < kv_size(colors); i++) {
       const ParserHighlightChunk chunk = kv_A(colors, i);
-      Array chunk_arr = (Array) {
-        .items = xmalloc(4 * sizeof(chunk_arr.items[0])),
-        .capacity = 4,
-        .size = 4,
-      };
-      chunk_arr.items[0] = INTEGER_OBJ((Integer)chunk.start.line);
-      chunk_arr.items[1] = INTEGER_OBJ((Integer)chunk.start.col);
-      chunk_arr.items[2] = INTEGER_OBJ((Integer)chunk.end_col);
-      chunk_arr.items[3] = STRING_OBJ(cstr_to_string(chunk.group));
-      hl.items[i] = ARRAY_OBJ(chunk_arr);
+      Array chunk_arr = arena_array(arena, 4);
+      ADD_C(chunk_arr, INTEGER_OBJ((Integer)chunk.start.line));
+      ADD_C(chunk_arr, INTEGER_OBJ((Integer)chunk.start.col));
+      ADD_C(chunk_arr, INTEGER_OBJ((Integer)chunk.end_col));
+      ADD_C(chunk_arr, CSTR_AS_OBJ(chunk.group));
+
+      ADD_C(hl, ARRAY_OBJ(chunk_arr));
     }
-    ret.items[ret.size++] = (KeyValuePair) {
-      .key = STATIC_CSTR_TO_STRING("highlight"),
-      .value = ARRAY_OBJ(hl),
-    };
+    PUT_C(ret, "highlight", ARRAY_OBJ(hl));
   }
   kvi_destroy(colors);
 
   // Walk over the AST, freeing nodes in process.
   ExprASTConvStack ast_conv_stack;
   kvi_init(ast_conv_stack);
+  Object ast = NIL;
   kvi_push(ast_conv_stack, ((ExprASTConvStackItem) {
     .node_p = &east.root,
-    .ret_node_p = &ret.items[0].value,
+    .ret_node_p = &ast,
   }));
   while (kv_size(ast_conv_stack)) {
     ExprASTConvStackItem cur_item = kv_last(ast_conv_stack);
@@ -518,7 +515,7 @@ Dictionary nvim_parse_expression(String expr, String flags, Boolean highlight, E
       kv_drop(ast_conv_stack, 1);
     } else {
       if (cur_item.ret_node_p->type == kObjectTypeNil) {
-        size_t items_size = (size_t)(3  // "type", "start" and "len"
+        size_t items_size = (size_t)(3  // "type", "start" and "len"  // NOLINT(bugprone-misplaced-widening-cast)
                                      + (node->children != NULL)  // "children"
                                      + (node->type == kExprNodeOption
                                         || node->type == kExprNodePlainIdentifier)  // "scope"
@@ -535,28 +532,17 @@ Dictionary nvim_parse_expression(String expr, String flags, Boolean highlight, E
                                         || node->type == kExprNodeSingleQuotedString)  // "svalue"
                                      + (node->type == kExprNodeAssignment)  // "augmentation"
                                      + 0);
-        Dictionary ret_node = {
-          .items = xmalloc(items_size * sizeof(ret_node.items[0])),
-          .capacity = items_size,
-          .size = 0,
-        };
-        *cur_item.ret_node_p = DICTIONARY_OBJ(ret_node);
+        Dict ret_node = arena_dict(arena, items_size);
+        *cur_item.ret_node_p = DICT_OBJ(ret_node);
       }
-      Dictionary *ret_node = &cur_item.ret_node_p->data.dictionary;
+      Dict *ret_node = &cur_item.ret_node_p->data.dict;
       if (node->children != NULL) {
         const size_t num_children = 1 + (node->children->next != NULL);
-        Array children_array = {
-          .items = xmalloc(num_children * sizeof(children_array.items[0])),
-          .capacity = num_children,
-          .size = num_children,
-        };
+        Array children_array = arena_array(arena, num_children);
         for (size_t i = 0; i < num_children; i++) {
-          children_array.items[i] = NIL;
+          ADD_C(children_array, NIL);
         }
-        ret_node->items[ret_node->size++] = (KeyValuePair) {
-          .key = STATIC_CSTR_TO_STRING("children"),
-          .value = ARRAY_OBJ(children_array),
-        };
+        PUT_C(*ret_node, "children", ARRAY_OBJ(children_array));
         kvi_push(ast_conv_stack, ((ExprASTConvStackItem) {
           .node_p = &node->children,
           .ret_node_p = &children_array.items[0],
@@ -568,126 +554,60 @@ Dictionary nvim_parse_expression(String expr, String flags, Boolean highlight, E
         }));
       } else {
         kv_drop(ast_conv_stack, 1);
-        ret_node->items[ret_node->size++] = (KeyValuePair) {
-          .key = STATIC_CSTR_TO_STRING("type"),
-          .value = STRING_OBJ(cstr_to_string(east_node_type_tab[node->type])),
-        };
-        Array start_array = {
-          .items = xmalloc(2 * sizeof(start_array.items[0])),
-          .capacity = 2,
-          .size = 2,
-        };
-        start_array.items[0] = INTEGER_OBJ((Integer)node->start.line);
-        start_array.items[1] = INTEGER_OBJ((Integer)node->start.col);
-        ret_node->items[ret_node->size++] = (KeyValuePair) {
-          .key = STATIC_CSTR_TO_STRING("start"),
-          .value = ARRAY_OBJ(start_array),
-        };
-        ret_node->items[ret_node->size++] = (KeyValuePair) {
-          .key = STATIC_CSTR_TO_STRING("len"),
-          .value = INTEGER_OBJ((Integer)node->len),
-        };
+        PUT_C(*ret_node, "type", CSTR_AS_OBJ(east_node_type_tab[node->type]));
+        Array start_array = arena_array(arena, 2);
+        ADD_C(start_array, INTEGER_OBJ((Integer)node->start.line));
+        ADD_C(start_array, INTEGER_OBJ((Integer)node->start.col));
+        PUT_C(*ret_node, "start", ARRAY_OBJ(start_array));
+
+        PUT_C(*ret_node, "len", INTEGER_OBJ((Integer)node->len));
         switch (node->type) {
         case kExprNodeDoubleQuotedString:
-        case kExprNodeSingleQuotedString:
-          ret_node->items[ret_node->size++] = (KeyValuePair) {
-            .key = STATIC_CSTR_TO_STRING("svalue"),
-            .value = STRING_OBJ(((String) {
-              .data = node->data.str.value,
-              .size = node->data.str.size,
-            })),
-          };
+        case kExprNodeSingleQuotedString: {
+          Object str = CBUF_TO_ARENA_OBJ(arena, node->data.str.value, node->data.str.size);
+          PUT_C(*ret_node, "svalue", str);
+          xfree(node->data.str.value);
           break;
+        }
         case kExprNodeOption:
-          ret_node->items[ret_node->size++] = (KeyValuePair) {
-            .key = STATIC_CSTR_TO_STRING("scope"),
-            .value = INTEGER_OBJ(node->data.opt.scope),
-          };
-          ret_node->items[ret_node->size++] = (KeyValuePair) {
-            .key = STATIC_CSTR_TO_STRING("ident"),
-            .value = STRING_OBJ(((String) {
-              .data = xmemdupz(node->data.opt.ident,
-                               node->data.opt.ident_len),
-              .size = node->data.opt.ident_len,
-            })),
-          };
+          PUT_C(*ret_node, "scope", INTEGER_OBJ(node->data.opt.scope));
+          PUT_C(*ret_node, "ident", CBUF_TO_ARENA_OBJ(arena, node->data.opt.ident,
+                                                      node->data.opt.ident_len));
           break;
         case kExprNodePlainIdentifier:
-          ret_node->items[ret_node->size++] = (KeyValuePair) {
-            .key = STATIC_CSTR_TO_STRING("scope"),
-            .value = INTEGER_OBJ(node->data.var.scope),
-          };
-          ret_node->items[ret_node->size++] = (KeyValuePair) {
-            .key = STATIC_CSTR_TO_STRING("ident"),
-            .value = STRING_OBJ(((String) {
-              .data = xmemdupz(node->data.var.ident,
-                               node->data.var.ident_len),
-              .size = node->data.var.ident_len,
-            })),
-          };
+          PUT_C(*ret_node, "scope", INTEGER_OBJ(node->data.var.scope));
+          PUT_C(*ret_node, "ident", CBUF_TO_ARENA_OBJ(arena, node->data.var.ident,
+                                                      node->data.var.ident_len));
           break;
         case kExprNodePlainKey:
-          ret_node->items[ret_node->size++] = (KeyValuePair) {
-            .key = STATIC_CSTR_TO_STRING("ident"),
-            .value = STRING_OBJ(((String) {
-              .data = xmemdupz(node->data.var.ident,
-                               node->data.var.ident_len),
-              .size = node->data.var.ident_len,
-            })),
-          };
+          PUT_C(*ret_node, "ident", CBUF_TO_ARENA_OBJ(arena, node->data.var.ident,
+                                                      node->data.var.ident_len));
           break;
         case kExprNodeEnvironment:
-          ret_node->items[ret_node->size++] = (KeyValuePair) {
-            .key = STATIC_CSTR_TO_STRING("ident"),
-            .value = STRING_OBJ(((String) {
-              .data = xmemdupz(node->data.env.ident,
-                               node->data.env.ident_len),
-              .size = node->data.env.ident_len,
-            })),
-          };
+          PUT_C(*ret_node, "ident", CBUF_TO_ARENA_OBJ(arena, node->data.env.ident,
+                                                      node->data.env.ident_len));
           break;
         case kExprNodeRegister:
-          ret_node->items[ret_node->size++] = (KeyValuePair) {
-            .key = STATIC_CSTR_TO_STRING("name"),
-            .value = INTEGER_OBJ(node->data.reg.name),
-          };
+          PUT_C(*ret_node, "name", INTEGER_OBJ(node->data.reg.name));
           break;
         case kExprNodeComparison:
-          ret_node->items[ret_node->size++] = (KeyValuePair) {
-            .key = STATIC_CSTR_TO_STRING("cmp_type"),
-            .value = STRING_OBJ(cstr_to_string(eltkn_cmp_type_tab[node->data.cmp.type])),
-          };
-          ret_node->items[ret_node->size++] = (KeyValuePair) {
-            .key = STATIC_CSTR_TO_STRING("ccs_strategy"),
-            .value = STRING_OBJ(cstr_to_string(ccs_tab[node->data.cmp.ccs])),
-          };
-          ret_node->items[ret_node->size++] = (KeyValuePair) {
-            .key = STATIC_CSTR_TO_STRING("invert"),
-            .value = BOOLEAN_OBJ(node->data.cmp.inv),
-          };
+          PUT_C(*ret_node, "cmp_type", CSTR_AS_OBJ(eltkn_cmp_type_tab[node->data.cmp.type]));
+          PUT_C(*ret_node, "ccs_strategy", CSTR_AS_OBJ(ccs_tab[node->data.cmp.ccs]));
+          PUT_C(*ret_node, "invert", BOOLEAN_OBJ(node->data.cmp.inv));
           break;
         case kExprNodeFloat:
-          ret_node->items[ret_node->size++] = (KeyValuePair) {
-            .key = STATIC_CSTR_TO_STRING("fvalue"),
-            .value = FLOAT_OBJ(node->data.flt.value),
-          };
+          PUT_C(*ret_node, "fvalue", FLOAT_OBJ(node->data.flt.value));
           break;
         case kExprNodeInteger:
-          ret_node->items[ret_node->size++] = (KeyValuePair) {
-            .key = STATIC_CSTR_TO_STRING("ivalue"),
-            .value = INTEGER_OBJ((Integer)(node->data.num.value > API_INTEGER_MAX
-                                           ? API_INTEGER_MAX
-                                           : (Integer)node->data.num.value)),
-          };
+          PUT_C(*ret_node, "ivalue", INTEGER_OBJ((Integer)(node->data.num.value > API_INTEGER_MAX
+                                                           ? API_INTEGER_MAX
+                                                           : (Integer)node->data.num.value)));
           break;
         case kExprNodeAssignment: {
           const ExprAssignmentType asgn_type = node->data.ass.type;
-          ret_node->items[ret_node->size++] = (KeyValuePair) {
-            .key = STATIC_CSTR_TO_STRING("augmentation"),
-            .value = STRING_OBJ(asgn_type == kExprAsgnPlain
-                                ? (String)STRING_INIT
-                                : cstr_to_string(expr_asgn_type_tab[asgn_type])),
-          };
+          String str = (asgn_type == kExprAsgnPlain
+                        ? (String)STRING_INIT : cstr_as_string(expr_asgn_type_tab[asgn_type]));
+          PUT_C(*ret_node, "augmentation", STRING_OBJ(str));
           break;
         }
         case kExprNodeMissing:
@@ -720,14 +640,14 @@ Dictionary nvim_parse_expression(String expr, String flags, Boolean highlight, E
         case kExprNodeMod:
           break;
         }
-        assert(cur_item.ret_node_p->data.dictionary.size
-               == cur_item.ret_node_p->data.dictionary.capacity);
+        assert(cur_item.ret_node_p->data.dict.size == cur_item.ret_node_p->data.dict.capacity);
         xfree(*cur_item.node_p);
         *cur_item.node_p = NULL;
       }
     }
   }
   kvi_destroy(ast_conv_stack);
+  PUT_C(ret, "ast", ast);
 
   assert(ret.size == ret.capacity);
   // Should be a no-op actually, leaving it in case non-nodes will need to be
