@@ -11,7 +11,9 @@
 #include "nvim/charset.h"
 #include "nvim/cmdexpand_defs.h"
 #include "nvim/cmdhist.h"
+#include "nvim/errors.h"
 #include "nvim/eval/typval.h"
+#include "nvim/eval/typval_defs.h"
 #include "nvim/ex_cmds.h"
 #include "nvim/ex_cmds_defs.h"
 #include "nvim/ex_getln.h"
@@ -189,7 +191,7 @@ static inline void hist_free_entry(histentry_T *hisptr)
   FUNC_ATTR_NONNULL_ALL
 {
   xfree(hisptr->hisstr);
-  tv_list_unref(hisptr->additional_elements);
+  xfree(hisptr->additional_data);
   clear_hist_entry(hisptr);
 }
 
@@ -220,7 +222,7 @@ static int in_history(int type, const char *str, int move_to_front, int sep)
     // well.
     char *p = history[type][i].hisstr;
     if (strcmp(str, p) == 0
-        && (type != HIST_SEARCH || sep == p[strlen(p) + 1])) {
+        && (type != HIST_SEARCH || sep == p[history[type][i].hisstrlen + 1])) {
       if (!move_to_front) {
         return true;
       }
@@ -236,8 +238,9 @@ static int in_history(int type, const char *str, int move_to_front, int sep)
     return false;
   }
 
-  list_T *const list = history[type][i].additional_elements;
+  AdditionalData *ad = history[type][i].additional_data;
   char *const save_hisstr = history[type][i].hisstr;
+  const size_t save_hisstrlen = history[type][i].hisstrlen;
   while (i != hisidx[type]) {
     if (++i >= hislen) {
       i = 0;
@@ -245,11 +248,12 @@ static int in_history(int type, const char *str, int move_to_front, int sep)
     history[type][last_i] = history[type][i];
     last_i = i;
   }
-  tv_list_unref(list);
+  xfree(ad);
   history[type][i].hisnum = ++hisnum[type];
   history[type][i].hisstr = save_hisstr;
+  history[type][i].hisstrlen = save_hisstrlen;
   history[type][i].timestamp = os_time();
-  history[type][i].additional_elements = NULL;
+  history[type][i].additional_data = NULL;
   return true;
 }
 
@@ -294,7 +298,7 @@ static int last_maptick = -1;           // last seen maptick
 /// @param histype  may be one of the HIST_ values.
 /// @param in_map   consider maptick when inside a mapping
 /// @param sep      separator character used (search hist)
-void add_to_history(int histype, const char *new_entry, int in_map, int sep)
+void add_to_history(int histype, const char *new_entry, size_t new_entrylen, bool in_map, int sep)
 {
   histentry_T *hisptr;
 
@@ -334,11 +338,11 @@ void add_to_history(int histype, const char *new_entry, int in_map, int sep)
   hist_free_entry(hisptr);
 
   // Store the separator after the NUL of the string.
-  size_t len = strlen(new_entry);
-  hisptr->hisstr = xstrnsave(new_entry, len + 2);
+  hisptr->hisstr = xstrnsave(new_entry, new_entrylen + 2);
   hisptr->timestamp = os_time();
-  hisptr->additional_elements = NULL;
-  hisptr->hisstr[len + 1] = (char)sep;
+  hisptr->additional_data = NULL;
+  hisptr->hisstr[new_entrylen + 1] = (char)sep;
+  hisptr->hisstrlen = new_entrylen;
 
   hisptr->hisnum = ++hisnum[histype];
   if (histype == HIST_SEARCH && in_map) {
@@ -400,19 +404,6 @@ static int calc_hist_idx(int histype, int num)
   return -1;
 }
 
-/// Get a history entry by its index.
-///
-/// @param histype  may be one of the HIST_ values.
-static char *get_history_entry(int histype, int idx)
-{
-  idx = calc_hist_idx(histype, idx);
-  if (idx >= 0) {
-    return history[histype][idx].hisstr;
-  } else {
-    return "";
-  }
-}
-
 /// Clear all entries in a history
 ///
 /// @param[in]  histype  One of the HIST_ values.
@@ -536,7 +527,7 @@ void f_histadd(typval_T *argvars, typval_T *rettv, EvalFuncData fptr)
   }
 
   init_history();
-  add_to_history(histype, str, false, NUL);
+  add_to_history(histype, str, strlen(str), false, NUL);
   rettv->vval.v_number = true;
 }
 
@@ -575,10 +566,15 @@ void f_histget(typval_T *argvars, typval_T *rettv, EvalFuncData fptr)
     if (argvars[1].v_type == VAR_UNKNOWN) {
       idx = get_history_idx(type);
     } else {
-      idx = (int)tv_get_number_chk(&argvars[1], NULL);
+      idx = (int)tv_get_number_chk(&argvars[1], NULL);  // -1 on type error
+    }
+    idx = calc_hist_idx(type, idx);
+    if (idx < 0) {
+      rettv->vval.v_string = xstrnsave("", 0);
+    } else {
+      rettv->vval.v_string = xstrnsave(history[type][idx].hisstr,
+                                       history[type][idx].hisstrlen);
     }
-    // -1 on type error
-    rettv->vval.v_string = xstrdup(get_history_entry(type, idx));
   }
   rettv->v_type = VAR_STRING;
 }
@@ -591,9 +587,10 @@ void f_histnr(typval_T *argvars, typval_T *rettv, EvalFuncData fptr)
                   ? HIST_INVALID
                   : get_histtype(histname, strlen(histname), false);
   if (i != HIST_INVALID) {
-    i = get_history_idx(i);
+    rettv->vval.v_number = get_history_idx(i);
+  } else {
+    rettv->vval.v_number = HIST_INVALID;
   }
-  rettv->vval.v_number = i;
 }
 
 /// :history command - print a history
@@ -642,10 +639,8 @@ void ex_history(exarg_T *eap)
   }
 
   for (; !got_int && histype1 <= histype2; histype1++) {
-    xstrlcpy(IObuff, "\n      #  ", IOSIZE);
     assert(history_names[histype1] != NULL);
-    xstrlcat(IObuff, history_names[histype1], IOSIZE);
-    xstrlcat(IObuff, " history", IOSIZE);
+    vim_snprintf(IObuff, IOSIZE, "\n      #  %s history", history_names[histype1]);
     msg_puts_title(IObuff);
     int idx = hisidx[histype1];
     histentry_T *hist = history[histype1];
@@ -663,17 +658,17 @@ void ex_history(exarg_T *eap)
           i = 0;
         }
         if (hist[i].hisstr != NULL
-            && hist[i].hisnum >= j && hist[i].hisnum <= k) {
+            && hist[i].hisnum >= j && hist[i].hisnum <= k
+            && !message_filtered(hist[i].hisstr)) {
           msg_putchar('\n');
-          snprintf(IObuff, IOSIZE, "%c%6d  ", i == idx ? '>' : ' ',
-                   hist[i].hisnum);
+          int len = snprintf(IObuff, IOSIZE,
+                             "%c%6d  ", i == idx ? '>' : ' ', hist[i].hisnum);
           if (vim_strsize(hist[i].hisstr) > Columns - 10) {
-            trunc_string(hist[i].hisstr, IObuff + strlen(IObuff),
-                         Columns - 10, IOSIZE - (int)strlen(IObuff));
+            trunc_string(hist[i].hisstr, IObuff + len, Columns - 10, IOSIZE - len);
           } else {
-            xstrlcat(IObuff, hist[i].hisstr, IOSIZE);
+            xstrlcpy(IObuff + len, hist[i].hisstr, (size_t)(IOSIZE - len));
           }
-          msg_outtrans(IObuff, 0);
+          msg_outtrans(IObuff, 0, false);
         }
         if (i == idx) {
           break;
