@@ -1,31 +1,24 @@
 #pragma once
 
-#include <msgpack/sbuffer.h>
-#include <stddef.h>
+#include <stddef.h>  // IWYU pragma: keep
 
 #include "klib/kvec.h"
 #include "nvim/api/private/defs.h"
 
 typedef struct {
-  msgpack_sbuffer regs;     ///< Registers.
-  msgpack_sbuffer jumps;    ///< Jumplist.
-  msgpack_sbuffer bufs;     ///< Buffer list.
-  msgpack_sbuffer gvars;    ///< Global variables.
+  String regs;     ///< Registers.
+  String jumps;    ///< Jumplist.
+  String bufs;     ///< Buffer list.
+  String gvars;    ///< Global variables.
   Array funcs;              ///< Functions.
 } Context;
 typedef kvec_t(Context) ContextVec;
 
-#define MSGPACK_SBUFFER_INIT (msgpack_sbuffer) { \
-  .size = 0, \
-  .data = NULL, \
-  .alloc = 0, \
-}
-
 #define CONTEXT_INIT (Context) { \
-  .regs = MSGPACK_SBUFFER_INIT, \
-  .jumps = MSGPACK_SBUFFER_INIT, \
-  .bufs = MSGPACK_SBUFFER_INIT, \
-  .gvars = MSGPACK_SBUFFER_INIT, \
+  .regs = STRING_INIT, \
+  .jumps = STRING_INIT, \
+  .bufs = STRING_INIT, \
+  .gvars = STRING_INIT, \
   .funcs = ARRAY_DICT_INIT, \
 }
 
