@@ -1,23 +1,23 @@
-#ifndef NVIM_EVAL_FUNCS_H
-#define NVIM_EVAL_FUNCS_H
+#pragma once
 
 #include <stdbool.h>
 #include <stdint.h>
 
-#include "nvim/api/private/dispatch.h"
-#include "nvim/buffer_defs.h"
-#include "nvim/eval/typval.h"
+#include "nvim/cmdexpand_defs.h"  // IWYU pragma: keep
 #include "nvim/eval/typval_defs.h"
-#include "nvim/types.h"
+#include "nvim/pos_defs.h"  // IWYU pragma: keep
+#include "nvim/types_defs.h"
 
-/// Prototype of C function that implements VimL function
+/// Prototype of C function that implements Vimscript function
 typedef void (*VimLFunc)(typval_T *args, typval_T *rvar, EvalFuncData data);
 
 /// Special flags for base_arg @see EvalFuncDef
-#define BASE_NONE 0          ///< Not a method (no base argument).
-#define BASE_LAST UINT8_MAX  ///< Use the last argument as the method base.
+enum {
+  BASE_NONE = 0,          ///< Not a method (no base argument).
+  BASE_LAST = UINT8_MAX,  ///< Use the last argument as the method base.
+};
 
-/// Structure holding VimL function definition
+/// Structure holding Vimscript function definition
 typedef struct {
   char *name;         ///< Name of the function.
   uint8_t min_argc;   ///< Minimal number of arguments.
@@ -31,4 +31,3 @@ typedef struct {
 #ifdef INCLUDE_GENERATED_DECLARATIONS
 # include "eval/funcs.h.generated.h"
 #endif
-#endif  // NVIM_EVAL_FUNCS_H
