@@ -109,7 +109,7 @@ typedef enum {
   VAR_STRING,       ///< String, .v_string is used.
   VAR_FUNC,         ///< Function reference, .v_string is used as function name.
   VAR_LIST,         ///< List, .v_list is used.
-  VAR_DICT,         ///< Dictionary, .v_dict is used.
+  VAR_DICT,         ///< Dict, .v_dict is used.
   VAR_FLOAT,        ///< Floating-point value, .v_float is used.
   VAR_BOOL,         ///< true, false
   VAR_SPECIAL,      ///< Special value (null), .v_special is used.
@@ -141,7 +141,7 @@ typedef struct {
     float_T v_float;            ///< Floating-point number, for VAR_FLOAT.
     char *v_string;             ///< String, for VAR_STRING and VAR_FUNC, can be NULL.
     list_T *v_list;             ///< List for VAR_LIST, can be NULL.
-    dict_T *v_dict;             ///< Dictionary for VAR_DICT, can be NULL.
+    dict_T *v_dict;             ///< Dict for VAR_DICT, can be NULL.
     partial_T *v_partial;       ///< Closure: function with args.
     blob_T *v_blob;             ///< Blob for VAR_BLOB, can be NULL.
   } vval;                       ///< Actual value.
@@ -259,7 +259,7 @@ struct dictvar_S {
   dict_T *dv_copydict;    ///< Copied dict used by deepcopy().
   dict_T *dv_used_next;   ///< Next dictionary in used dictionaries list.
   dict_T *dv_used_prev;   ///< Previous dictionary in used dictionaries list.
-  QUEUE watchers;         ///< Dictionary key watchers set by user code.
+  QUEUE watchers;         ///< Dict key watchers set by user code.
 
   LuaRef lua_table_ref;
 };
@@ -286,14 +286,9 @@ typedef struct {
   scid_T sc_sid;     ///< script ID
   int sc_seq;        ///< sourcing sequence number
   linenr_T sc_lnum;  ///< line number
+  uint64_t sc_chan;  ///< Only used when sc_sid is SID_API_CLIENT.
 } sctx_T;
 
-/// Stores an identifier of a script or channel that last set an option.
-typedef struct {
-  sctx_T script_ctx;       /// script context where the option was last set
-  uint64_t channel_id;     /// Only used when script_id is SID_API_CLIENT.
-} LastSet;
-
 enum { MAX_FUNC_ARGS = 20, };  ///< Maximum number of function arguments
 enum { VAR_SHORT_LEN = 20, };  ///< Short variable name length
 enum { FIXVAR_CNT = 12, };     ///< Number of fixed variables used for arguments
@@ -357,13 +352,15 @@ struct ufunc {
   funccall_T *uf_scoped;       ///< l: local variables for closure
   char *uf_name_exp;    ///< if "uf_name[]" starts with SNR the name with
                         ///< "<SNR>" as a string, otherwise NULL
-  char uf_name[];    ///< Name of function (actual size equals name);
-                     ///< can start with <SNR>123_
-                     ///< (<SNR> is K_SPECIAL KS_EXTRA KE_SNR)
+  size_t uf_namelen;    ///< Length of uf_name (excluding the NUL)
+  char uf_name[];       ///< Name of function (actual size equals name);
+                        ///< can start with <SNR>123_
+                        ///< (<SNR> is K_SPECIAL KS_EXTRA KE_SNR)
 };
 
 struct partial_S {
   int pt_refcount;    ///< Reference count.
+  int pt_copyID;
   char *pt_name;      ///< Function name; when NULL use pt_func->name.
   ufunc_T *pt_func;   ///< Function pointer; when NULL lookup function with pt_name.
   bool pt_auto;       ///< When true the partial was created by using dict.member
