@@ -1,23 +1,23 @@
-#ifndef NVIM_MEMLINE_DEFS_H
-#define NVIM_MEMLINE_DEFS_H
+#pragma once
 
 #include "nvim/memfile_defs.h"
+#include "nvim/pos_defs.h"
 
 ///
 /// When searching for a specific line, we remember what blocks in the tree
 /// are the branches leading to that block. This is stored in ml_stack.  Each
 /// entry is a pointer to info in a block (may be data block or pointer block)
 ///
-typedef struct info_pointer {
+typedef struct {
   blocknr_T ip_bnum;            // block number
   linenr_T ip_low;              // lowest lnum in this block
   linenr_T ip_high;             // highest lnum in this block
   int ip_index;                 // index for block with current lnum
 } infoptr_T;    // block/index pair
 
-typedef struct ml_chunksize {
+typedef struct {
   int mlcs_numlines;
-  long mlcs_totalsize;
+  int mlcs_totalsize;
 } chunksize_T;
 
 // Flags when calling ml_updatechunk()
@@ -40,7 +40,7 @@ typedef struct ml_chunksize {
 ///             memory.  With this structure it is roughly (N * 128) pointer
 ///             moves, where N is the height (typically 1-3).
 ///
-typedef struct memline {
+typedef struct {
   linenr_T ml_line_count;       // number of lines in the buffer
 
   memfile_T *ml_mfp;          // pointer to associated memfile
@@ -56,6 +56,7 @@ typedef struct memline {
 #define ML_ALLOCATED    0x10    // ml_line_ptr is an allocated copy
   int ml_flags;
 
+  colnr_T ml_line_len;          // length of the cached line + NUL
   linenr_T ml_line_lnum;        // line number of cached line, 0 if not valid
   char *ml_line_ptr;            // pointer to cached line
   size_t ml_line_offset;        // cached byte offset of ml_line_lnum
@@ -69,5 +70,3 @@ typedef struct memline {
   int ml_numchunks;
   int ml_usedchunks;
 } memline_T;
-
-#endif  // NVIM_MEMLINE_DEFS_H
