@@ -1,11 +1,11 @@
 #pragma once
 
-#include <msgpack.h>
 #include <stdbool.h>
 #include <uv.h>
 
 #include "nvim/api/private/dispatch.h"
 #include "nvim/map_defs.h"
+#include "nvim/ui_defs.h"
 
 typedef struct Channel Channel;
 typedef struct Unpacker Unpacker;
@@ -37,11 +37,11 @@ typedef struct {
 } RequestEvent;
 
 typedef struct {
-  Set(cstr_t) subscribed_events[1];
   bool closed;
   Unpacker *unpacker;
+  RemoteUI *ui;
   uint32_t next_request_id;
   kvec_t(ChannelCallFrame *) call_stack;
-  Dictionary info;
+  Dict info;
   ClientType client_type;
 } RpcState;
