@@ -3,7 +3,6 @@
 #include <stdbool.h>
 #include <stdint.h>
 #include <stdio.h>
-#include <string.h>
 
 #include "nvim/ascii_defs.h"
 #include "nvim/buffer_defs.h"
@@ -20,6 +19,7 @@
 #include "nvim/memline.h"
 #include "nvim/memory.h"
 #include "nvim/move.h"
+#include "nvim/normal.h"
 #include "nvim/option_vars.h"
 #include "nvim/pos_defs.h"
 #include "nvim/search.h"
@@ -428,6 +428,13 @@ int end_word(int count, bool bigword, bool stop, bool empty)
 
   curwin->w_cursor.coladd = 0;
   cls_bigword = bigword;
+
+  // If adjusted cursor position previously, unadjust it.
+  if (*p_sel == 'e' && VIsual_active && VIsual_mode == 'v'
+      && VIsual_select_exclu_adj) {
+    unadjust_for_sel();
+  }
+
   while (--count >= 0) {
     // When inside a range of folded lines, move to the last char of the
     // last line.
@@ -1193,13 +1200,18 @@ again:
   pos_T end_pos = curwin->w_cursor;
 
   if (!do_include) {
-    // Exclude the start tag.
+    // Exclude the start tag,
+    // but skip over '>' if it appears in quotes
+    bool in_quotes = false;
     curwin->w_cursor = start_pos;
     while (inc_cursor() >= 0) {
-      if (*get_cursor_pos_ptr() == '>') {
+      p = get_cursor_pos_ptr();
+      if (*p == '>' && !in_quotes) {
         inc_cursor();
         start_pos = curwin->w_cursor;
         break;
+      } else if (*p == '"' || *p == '\'') {
+        in_quotes = !in_quotes;
       }
     }
     curwin->w_cursor = end_pos;
@@ -1264,11 +1276,7 @@ int current_par(oparg_T *oap, int count, bool include, int type)
   // When visual area is more than one line: extend it.
   if (VIsual_active && start_lnum != VIsual.lnum) {
 extend:
-    if (start_lnum < VIsual.lnum) {
-      dir = BACKWARD;
-    } else {
-      dir = FORWARD;
-    }
+    dir = start_lnum < VIsual.lnum ? BACKWARD : FORWARD;
     for (int i = count; --i >= 0;) {
       if (start_lnum ==
           (dir == BACKWARD ? 1 : curbuf->b_ml.ml_line_count)) {
