@@ -37,7 +37,6 @@ describe('mappings with <Cmd>', function()
       [9] = { background = Screen.colors.LightMagenta },
       [10] = { foreground = Screen.colors.Red },
     })
-    screen:attach()
 
     cmdmap('<F3>', 'let m = mode(1)')
     cmdmap('<F4>', 'normal! ww')
@@ -575,7 +574,7 @@ describe('mappings with <Cmd>', function()
     eq('i', eval('mode(1)'))
 
     -- also works as part of abbreviation
-    feed('<space>foo ')
+    feed(' foo ')
     screen:expect([[
       in bar ^deed some short little lines                              |
       of stuff test text                                               |
