@@ -1,10 +1,10 @@
 -- Some tests for buffer-local autocommands
 
-local helpers = require('test.functional.helpers')(after_each)
+local n = require('test.functional.testnvim')()
 
-local clear = helpers.clear
-local expect = helpers.expect
-local command = helpers.command
+local clear = n.clear
+local expect = n.expect
+local command = n.command
 
 local fname = 'Xtest-functional-legacy-054'
 
