@@ -17,7 +17,6 @@ describe('messages', function()
   -- oldtest: Test_warning_scroll()
   it('a warning causes scrolling if and only if it has a stacktrace', function()
     screen = Screen.new(75, 6)
-    screen:attach()
 
     -- When the warning comes from a script, messages are scrolled so that the
     -- stacktrace is visible.
@@ -45,7 +44,6 @@ describe('messages', function()
   -- oldtest: Test_message_not_cleared_after_mode()
   it('clearing mode does not remove message', function()
     screen = Screen.new(60, 10)
-    screen:attach()
     exec([[
       nmap <silent> gx :call DebugSilent('normal')<CR>
       vmap <silent> gx :call DebugSilent('visual')<CR>
@@ -84,11 +82,38 @@ describe('messages', function()
       NoSuchFil^e                                                  |
       three                                                       |
       {1:~                                                           }|*5
-      from DebugSilent visual                                     |
+                                                                  |
       {9:E447: Can't find file "NoSuchFile" in path}                  |
     ]])
   end)
 
+  -- oldtest: Test_mode_cleared_after_silent_message()
+  it('mode is cleared properly after silent message', function()
+    screen = Screen.new(60, 10)
+    exec([[
+      edit XsilentMessageMode.txt
+      call setline(1, 'foobar')
+      autocmd TextChanged * silent update
+    ]])
+    finally(function()
+      os.remove('XsilentMessageMode.txt')
+    end)
+
+    feed('v')
+    screen:expect([[
+      ^foobar                                                      |
+      {1:~                                                           }|*8
+      {5:-- VISUAL --}                                                |
+    ]])
+
+    feed('d')
+    screen:expect([[
+      ^oobar                                                       |
+      {1:~                                                           }|*8
+                                                                  |
+    ]])
+  end)
+
   describe('more prompt', function()
     before_each(function()
       command('set more')
@@ -97,11 +122,6 @@ describe('messages', function()
     -- oldtest: Test_message_more()
     it('works', function()
       screen = Screen.new(75, 6)
-      screen:set_default_attr_ids({
-        [1] = { bold = true, foreground = Screen.colors.SeaGreen }, -- MoreMsg
-        [2] = { foreground = Screen.colors.Brown }, -- LineNr
-      })
-      screen:attach()
 
       command('call setline(1, range(1, 100))')
 
@@ -116,194 +136,194 @@ describe('messages', function()
       ]])
       feed('\n')
       screen:expect([[
-        {2:  1 }1                                                                      |
-        {2:  2 }2                                                                      |
-        {2:  3 }3                                                                      |
-        {2:  4 }4                                                                      |
-        {2:  5 }5                                                                      |
-        {1:-- More --}^                                                                 |
+        {8:  1 }1                                                                      |
+        {8:  2 }2                                                                      |
+        {8:  3 }3                                                                      |
+        {8:  4 }4                                                                      |
+        {8:  5 }5                                                                      |
+        {6:-- More --}^                                                                 |
       ]])
 
       feed('?')
       screen:expect([[
-        {2:  1 }1                                                                      |
-        {2:  2 }2                                                                      |
-        {2:  3 }3                                                                      |
-        {2:  4 }4                                                                      |
-        {2:  5 }5                                                                      |
-        {1:-- More -- SPACE/d/j: screen/page/line down, b/u/k: up, q: quit }^           |
+        {8:  1 }1                                                                      |
+        {8:  2 }2                                                                      |
+        {8:  3 }3                                                                      |
+        {8:  4 }4                                                                      |
+        {8:  5 }5                                                                      |
+        {6:-- More -- SPACE/d/j: screen/page/line down, b/u/k: up, q: quit }^           |
       ]])
 
       -- Down a line with j, <CR>, <NL> or <Down>.
       feed('j')
       screen:expect([[
-        {2:  2 }2                                                                      |
-        {2:  3 }3                                                                      |
-        {2:  4 }4                                                                      |
-        {2:  5 }5                                                                      |
-        {2:  6 }6                                                                      |
-        {1:-- More --}^                                                                 |
+        {8:  2 }2                                                                      |
+        {8:  3 }3                                                                      |
+        {8:  4 }4                                                                      |
+        {8:  5 }5                                                                      |
+        {8:  6 }6                                                                      |
+        {6:-- More --}^                                                                 |
       ]])
       feed('<NL>')
       screen:expect([[
-        {2:  3 }3                                                                      |
-        {2:  4 }4                                                                      |
-        {2:  5 }5                                                                      |
-        {2:  6 }6                                                                      |
-        {2:  7 }7                                                                      |
-        {1:-- More --}^                                                                 |
+        {8:  3 }3                                                                      |
+        {8:  4 }4                                                                      |
+        {8:  5 }5                                                                      |
+        {8:  6 }6                                                                      |
+        {8:  7 }7                                                                      |
+        {6:-- More --}^                                                                 |
       ]])
       feed('<CR>')
       screen:expect([[
-        {2:  4 }4                                                                      |
-        {2:  5 }5                                                                      |
-        {2:  6 }6                                                                      |
-        {2:  7 }7                                                                      |
-        {2:  8 }8                                                                      |
-        {1:-- More --}^                                                                 |
+        {8:  4 }4                                                                      |
+        {8:  5 }5                                                                      |
+        {8:  6 }6                                                                      |
+        {8:  7 }7                                                                      |
+        {8:  8 }8                                                                      |
+        {6:-- More --}^                                                                 |
       ]])
       feed('<Down>')
       screen:expect([[
-        {2:  5 }5                                                                      |
-        {2:  6 }6                                                                      |
-        {2:  7 }7                                                                      |
-        {2:  8 }8                                                                      |
-        {2:  9 }9                                                                      |
-        {1:-- More --}^                                                                 |
+        {8:  5 }5                                                                      |
+        {8:  6 }6                                                                      |
+        {8:  7 }7                                                                      |
+        {8:  8 }8                                                                      |
+        {8:  9 }9                                                                      |
+        {6:-- More --}^                                                                 |
       ]])
 
       -- Down a screen with <Space>, f, or <PageDown>.
       feed('f')
       screen:expect([[
-        {2: 10 }10                                                                     |
-        {2: 11 }11                                                                     |
-        {2: 12 }12                                                                     |
-        {2: 13 }13                                                                     |
-        {2: 14 }14                                                                     |
-        {1:-- More --}^                                                                 |
+        {8: 10 }10                                                                     |
+        {8: 11 }11                                                                     |
+        {8: 12 }12                                                                     |
+        {8: 13 }13                                                                     |
+        {8: 14 }14                                                                     |
+        {6:-- More --}^                                                                 |
       ]])
       feed('<Space>')
       screen:expect([[
-        {2: 15 }15                                                                     |
-        {2: 16 }16                                                                     |
-        {2: 17 }17                                                                     |
-        {2: 18 }18                                                                     |
-        {2: 19 }19                                                                     |
-        {1:-- More --}^                                                                 |
+        {8: 15 }15                                                                     |
+        {8: 16 }16                                                                     |
+        {8: 17 }17                                                                     |
+        {8: 18 }18                                                                     |
+        {8: 19 }19                                                                     |
+        {6:-- More --}^                                                                 |
       ]])
       feed('<PageDown>')
       screen:expect([[
-        {2: 20 }20                                                                     |
-        {2: 21 }21                                                                     |
-        {2: 22 }22                                                                     |
-        {2: 23 }23                                                                     |
-        {2: 24 }24                                                                     |
-        {1:-- More --}^                                                                 |
+        {8: 20 }20                                                                     |
+        {8: 21 }21                                                                     |
+        {8: 22 }22                                                                     |
+        {8: 23 }23                                                                     |
+        {8: 24 }24                                                                     |
+        {6:-- More --}^                                                                 |
       ]])
 
       -- Down a page (half a screen) with d.
       feed('d')
       screen:expect([[
-        {2: 23 }23                                                                     |
-        {2: 24 }24                                                                     |
-        {2: 25 }25                                                                     |
-        {2: 26 }26                                                                     |
-        {2: 27 }27                                                                     |
-        {1:-- More --}^                                                                 |
+        {8: 23 }23                                                                     |
+        {8: 24 }24                                                                     |
+        {8: 25 }25                                                                     |
+        {8: 26 }26                                                                     |
+        {8: 27 }27                                                                     |
+        {6:-- More --}^                                                                 |
       ]])
 
       -- Down all the way with 'G'.
       feed('G')
       screen:expect([[
-        {2: 96 }96                                                                     |
-        {2: 97 }97                                                                     |
-        {2: 98 }98                                                                     |
-        {2: 99 }99                                                                     |
-        {2:100 }100                                                                    |
-        {1:Press ENTER or type command to continue}^                                    |
+        {8: 96 }96                                                                     |
+        {8: 97 }97                                                                     |
+        {8: 98 }98                                                                     |
+        {8: 99 }99                                                                     |
+        {8:100 }100                                                                    |
+        {6:Press ENTER or type command to continue}^                                    |
       ]])
 
       -- Up a line k, <BS> or <Up>.
       feed('k')
       screen:expect([[
-        {2: 95 }95                                                                     |
-        {2: 96 }96                                                                     |
-        {2: 97 }97                                                                     |
-        {2: 98 }98                                                                     |
-        {2: 99 }99                                                                     |
-        {1:-- More --}^                                                                 |
+        {8: 95 }95                                                                     |
+        {8: 96 }96                                                                     |
+        {8: 97 }97                                                                     |
+        {8: 98 }98                                                                     |
+        {8: 99 }99                                                                     |
+        {6:-- More --}^                                                                 |
       ]])
       feed('<BS>')
       screen:expect([[
-        {2: 94 }94                                                                     |
-        {2: 95 }95                                                                     |
-        {2: 96 }96                                                                     |
-        {2: 97 }97                                                                     |
-        {2: 98 }98                                                                     |
-        {1:-- More --}^                                                                 |
+        {8: 94 }94                                                                     |
+        {8: 95 }95                                                                     |
+        {8: 96 }96                                                                     |
+        {8: 97 }97                                                                     |
+        {8: 98 }98                                                                     |
+        {6:-- More --}^                                                                 |
       ]])
       feed('<Up>')
       screen:expect([[
-        {2: 93 }93                                                                     |
-        {2: 94 }94                                                                     |
-        {2: 95 }95                                                                     |
-        {2: 96 }96                                                                     |
-        {2: 97 }97                                                                     |
-        {1:-- More --}^                                                                 |
+        {8: 93 }93                                                                     |
+        {8: 94 }94                                                                     |
+        {8: 95 }95                                                                     |
+        {8: 96 }96                                                                     |
+        {8: 97 }97                                                                     |
+        {6:-- More --}^                                                                 |
       ]])
 
       -- Up a screen with b or <PageUp>.
       feed('b')
       screen:expect([[
-        {2: 88 }88                                                                     |
-        {2: 89 }89                                                                     |
-        {2: 90 }90                                                                     |
-        {2: 91 }91                                                                     |
-        {2: 92 }92                                                                     |
-        {1:-- More --}^                                                                 |
+        {8: 88 }88                                                                     |
+        {8: 89 }89                                                                     |
+        {8: 90 }90                                                                     |
+        {8: 91 }91                                                                     |
+        {8: 92 }92                                                                     |
+        {6:-- More --}^                                                                 |
       ]])
       feed('<PageUp>')
       screen:expect([[
-        {2: 83 }83                                                                     |
-        {2: 84 }84                                                                     |
-        {2: 85 }85                                                                     |
-        {2: 86 }86                                                                     |
-        {2: 87 }87                                                                     |
-        {1:-- More --}^                                                                 |
+        {8: 83 }83                                                                     |
+        {8: 84 }84                                                                     |
+        {8: 85 }85                                                                     |
+        {8: 86 }86                                                                     |
+        {8: 87 }87                                                                     |
+        {6:-- More --}^                                                                 |
       ]])
 
       -- Up a page (half a screen) with u.
       feed('u')
       screen:expect([[
-        {2: 80 }80                                                                     |
-        {2: 81 }81                                                                     |
-        {2: 82 }82                                                                     |
-        {2: 83 }83                                                                     |
-        {2: 84 }84                                                                     |
-        {1:-- More --}^                                                                 |
+        {8: 80 }80                                                                     |
+        {8: 81 }81                                                                     |
+        {8: 82 }82                                                                     |
+        {8: 83 }83                                                                     |
+        {8: 84 }84                                                                     |
+        {6:-- More --}^                                                                 |
       ]])
 
       -- Up all the way with 'g'.
       feed('g')
       screen:expect([[
         :%p#                                                                       |
-        {2:  1 }1                                                                      |
-        {2:  2 }2                                                                      |
-        {2:  3 }3                                                                      |
-        {2:  4 }4                                                                      |
-        {1:-- More --}^                                                                 |
+        {8:  1 }1                                                                      |
+        {8:  2 }2                                                                      |
+        {8:  3 }3                                                                      |
+        {8:  4 }4                                                                      |
+        {6:-- More --}^                                                                 |
       ]])
 
       -- All the way down. Pressing f should do nothing but pressing
       -- space should end the more prompt.
       feed('G')
       screen:expect([[
-        {2: 96 }96                                                                     |
-        {2: 97 }97                                                                     |
-        {2: 98 }98                                                                     |
-        {2: 99 }99                                                                     |
-        {2:100 }100                                                                    |
-        {1:Press ENTER or type command to continue}^                                    |
+        {8: 96 }96                                                                     |
+        {8: 97 }97                                                                     |
+        {8: 98 }98                                                                     |
+        {8: 99 }99                                                                     |
+        {8:100 }100                                                                    |
+        {6:Press ENTER or type command to continue}^                                    |
       ]])
       feed('f')
       screen:expect_unchanged()
@@ -320,34 +340,34 @@ describe('messages', function()
       -- Pressing g< shows the previous command output.
       feed('g<lt>')
       screen:expect([[
-        {2: 96 }96                                                                     |
-        {2: 97 }97                                                                     |
-        {2: 98 }98                                                                     |
-        {2: 99 }99                                                                     |
-        {2:100 }100                                                                    |
-        {1:Press ENTER or type command to continue}^                                    |
+        {8: 96 }96                                                                     |
+        {8: 97 }97                                                                     |
+        {8: 98 }98                                                                     |
+        {8: 99 }99                                                                     |
+        {8:100 }100                                                                    |
+        {6:Press ENTER or type command to continue}^                                    |
       ]])
 
       -- A command line that doesn't print text is appended to scrollback,
       -- even if it invokes a nested command line.
       feed([[:<C-R>=':'<CR>:<CR>g<lt>]])
       screen:expect([[
-        {2: 97 }97                                                                     |
-        {2: 98 }98                                                                     |
-        {2: 99 }99                                                                     |
-        {2:100 }100                                                                    |
+        {8: 97 }97                                                                     |
+        {8: 98 }98                                                                     |
+        {8: 99 }99                                                                     |
+        {8:100 }100                                                                    |
         :::                                                                        |
-        {1:Press ENTER or type command to continue}^                                    |
+        {6:Press ENTER or type command to continue}^                                    |
       ]])
 
       feed(':%p#\n')
       screen:expect([[
-        {2:  1 }1                                                                      |
-        {2:  2 }2                                                                      |
-        {2:  3 }3                                                                      |
-        {2:  4 }4                                                                      |
-        {2:  5 }5                                                                      |
-        {1:-- More --}^                                                                 |
+        {8:  1 }1                                                                      |
+        {8:  2 }2                                                                      |
+        {8:  3 }3                                                                      |
+        {8:  4 }4                                                                      |
+        {8:  5 }5                                                                      |
+        {6:-- More --}^                                                                 |
       ]])
 
       -- Stop command output with q, <Esc> or CTRL-C.
@@ -364,37 +384,36 @@ describe('messages', function()
       -- Execute a : command from the more prompt
       feed(':%p#\n')
       screen:expect([[
-        {2:  1 }1                                                                      |
-        {2:  2 }2                                                                      |
-        {2:  3 }3                                                                      |
-        {2:  4 }4                                                                      |
-        {2:  5 }5                                                                      |
-        {1:-- More --}^                                                                 |
+        {8:  1 }1                                                                      |
+        {8:  2 }2                                                                      |
+        {8:  3 }3                                                                      |
+        {8:  4 }4                                                                      |
+        {8:  5 }5                                                                      |
+        {6:-- More --}^                                                                 |
       ]])
       feed(':')
       screen:expect([[
-        {2:  1 }1                                                                      |
-        {2:  2 }2                                                                      |
-        {2:  3 }3                                                                      |
-        {2:  4 }4                                                                      |
-        {2:  5 }5                                                                      |
+        {8:  1 }1                                                                      |
+        {8:  2 }2                                                                      |
+        {8:  3 }3                                                                      |
+        {8:  4 }4                                                                      |
+        {8:  5 }5                                                                      |
         :^                                                                          |
       ]])
       feed("echo 'Hello'\n")
       screen:expect([[
-        {2:  2 }2                                                                      |
-        {2:  3 }3                                                                      |
-        {2:  4 }4                                                                      |
-        {2:  5 }5                                                                      |
+        {8:  2 }2                                                                      |
+        {8:  3 }3                                                                      |
+        {8:  4 }4                                                                      |
+        {8:  5 }5                                                                      |
         Hello                                                                      |
-        {1:Press ENTER or type command to continue}^                                    |
+        {6:Press ENTER or type command to continue}^                                    |
       ]])
     end)
 
     -- oldtest: Test_echo_verbose_system()
     it('verbose message before echo command', function()
       screen = Screen.new(60, 10)
-      screen:attach()
 
       command('cd ' .. nvim_dir)
       api.nvim_set_option_value('shell', './shell-test', {})
@@ -494,7 +513,6 @@ describe('messages', function()
     -- oldtest: Test_quit_long_message()
     it('with control characters can be quit vim-patch:8.2.1844', function()
       screen = Screen.new(40, 10)
-      screen:attach()
 
       feed([[:echom range(9999)->join("\x01")<CR>]])
       screen:expect([[
@@ -521,7 +539,6 @@ describe('messages', function()
   describe('mode is cleared when', function()
     before_each(function()
       screen = Screen.new(40, 6)
-      screen:attach()
     end)
 
     -- oldtest: Test_mode_message_at_leaving_insert_by_ctrl_c()
@@ -591,7 +608,6 @@ describe('messages', function()
   -- oldtest: Test_ask_yesno()
   it('y/n prompt works', function()
     screen = Screen.new(75, 6)
-    screen:attach()
     command('set noincsearch nohlsearch inccommand=')
     command('call setline(1, range(1, 2))')
 
@@ -644,7 +660,6 @@ describe('messages', function()
   -- oldtest: Test_fileinfo_tabpage_cmdheight()
   it("fileinfo works when 'cmdheight' has just decreased", function()
     screen = Screen.new(40, 6)
-    screen:attach()
 
     exec([[
       set shortmess-=o
@@ -673,7 +688,6 @@ describe('messages', function()
   -- oldtest: Test_fileinfo_after_echo()
   it('fileinfo does not overwrite echo message vim-patch:8.2.4156', function()
     screen = Screen.new(40, 6)
-    screen:attach()
 
     exec([[
       set shortmess-=F
@@ -698,4 +712,43 @@ describe('messages', function()
     ]])
     os.remove('b.txt')
   end)
+
+  -- oldtest: Test_messagesopt_wait()
+  it('&messagesopt "wait"', function()
+    screen = Screen.new(45, 6)
+    command('set cmdheight=1')
+
+    -- Check hit-enter prompt
+    command('set messagesopt=hit-enter,history:500')
+    feed(":echo 'foo' | echo 'bar' | echo 'baz'\n")
+    screen:expect([[
+                                                   |
+      {3:                                             }|
+      foo                                          |
+      bar                                          |
+      baz                                          |
+      {6:Press ENTER or type command to continue}^      |
+    ]])
+    feed('<CR>')
+
+    -- Check no hit-enter prompt when "wait:" is set
+    command('set messagesopt=wait:500,history:500')
+    feed(":echo 'foo' | echo 'bar' | echo 'baz'\n")
+    screen:expect({
+      grid = [[
+                                                   |
+      {1:~                                            }|
+      {3:                                             }|
+      foo                                          |
+      bar                                          |
+      baz                                          |
+    ]],
+      timeout = 500,
+    })
+    screen:expect([[
+      ^                                             |
+      {1:~                                            }|*4
+                                                   |
+    ]])
+  end)
 end)
