@@ -25,10 +25,9 @@ after_each(function()
 end)
 
 describe('semantic token highlighting', function()
-  local screen
+  local screen --- @type test.functional.ui.screen
   before_each(function()
     screen = Screen.new(40, 16)
-    screen:attach()
     screen:set_default_attr_ids {
       [1] = { bold = true, foreground = Screen.colors.Blue1 },
       [2] = { foreground = Screen.colors.DarkCyan },
@@ -84,10 +83,8 @@ describe('semantic token highlighting', function()
 
     before_each(function()
       exec_lua(create_server_definition)
-      exec_lua(
-        [[
-        local legend, response, edit_response = ...
-        server = _create_server({
+      exec_lua(function()
+        _G.server = _G._create_server({
           capabilities = {
             semanticTokensProvider = {
               full = { delta = true },
@@ -101,24 +98,19 @@ describe('semantic token highlighting', function()
             ['textDocument/semanticTokens/full/delta'] = function(_, _, callback)
               callback(nil, vim.fn.json_decode(edit_response))
             end,
-          }
+          },
         })
-      ]],
-        legend,
-        response,
-        edit_response
-      )
+      end, legend, response, edit_response)
     end)
 
     it('buffer is highlighted when attached', function()
-      exec_lua([[
-        bufnr = vim.api.nvim_get_current_buf()
+      insert(text)
+      exec_lua(function()
+        local bufnr = vim.api.nvim_get_current_buf()
         vim.api.nvim_win_set_buf(0, bufnr)
         vim.bo[bufnr].filetype = 'some-filetype'
-        client_id = vim.lsp.start({ name = 'dummy', cmd = server.cmd })
-      ]])
-
-      insert(text)
+        vim.lsp.start({ name = 'dummy', cmd = _G.server.cmd })
+      end)
 
       screen:expect {
         grid = [[
@@ -141,23 +133,20 @@ describe('semantic token highlighting', function()
     end)
 
     it('use LspTokenUpdate and highlight_token', function()
-      exec_lua([[
-        vim.api.nvim_create_autocmd("LspTokenUpdate", {
+      insert(text)
+      exec_lua(function()
+        vim.api.nvim_create_autocmd('LspTokenUpdate', {
           callback = function(args)
-            local token = args.data.token
-            if token.type == "function" and token.modifiers.declaration then
-              vim.lsp.semantic_tokens.highlight_token(
-                token, args.buf, args.data.client_id, "Macro"
-              )
+            local token = args.data.token --- @type STTokenRange
+            if token.type == 'function' and token.modifiers.declaration then
+              vim.lsp.semantic_tokens.highlight_token(token, args.buf, args.data.client_id, 'Macro')
             end
           end,
         })
-        bufnr = vim.api.nvim_get_current_buf()
+        local bufnr = vim.api.nvim_get_current_buf()
         vim.api.nvim_win_set_buf(0, bufnr)
-        client_id = vim.lsp.start({ name = 'dummy', cmd = server.cmd })
-      ]])
-
-      insert(text)
+        vim.lsp.start({ name = 'dummy', cmd = _G.server.cmd })
+      end)
 
       screen:expect {
         grid = [[
@@ -180,18 +169,23 @@ describe('semantic token highlighting', function()
     end)
 
     it('buffer is unhighlighted when client is detached', function()
-      exec_lua([[
-        bufnr = vim.api.nvim_get_current_buf()
-        vim.api.nvim_win_set_buf(0, bufnr)
-        client_id = vim.lsp.start({ name = 'dummy', cmd = server.cmd })
-      ]])
-
       insert(text)
 
-      exec_lua([[
+      local bufnr = n.api.nvim_get_current_buf()
+      local client_id = exec_lua(function()
+        vim.api.nvim_win_set_buf(0, bufnr)
+        local client_id = vim.lsp.start({ name = 'dummy', cmd = _G.server.cmd })
+        vim.wait(1000, function()
+          return #_G.server.messages > 1
+        end)
+        return client_id
+      end)
+
+      exec_lua(function()
+        --- @diagnostic disable-next-line:duplicate-set-field
         vim.notify = function() end
         vim.lsp.buf_detach_client(bufnr, client_id)
-      ]])
+      end)
 
       screen:expect {
         grid = [[
@@ -216,18 +210,19 @@ describe('semantic token highlighting', function()
     it(
       'buffer is highlighted and unhighlighted when semantic token highlighting is started and stopped',
       function()
-        exec_lua([[
-        bufnr = vim.api.nvim_get_current_buf()
-        vim.api.nvim_win_set_buf(0, bufnr)
-        client_id = vim.lsp.start({ name = 'dummy', cmd = server.cmd })
-      ]])
+        local bufnr = n.api.nvim_get_current_buf()
+        local client_id = exec_lua(function()
+          vim.api.nvim_win_set_buf(0, bufnr)
+          return vim.lsp.start({ name = 'dummy', cmd = _G.server.cmd })
+        end)
 
         insert(text)
 
-        exec_lua([[
-        vim.notify = function() end
-        vim.lsp.semantic_tokens.stop(bufnr, client_id)
-      ]])
+        exec_lua(function()
+          --- @diagnostic disable-next-line:duplicate-set-field
+          vim.notify = function() end
+          vim.lsp.semantic_tokens.stop(bufnr, client_id)
+        end)
 
         screen:expect {
           grid = [[
@@ -248,9 +243,9 @@ describe('semantic token highlighting', function()
       ]],
         }
 
-        exec_lua([[
-        vim.lsp.semantic_tokens.start(bufnr, client_id)
-      ]])
+        exec_lua(function()
+          vim.lsp.semantic_tokens.start(bufnr, client_id)
+        end)
 
         screen:expect {
           grid = [[
@@ -273,15 +268,68 @@ describe('semantic token highlighting', function()
       end
     )
 
-    it('buffer is re-highlighted when force refreshed', function()
-      exec_lua([[
-        bufnr = vim.api.nvim_get_current_buf()
-        vim.api.nvim_win_set_buf(0, bufnr)
-        client_id = vim.lsp.start({ name = 'dummy', cmd = server.cmd })
-      ]])
+    it('highlights start and stop when using "0" for current buffer', function()
+      local client_id = exec_lua(function()
+        return vim.lsp.start({ name = 'dummy', cmd = _G.server.cmd })
+      end)
 
       insert(text)
 
+      exec_lua(function()
+        --- @diagnostic disable-next-line:duplicate-set-field
+        vim.notify = function() end
+        vim.lsp.semantic_tokens.stop(0, client_id)
+      end)
+
+      screen:expect {
+        grid = [[
+        #include <iostream>                     |
+                                                |
+        int main()                              |
+        {                                       |
+            int x;                              |
+        #ifdef __cplusplus                      |
+            std::cout << x << "\n";             |
+        #else                                   |
+            printf("%d\n", x);                  |
+        #endif                                  |
+        }                                       |
+        ^}                                       |
+        {1:~                                       }|*3
+                                                |
+      ]],
+      }
+
+      exec_lua(function()
+        vim.lsp.semantic_tokens.start(0, client_id)
+      end)
+
+      screen:expect {
+        grid = [[
+        #include <iostream>                     |
+                                                |
+        int {8:main}()                              |
+        {                                       |
+            int {7:x};                              |
+        #ifdef {5:__cplusplus}                      |
+            {4:std}::{2:cout} << {2:x} << "\n";             |
+        {6:#else}                                   |
+        {6:    printf("%d\n", x);}                  |
+        {6:#endif}                                  |
+        }                                       |
+        ^}                                       |
+        {1:~                                       }|*3
+                                                |
+      ]],
+      }
+    end)
+
+    it('buffer is re-highlighted when force refreshed', function()
+      insert(text)
+      exec_lua(function()
+        vim.lsp.start({ name = 'dummy', cmd = _G.server.cmd })
+      end)
+
       screen:expect {
         grid = [[
         #include <iostream>                     |
@@ -301,9 +349,9 @@ describe('semantic token highlighting', function()
       ]],
       }
 
-      exec_lua([[
-        vim.lsp.semantic_tokens.force_refresh(bufnr)
-      ]])
+      exec_lua(function()
+        vim.lsp.semantic_tokens.force_refresh()
+      end)
 
       screen:expect {
         grid = [[
@@ -327,7 +375,9 @@ describe('semantic token highlighting', function()
 
       local messages = exec_lua('return server.messages')
       local token_request_count = 0
-      for _, message in ipairs(messages) do
+      for _, message in
+        ipairs(messages --[[@as {method:string,params:table}[] ]])
+      do
         assert(message.method ~= 'textDocument/semanticTokens/full/delta', 'delta request received')
         if message.method == 'textDocument/semanticTokens/full' then
           token_request_count = token_request_count + 1
@@ -337,31 +387,29 @@ describe('semantic token highlighting', function()
     end)
 
     it('destroys the highlighter if the buffer is deleted', function()
-      exec_lua([[
-        bufnr = vim.api.nvim_get_current_buf()
-        vim.api.nvim_win_set_buf(0, bufnr)
-        client_id = vim.lsp.start({ name = 'dummy', cmd = server.cmd })
-      ]])
+      exec_lua(function()
+        vim.lsp.start({ name = 'dummy', cmd = _G.server.cmd })
+      end)
 
       insert(text)
 
-      local highlighters = exec_lua([[
-        vim.api.nvim_buf_delete(bufnr, { force = true })
-        local semantic_tokens = vim.lsp.semantic_tokens
-        return semantic_tokens.__STHighlighter.active
-      ]])
-
-      eq({}, highlighters)
+      eq(
+        {},
+        exec_lua(function()
+          local bufnr = vim.api.nvim_get_current_buf()
+          vim.api.nvim_buf_delete(bufnr, { force = true })
+          return vim.lsp.semantic_tokens.__STHighlighter.active
+        end)
+      )
     end)
 
     it('updates highlights with delta request on buffer change', function()
-      exec_lua([[
-        bufnr = vim.api.nvim_get_current_buf()
-        vim.api.nvim_win_set_buf(0, bufnr)
-        client_id = vim.lsp.start({ name = 'dummy', cmd = server.cmd })
-      ]])
-
       insert(text)
+
+      exec_lua(function()
+        vim.lsp.start({ name = 'dummy', cmd = _G.server.cmd })
+      end)
+
       screen:expect {
         grid = [[
         #include <iostream>                     |
@@ -402,45 +450,49 @@ describe('semantic token highlighting', function()
     end)
 
     it('prevents starting semantic token highlighting with invalid conditions', function()
-      exec_lua([[
-        bufnr = vim.api.nvim_get_current_buf()
-        vim.api.nvim_win_set_buf(0, bufnr)
-        client_id = vim.lsp.start_client({ name = 'dummy', cmd = server.cmd })
-        notifications = {}
-        vim.notify = function(...) table.insert(notifications, 1, {...}) end
-      ]])
-      eq(false, exec_lua('return vim.lsp.buf_is_attached(bufnr, client_id)'))
+      local client_id = exec_lua(function()
+        _G.notifications = {}
+        --- @diagnostic disable-next-line:duplicate-set-field
+        vim.notify = function(...)
+          table.insert(_G.notifications, 1, { ... })
+        end
+        return vim.lsp.start({ name = 'dummy', cmd = _G.server.cmd }, { attach = false })
+      end)
+      eq(false, exec_lua('return vim.lsp.buf_is_attached(0, ...)', client_id))
 
       insert(text)
 
-      local notifications = exec_lua([[
-        vim.lsp.semantic_tokens.start(bufnr, client_id)
-        return notifications
-      ]])
-      matches('%[LSP%] Client with id %d not attached to buffer %d', notifications[1][1])
+      matches(
+        '%[LSP%] Client with id %d not attached to buffer %d',
+        exec_lua(function()
+          vim.lsp.semantic_tokens.start(0, client_id)
+          return _G.notifications[1][1]
+        end)
+      )
 
-      notifications = exec_lua([[
-        vim.lsp.semantic_tokens.start(bufnr, client_id + 1)
-        return notifications
-      ]])
-      matches('%[LSP%] No client with id %d', notifications[1][1])
+      matches(
+        '%[LSP%] No client with id %d',
+        exec_lua(function()
+          vim.lsp.semantic_tokens.start(0, client_id + 1)
+          return _G.notifications[1][1]
+        end)
+      )
     end)
 
     it(
       'opt-out: does not activate semantic token highlighting if disabled in client attach',
       function()
-        exec_lua([[
-          bufnr = vim.api.nvim_get_current_buf()
-          vim.api.nvim_win_set_buf(0, bufnr)
-          client_id = vim.lsp.start({
+        local client_id = exec_lua(function()
+          return vim.lsp.start({
             name = 'dummy',
-            cmd = server.cmd,
-            on_attach = vim.schedule_wrap(function(client, bufnr)
+            cmd = _G.server.cmd,
+            --- @param client vim.lsp.Client
+            on_attach = vim.schedule_wrap(function(client, _bufnr)
               client.server_capabilities.semanticTokensProvider = nil
             end),
           })
-        ]])
-        eq(true, exec_lua('return vim.lsp.buf_is_attached(bufnr, client_id)'))
+        end)
+        eq(true, exec_lua('return vim.lsp.buf_is_attached(0, ...)', client_id))
 
         insert(text)
 
@@ -463,13 +515,18 @@ describe('semantic token highlighting', function()
         ]],
         }
 
-        local notifications = exec_lua([[
-          local notifications = {}
-          vim.notify = function(...) table.insert(notifications, 1, {...}) end
-          vim.lsp.semantic_tokens.start(bufnr, client_id)
-          return notifications
-        ]])
-        eq('[LSP] Server does not support semantic tokens', notifications[1][1])
+        eq(
+          '[LSP] Server does not support semantic tokens',
+          exec_lua(function()
+            local notifications = {}
+            --- @diagnostic disable-next-line:duplicate-set-field
+            vim.notify = function(...)
+              table.insert(notifications, 1, { ... })
+            end
+            vim.lsp.semantic_tokens.start(0, client_id)
+            return notifications[1][1]
+          end)
+        )
 
         screen:expect {
           grid = [[
@@ -494,28 +551,32 @@ describe('semantic token highlighting', function()
     )
 
     it('ignores null responses from the server', function()
-      exec_lua([[
-        local legend, response, edit_response = ...
-        server2 = _create_server({
+      local client_id = exec_lua(function()
+        _G.server2 = _G._create_server({
           capabilities = {
             semanticTokensProvider = {
               full = { delta = false },
             },
           },
           handlers = {
+            --- @param callback function
             ['textDocument/semanticTokens/full'] = function(_, _, callback)
               callback(nil, nil)
             end,
-            ['textDocument/semanticTokens/full/delta'] = function()
+            --- @param callback function
+            ['textDocument/semanticTokens/full/delta'] = function(_, _, callback)
               callback(nil, nil)
             end,
-          }
+          },
         })
-        bufnr = vim.api.nvim_get_current_buf()
-        vim.api.nvim_win_set_buf(0, bufnr)
-        client_id = vim.lsp.start({ name = 'dummy', cmd = server2.cmd })
-      ]])
-      eq(true, exec_lua('return vim.lsp.buf_is_attached(bufnr, client_id)'))
+        return vim.lsp.start({ name = 'dummy', cmd = _G.server2.cmd })
+      end)
+      eq(
+        true,
+        exec_lua(function()
+          return vim.lsp.buf_is_attached(0, client_id)
+        end)
+      )
 
       insert(text)
 
@@ -539,11 +600,43 @@ describe('semantic token highlighting', function()
       }
     end)
 
+    it('resets active request after receiving error responses from the server', function()
+      local error = { code = -32801, message = 'Content modified' }
+      exec_lua(function()
+        _G.server2 = _G._create_server({
+          capabilities = {
+            semanticTokensProvider = {
+              full = { delta = false },
+            },
+          },
+          handlers = {
+            -- There is same logic for handling nil responses and error responses,
+            -- so keep responses not nil.
+            --
+            -- if an error response was not be handled, this test will hang on here.
+            --- @param callback function
+            ['textDocument/semanticTokens/full'] = function(_, _, callback)
+              callback(error, vim.fn.json_decode(response))
+            end,
+            --- @param callback function
+            ['textDocument/semanticTokens/full/delta'] = function(_, _, callback)
+              callback(error, vim.fn.json_decode(response))
+            end,
+          },
+        })
+        return vim.lsp.start({ name = 'dummy', cmd = _G.server2.cmd })
+      end)
+      screen:expect([[
+        ^                                        |
+        {1:~                                       }|*14
+                                                |
+      ]])
+    end)
+
     it('does not send delta requests if not supported by server', function()
-      exec_lua(
-        [[
-        local legend, response, edit_response = ...
-        server2 = _create_server({
+      insert(text)
+      exec_lua(function()
+        _G.server2 = _G._create_server({
           capabilities = {
             semanticTokensProvider = {
               full = { delta = false },
@@ -557,18 +650,11 @@ describe('semantic token highlighting', function()
             ['textDocument/semanticTokens/full/delta'] = function(_, _, callback)
               callback(nil, vim.fn.json_decode(edit_response))
             end,
-          }
+          },
         })
-        bufnr = vim.api.nvim_get_current_buf()
-        vim.api.nvim_win_set_buf(0, bufnr)
-        client_id = vim.lsp.start({ name = 'dummy', cmd = server2.cmd })
-      ]],
-        legend,
-        response,
-        edit_response
-      )
+        return vim.lsp.start({ name = 'dummy', cmd = _G.server2.cmd })
+      end)
 
-      insert(text)
       screen:expect {
         grid = [[
         #include <iostream>                     |
@@ -612,7 +698,9 @@ describe('semantic token highlighting', function()
       }
       local messages = exec_lua('return server2.messages')
       local token_request_count = 0
-      for _, message in ipairs(messages) do
+      for _, message in
+        ipairs(messages --[[@as {method:string,params:table}[] ]])
+      do
         assert(message.method ~= 'textDocument/semanticTokens/full/delta', 'delta request received')
         if message.method == 'textDocument/semanticTokens/full' then
           token_request_count = token_request_count + 1
@@ -1007,10 +1095,8 @@ b = "as"]],
     }) do
       it(test.it, function()
         exec_lua(create_server_definition)
-        exec_lua(
-          [[
-          local legend, resp = ...
-          server = _create_server({
+        local client_id = exec_lua(function(legend, resp)
+          _G.server = _G._create_server({
             capabilities = {
               semanticTokensProvider = {
                 full = { delta = false },
@@ -1021,25 +1107,22 @@ b = "as"]],
               ['textDocument/semanticTokens/full'] = function(_, _, callback)
                 callback(nil, vim.fn.json_decode(resp))
               end,
-            }
+            },
           })
-          bufnr = vim.api.nvim_get_current_buf()
-          vim.api.nvim_win_set_buf(0, bufnr)
-          client_id = vim.lsp.start({ name = 'dummy', cmd = server.cmd })
-        ]],
-          test.legend,
-          test.response
-        )
+          return vim.lsp.start({ name = 'dummy', cmd = _G.server.cmd })
+        end, test.legend, test.response)
 
         insert(test.text)
 
         test.expected_screen()
 
-        local highlights = exec_lua([[
-          local semantic_tokens = vim.lsp.semantic_tokens
-          return semantic_tokens.__STHighlighter.active[bufnr].client_state[client_id].current_result.highlights
-        ]])
-        eq(test.expected, highlights)
+        eq(
+          test.expected,
+          exec_lua(function()
+            local bufnr = vim.api.nvim_get_current_buf()
+            return vim.lsp.semantic_tokens.__STHighlighter.active[bufnr].client_state[client_id].current_result.highlights
+          end)
+        )
       end)
     end
   end)
@@ -1392,11 +1475,11 @@ int main()
       },
     }) do
       it(test.it, function()
+        local bufnr = n.api.nvim_get_current_buf()
+        insert(test.text1)
         exec_lua(create_server_definition)
-        exec_lua(
-          [[
-          local legend, resp1, resp2 = ...
-          server = _create_server({
+        local client_id = exec_lua(function(legend, resp1, resp2)
+          _G.server = _G._create_server({
             capabilities = {
               semanticTokensProvider = {
                 full = { delta = true },
@@ -1410,54 +1493,44 @@ int main()
               ['textDocument/semanticTokens/full/delta'] = function(_, _, callback)
                 callback(nil, vim.fn.json_decode(resp2))
               end,
-            }
+            },
           })
-          bufnr = vim.api.nvim_get_current_buf()
-          vim.api.nvim_win_set_buf(0, bufnr)
-          client_id = vim.lsp.start({ name = 'dummy', cmd = server.cmd })
+          local client_id = assert(vim.lsp.start({ name = 'dummy', cmd = _G.server.cmd }))
 
           -- speed up vim.api.nvim_buf_set_lines calls by changing debounce to 10 for these tests
-          semantic_tokens = vim.lsp.semantic_tokens
           vim.schedule(function()
-            semantic_tokens.stop(bufnr, client_id)
-            semantic_tokens.start(bufnr, client_id, { debounce = 10 })
+            vim.lsp.semantic_tokens.stop(bufnr, client_id)
+            vim.lsp.semantic_tokens.start(bufnr, client_id, { debounce = 10 })
           end)
-        ]],
-          test.legend,
-          test.response1,
-          test.response2
-        )
-
-        insert(test.text1)
+          return client_id
+        end, test.legend, test.response1, test.response2)
 
         test.expected_screen1()
 
-        local highlights = exec_lua([[
-          return semantic_tokens.__STHighlighter.active[bufnr].client_state[client_id].current_result.highlights
-        ]])
-
-        eq(test.expected1, highlights)
+        eq(
+          test.expected1,
+          exec_lua(function()
+            return vim.lsp.semantic_tokens.__STHighlighter.active[bufnr].client_state[client_id].current_result.highlights
+          end)
+        )
 
         if test.edit then
           feed(test.edit)
         else
-          exec_lua(
-            [[
-            local text = ...
-            vim.api.nvim_buf_set_lines(bufnr, 0, -1, false, vim.fn.split(text, "\n"))
+          exec_lua(function(text)
+            vim.api.nvim_buf_set_lines(bufnr, 0, -1, false, vim.fn.split(text, '\n'))
             vim.wait(15) -- wait for debounce
-          ]],
-            test.text2
-          )
+          end, test.text2)
         end
 
         test.expected_screen2()
 
-        highlights = exec_lua([[
-          return semantic_tokens.__STHighlighter.active[bufnr].client_state[client_id].current_result.highlights
-        ]])
-
-        eq(test.expected2, highlights)
+        eq(
+          test.expected2,
+          exec_lua(function()
+            return vim.lsp.semantic_tokens.__STHighlighter.active[bufnr].client_state[client_id].current_result.highlights
+          end)
+        )
       end)
     end
   end)
