@@ -11,8 +11,8 @@ local function md_to_vimdoc(text, start_indent, indent, text_width)
     start_indent = start_indent or 0
     indent = indent or 0
     text_width = text_width or 70
-    local text_utils = require('scripts/text_utils')
-    return text_utils.md_to_vimdoc(table.concat(text, '\n'), start_indent, indent, text_width)
+    local util = require('src/gen/util')
+    return util.md_to_vimdoc(table.concat(text, '\n'), start_indent, indent, text_width)
   ]],
     text,
     start_indent,
