@@ -1,24 +1,28 @@
-local helpers = require('test.functional.helpers')(after_each)
-local thelpers = require('test.functional.terminal.helpers')
-local clear, eq, eval = helpers.clear, helpers.eq, helpers.eval
-local feed, nvim, command = helpers.feed, helpers.nvim, helpers.command
-local feed_data = thelpers.feed_data
-local is_os = helpers.is_os
-local skip = helpers.skip
+local t = require('test.testutil')
+local n = require('test.functional.testnvim')()
+
+local tt = require('test.functional.testterm')
+local clear, eq, eval = n.clear, t.eq, n.eval
+local feed, api, command = n.feed, n.api, n.command
+local feed_data = tt.feed_data
+local is_os = t.is_os
+local skip = t.skip
 
 describe(':terminal mouse', function()
   local screen
 
   before_each(function()
     clear()
-    nvim('set_option', 'statusline', '==========')
-    command('highlight StatusLine cterm=NONE')
-    command('highlight StatusLineNC cterm=NONE')
-    command('highlight VertSplit cterm=NONE')
-    screen = thelpers.screen_setup()
+    api.nvim_set_option_value('statusline', '==========', {})
+    screen = tt.setup_screen()
+    command('highlight StatusLine NONE')
+    command('highlight StatusLineNC NONE')
+    command('highlight StatusLineTerm NONE')
+    command('highlight StatusLineTermNC NONE')
+    command('highlight VertSplit NONE')
     local lines = {}
     for i = 1, 30 do
-      table.insert(lines, 'line'..tostring(i))
+      table.insert(lines, 'line' .. tostring(i))
     end
     table.insert(lines, '')
     feed_data(lines)
@@ -28,7 +32,7 @@ describe(':terminal mouse', function()
       line28                                            |
       line29                                            |
       line30                                            |
-      {1: }                                                 |
+      ^                                                  |
       {3:-- TERMINAL --}                                    |
     ]])
   end)
@@ -41,6 +45,7 @@ describe(':terminal mouse', function()
     end)
 
     it('will exit focus and trigger Normal mode mapping on mouse click', function()
+      feed([[<C-\><C-N>qri]])
       command('let g:got_leftmouse = 0')
       command('nnoremap <LeftMouse> <Cmd>let g:got_leftmouse = 1<CR>')
       eq('t', eval('mode(1)'))
@@ -48,9 +53,12 @@ describe(':terminal mouse', function()
       feed('<LeftMouse>')
       eq('nt', eval('mode(1)'))
       eq(1, eval('g:got_leftmouse'))
+      feed('q')
+      eq('i<LeftMouse>', eval('keytrans(@r)'))
     end)
 
     it('will exit focus and trigger Normal mode mapping on mouse click with modifier', function()
+      feed([[<C-\><C-N>qri]])
       command('let g:got_ctrl_leftmouse = 0')
       command('nnoremap <C-LeftMouse> <Cmd>let g:got_ctrl_leftmouse = 1<CR>')
       eq('t', eval('mode(1)'))
@@ -58,6 +66,8 @@ describe(':terminal mouse', function()
       feed('<C-LeftMouse>')
       eq('nt', eval('mode(1)'))
       eq(1, eval('g:got_ctrl_leftmouse'))
+      feed('q')
+      eq('i<C-LeftMouse>', eval('keytrans(@r)'))
     end)
 
     it('will exit focus on <C-\\> + mouse-scroll', function()
@@ -89,15 +99,15 @@ describe(':terminal mouse', function()
 
     describe('with mouse events enabled by the program', function()
       before_each(function()
-        thelpers.enable_mouse()
-        thelpers.feed_data('mouse enabled\n')
+        tt.enable_mouse()
+        tt.feed_data('mouse enabled\n')
         screen:expect([[
           line27                                            |
           line28                                            |
           line29                                            |
           line30                                            |
           mouse enabled                                     |
-          {1: }                                                 |
+          ^                                                  |
           {3:-- TERMINAL --}                                    |
         ]])
       end)
@@ -111,7 +121,7 @@ describe(':terminal mouse', function()
           line29                                            |
           line30                                            |
           mouse enabled                                     |
-           "#{1: }                                              |
+           "#^                                               |
           {3:-- TERMINAL --}                                    |
         ]])
         feed('<LeftDrag><2,2>')
@@ -121,7 +131,7 @@ describe(':terminal mouse', function()
           line29                                            |
           line30                                            |
           mouse enabled                                     |
-             @##{1: }                                           |
+             @##^                                            |
           {3:-- TERMINAL --}                                    |
         ]])
         feed('<LeftDrag><3,2>')
@@ -131,7 +141,7 @@ describe(':terminal mouse', function()
           line29                                            |
           line30                                            |
           mouse enabled                                     |
-                @$#{1: }                                        |
+                @$#^                                         |
           {3:-- TERMINAL --}                                    |
         ]])
         feed('<LeftRelease><3,2>')
@@ -141,7 +151,7 @@ describe(':terminal mouse', function()
           line29                                            |
           line30                                            |
           mouse enabled                                     |
-                   #$#{1: }                                     |
+                   #$#^                                      |
           {3:-- TERMINAL --}                                    |
         ]])
       end)
@@ -155,7 +165,7 @@ describe(':terminal mouse', function()
           line29                                            |
           line30                                            |
           mouse enabled                                     |
-          `!!{1: }                                              |
+          `!!^                                               |
           {3:-- TERMINAL --}                                    |
         ]])
       end)
@@ -169,7 +179,7 @@ describe(':terminal mouse', function()
           line29                                            |
           line30                                            |
           mouse enabled                                     |
-           "#{1: }                                              |
+           "#^                                               |
           {3:-- TERMINAL --}                                    |
         ]])
         feed('<ScrollWheelUp><1,2>')
@@ -179,7 +189,7 @@ describe(':terminal mouse', function()
           line29                                            |
           line30                                            |
           mouse enabled                                     |
-             `"#{1: }                                           |
+             `"#^                                            |
           {3:-- TERMINAL --}                                    |
         ]])
         feed('<LeftDrag><2,2>')
@@ -189,7 +199,7 @@ describe(':terminal mouse', function()
           line29                                            |
           line30                                            |
           mouse enabled                                     |
-                @##{1: }                                        |
+                @##^                                         |
           {3:-- TERMINAL --}                                    |
         ]])
         feed('<ScrollWheelUp><2,2>')
@@ -199,7 +209,7 @@ describe(':terminal mouse', function()
           line29                                            |
           line30                                            |
           mouse enabled                                     |
-                   `##{1: }                                     |
+                   `##^                                      |
           {3:-- TERMINAL --}                                    |
         ]])
         feed('<LeftRelease><2,2>')
@@ -209,7 +219,7 @@ describe(':terminal mouse', function()
           line29                                            |
           line30                                            |
           mouse enabled                                     |
-                      ###{1: }                                  |
+                      ###^                                   |
           {3:-- TERMINAL --}                                    |
         ]])
       end)
@@ -217,6 +227,15 @@ describe(':terminal mouse', function()
       it('will forward mouse clicks to the program with the correct even if set nu', function()
         skip(is_os('win'))
         command('set number')
+        screen:expect([[
+          {7: 11 }line28                                        |
+          {7: 12 }line29                                        |
+          {7: 13 }line30                                        |
+          {7: 14 }mouse enabled                                 |
+          {7: 15 }rows: 6, cols: 46                             |
+          {7: 16 }^                                              |
+          {3:-- TERMINAL --}                                    |
+        ]])
         -- When the display area such as a number is clicked, it returns to the
         -- normal mode.
         feed('<LeftMouse><3,0>')
@@ -227,7 +246,7 @@ describe(':terminal mouse', function()
           {7: 13 }line30                                        |
           {7: 14 }mouse enabled                                 |
           {7: 15 }rows: 6, cols: 46                             |
-          {7: 16 }{2: }                                             |
+          {7: 16 }                                              |
                                                             |
         ]])
         -- If click on the coordinate (0,1) of the region of the terminal
@@ -239,10 +258,138 @@ describe(':terminal mouse', function()
           {7: 13 }line30                                        |
           {7: 14 }mouse enabled                                 |
           {7: 15 }rows: 6, cols: 46                             |
-          {7: 16 } !"{1: }                                          |
+          {7: 16 } !"^                                           |
           {3:-- TERMINAL --}                                    |
         ]])
       end)
+
+      it('will lose focus if statusline is clicked', function()
+        command('set laststatus=2')
+        screen:expect([[
+          line29                                            |
+          line30                                            |
+          mouse enabled                                     |
+          rows: 5, cols: 50                                 |
+          ^                                                  |
+          ==========                                        |
+          {3:-- TERMINAL --}                                    |
+        ]])
+        feed('<LeftMouse><0,5>')
+        screen:expect([[
+          line29                                            |
+          line30                                            |
+          mouse enabled                                     |
+          rows: 5, cols: 50                                 |
+          ^                                                  |
+          ==========                                        |
+                                                            |
+        ]])
+        feed('<LeftDrag><0,4>')
+        screen:expect([[
+          mouse enabled                                     |
+          rows: 5, cols: 50                                 |
+          rows: 4, cols: 50                                 |
+          ^                                                  |
+          ==========                                        |
+                                                            |*2
+        ]])
+      end)
+
+      it('will lose focus if right separator is clicked', function()
+        command('rightbelow vnew | wincmd p | startinsert')
+        screen:expect([[
+          line29                  │                         |
+          line30                  │{4:~                        }|
+          mouse enabled           │{4:~                        }|
+          rows: 5, cols: 24       │{4:~                        }|
+          ^                        │{4:~                        }|
+          ==========               ==========               |
+          {3:-- TERMINAL --}                                    |
+        ]])
+        feed('<LeftMouse><24,0>')
+        screen:expect([[
+          line29                  │                         |
+          line30                  │{4:~                        }|
+          mouse enabled           │{4:~                        }|
+          rows: 5, cols: 24       │{4:~                        }|
+          ^                        │{4:~                        }|
+          ==========               ==========               |
+                                                            |
+        ]])
+        feed('<LeftDrag><23,0>')
+        screen:expect([[
+          line30                 │                          |
+          mouse enabled          │{4:~                         }|
+          rows: 5, cols: 24      │{4:~                         }|
+          rows: 5, cols: 23      │{4:~                         }|
+          ^                       │{4:~                         }|
+          ==========              ==========                |
+                                                            |
+        ]])
+      end)
+
+      it('will lose focus if winbar/tabline is clicked', function()
+        command('setlocal winbar=WINBAR')
+        screen:expect([[
+          {3:WINBAR                                            }|
+          line29                                            |
+          line30                                            |
+          mouse enabled                                     |
+          rows: 5, cols: 50                                 |
+          ^                                                  |
+          {3:-- TERMINAL --}                                    |
+        ]])
+        feed('<LeftMouse><0,0>')
+        screen:expect([[
+          {3:WINBAR                                            }|
+          line29                                            |
+          line30                                            |
+          mouse enabled                                     |
+          rows: 5, cols: 50                                 |
+          ^                                                  |
+                                                            |
+        ]])
+        command('set showtabline=2 tabline=TABLINE | startinsert')
+        screen:expect([[
+          {1:TABLINE                                           }|
+          {3:WINBAR                                            }|
+          mouse enabled                                     |
+          rows: 5, cols: 50                                 |
+          rows: 4, cols: 50                                 |
+          ^                                                  |
+          {3:-- TERMINAL --}                                    |
+        ]])
+        feed('<LeftMouse><0,0>')
+        screen:expect([[
+          {1:TABLINE                                           }|
+          {3:WINBAR                                            }|
+          mouse enabled                                     |
+          rows: 5, cols: 50                                 |
+          rows: 4, cols: 50                                 |
+          ^                                                  |
+                                                            |
+        ]])
+        command('setlocal winbar= | startinsert')
+        screen:expect([[
+          {1:TABLINE                                           }|
+          mouse enabled                                     |
+          rows: 5, cols: 50                                 |
+          rows: 4, cols: 50                                 |
+          rows: 5, cols: 50                                 |
+          ^                                                  |
+          {3:-- TERMINAL --}                                    |
+        ]])
+        feed('<LeftMouse><0,0>')
+        screen:expect([[
+          {1:TABLINE                                           }|
+          mouse enabled                                     |
+          rows: 5, cols: 50                                 |
+          rows: 4, cols: 50                                 |
+          rows: 5, cols: 50                                 |
+          ^                                                  |
+                                                            |
+        ]])
+      end)
     end)
 
     describe('with a split window and other buffer', function()
@@ -253,7 +400,7 @@ describe(':terminal mouse', function()
           line29                   │line29                  |
           line30                   │line30                  |
           rows: 5, cols: 25        │rows: 5, cols: 25       |
-          {2:^ }                        │{2: }                       |
+          ^                         │                        |
           ==========                ==========              |
           :vsp                                              |
         ]])
@@ -263,7 +410,7 @@ describe(':terminal mouse', function()
           {4:~                        }│line30                  |
           {4:~                        }│rows: 5, cols: 25       |
           {4:~                        }│rows: 5, cols: 24       |
-          {4:~                        }│{2: }                       |
+          {4:~                        }│                        |
           ==========                ==========              |
           :enew | set number                                |
         ]])
@@ -273,7 +420,7 @@ describe(':terminal mouse', function()
           {7: 28 }line                 │line30                  |
           {7: 29 }line                 │rows: 5, cols: 25       |
           {7: 30 }line                 │rows: 5, cols: 24       |
-          {7: 31 }^                     │{2: }                       |
+          {7: 31 }^                     │                        |
           ==========                ==========              |
                                                             |
         ]])
@@ -283,33 +430,33 @@ describe(':terminal mouse', function()
           {7: 28 }line                 │line30                  |
           {7: 29 }line                 │rows: 5, cols: 25       |
           {7: 30 }line                 │rows: 5, cols: 24       |
-          {7: 31 }                     │{1: }                       |
+          {7: 31 }                     │^                        |
           ==========                ==========              |
           {3:-- TERMINAL --}                                    |
         ]])
 
         -- enabling mouse won't affect interaction with other windows
-        thelpers.enable_mouse()
-        thelpers.feed_data('mouse enabled\n')
+        tt.enable_mouse()
+        tt.feed_data('mouse enabled\n')
         screen:expect([[
           {7: 27 }line                 │line30                  |
           {7: 28 }line                 │rows: 5, cols: 25       |
           {7: 29 }line                 │rows: 5, cols: 24       |
           {7: 30 }line                 │mouse enabled           |
-          {7: 31 }                     │{1: }                       |
+          {7: 31 }                     │^                        |
           ==========                ==========              |
           {3:-- TERMINAL --}                                    |
         ]])
       end)
 
-      it("won't lose focus if another window is scrolled", function()
+      it("scrolling another window keeps focus and respects 'mousescroll'", function()
         feed('<ScrollWheelUp><4,0><ScrollWheelUp><4,0>')
         screen:expect([[
           {7: 21 }line                 │line30                  |
           {7: 22 }line                 │rows: 5, cols: 25       |
           {7: 23 }line                 │rows: 5, cols: 24       |
           {7: 24 }line                 │mouse enabled           |
-          {7: 25 }line                 │{1: }                       |
+          {7: 25 }line                 │^                        |
           ==========                ==========              |
           {3:-- TERMINAL --}                                    |
         ]])
@@ -319,21 +466,7 @@ describe(':terminal mouse', function()
           {7: 27 }line                 │rows: 5, cols: 25       |
           {7: 28 }line                 │rows: 5, cols: 24       |
           {7: 29 }line                 │mouse enabled           |
-          {7: 30 }line                 │{1: }                       |
-          ==========                ==========              |
-          {3:-- TERMINAL --}                                    |
-        ]])
-      end)
-
-      it("scrolling another window respects 'mousescroll'", function()
-        command('set mousescroll=ver:1')
-        feed('<ScrollWheelUp><4,0>')
-        screen:expect([[
-          {7: 26 }line                 │line30                  |
-          {7: 27 }line                 │rows: 5, cols: 25       |
-          {7: 28 }line                 │rows: 5, cols: 24       |
-          {7: 29 }line                 │mouse enabled           |
-          {7: 30 }line                 │{1: }                       |
+          {7: 30 }line                 │^                        |
           ==========                ==========              |
           {3:-- TERMINAL --}                                    |
         ]])
@@ -344,13 +477,46 @@ describe(':terminal mouse', function()
           {7: 17 }line                 │rows: 5, cols: 25       |
           {7: 18 }line                 │rows: 5, cols: 24       |
           {7: 19 }line                 │mouse enabled           |
-          {7: 20 }line                 │{1: }                       |
+          {7: 20 }line                 │^                        |
           ==========                ==========              |
           {3:-- TERMINAL --}                                    |
         ]])
         command('set mousescroll=ver:0')
         feed('<ScrollWheelUp><0,0>')
         screen:expect_unchanged()
+        feed([[<C-\><C-N><C-W>w]])
+        command('setlocal nowrap')
+        feed('0<C-V>gg3ly$4p<C-W>wi')
+        screen:expect([[
+          {7:  1 }linelinelinelineline │line30                  |
+          {7:  2 }linelinelinelineline │rows: 5, cols: 25       |
+          {7:  3 }linelinelinelineline │rows: 5, cols: 24       |
+          {7:  4 }linelinelinelineline │mouse enabled           |
+          {7:  5 }linelinelinelineline │^                        |
+          ==========                ==========              |
+          {3:-- TERMINAL --}                                    |
+        ]])
+        feed('<ScrollWheelRight><4,0>')
+        screen:expect([[
+          {7:  1 }nelinelineline       │line30                  |
+          {7:  2 }nelinelineline       │rows: 5, cols: 25       |
+          {7:  3 }nelinelineline       │rows: 5, cols: 24       |
+          {7:  4 }nelinelineline       │mouse enabled           |
+          {7:  5 }nelinelineline       │^                        |
+          ==========                ==========              |
+          {3:-- TERMINAL --}                                    |
+        ]])
+        command('set mousescroll=hor:4')
+        feed('<ScrollWheelLeft><4,0>')
+        screen:expect([[
+          {7:  1 }nelinelinelineline   │line30                  |
+          {7:  2 }nelinelinelineline   │rows: 5, cols: 25       |
+          {7:  3 }nelinelinelineline   │rows: 5, cols: 24       |
+          {7:  4 }nelinelinelineline   │mouse enabled           |
+          {7:  5 }nelinelinelineline   │^                        |
+          ==========                ==========              |
+          {3:-- TERMINAL --}                                    |
+        ]])
       end)
 
       it('will lose focus if another window is clicked', function()
@@ -360,21 +526,21 @@ describe(':terminal mouse', function()
           {7: 28 }l^ine                 │rows: 5, cols: 25       |
           {7: 29 }line                 │rows: 5, cols: 24       |
           {7: 30 }line                 │mouse enabled           |
-          {7: 31 }                     │{2: }                       |
+          {7: 31 }                     │                        |
           ==========                ==========              |
                                                             |
         ]])
       end)
 
       it('handles terminal size when switching buffers', function()
-        nvim('set_option', 'hidden', true)
+        api.nvim_set_option_value('hidden', true, {})
         feed('<c-\\><c-n><c-w><c-w>')
         screen:expect([[
           {7: 27 }line                 │line30                  |
           {7: 28 }line                 │rows: 5, cols: 25       |
           {7: 29 }line                 │rows: 5, cols: 24       |
           {7: 30 }line                 │mouse enabled           |
-          {7: 31 }^                     │{2: }                       |
+          {7: 31 }^                     │                        |
           ==========                ==========              |
                                                             |
         ]])
@@ -384,7 +550,7 @@ describe(':terminal mouse', function()
           rows: 5, cols: 24        │rows: 5, cols: 24       |
           mouse enabled            │mouse enabled           |
           rows: 5, cols: 25        │rows: 5, cols: 25       |
-          {2:^ }                        │{2: }                       |
+          ^                         │                        |
           ==========                ==========              |
           :bn                                               |
         ]])
@@ -394,7 +560,7 @@ describe(':terminal mouse', function()
           {7: 28 }line                 │mouse enabled           |
           {7: 29 }line                 │rows: 5, cols: 25       |
           {7: 30 }line                 │rows: 5, cols: 24       |
-          {7: 31 }^                     │{2: }                       |
+          {7: 31 }^                     │                        |
           ==========                ==========              |
           :bn                                               |
         ]])
