@@ -12,6 +12,19 @@ local exec = n.exec
 local eq = t.eq
 local api = n.api
 
+local function WriteDiffFiles(text1, text2)
+  write_file('Xdifile1', text1)
+  write_file('Xdifile2', text2)
+  command('checktime')
+end
+
+local function WriteDiffFiles3(text1, text2, text3)
+  write_file('Xdifile1', text1)
+  write_file('Xdifile2', text2)
+  write_file('Xdifile3', text3)
+  command('checktime')
+end
+
 before_each(clear)
 
 describe('Diff mode screen', function()
@@ -40,7 +53,6 @@ describe('Diff mode screen', function()
     feed('<c-w>w:diffthis<cr><c-w>w')
 
     screen = Screen.new(40, 16)
-    screen:attach()
   end)
 
   it('Add a line in beginning of file 2', function()
@@ -614,6 +626,34 @@ int main(int argc, char **argv)
     ]])
   end)
 
+  it('Diff empty and non-empty file', function()
+    write_file(fname, '', false)
+    write_file(fname_2, 'foo\nbar\nbaz', false)
+    reread()
+
+    feed(':set diffopt=filler<cr>')
+    screen:expect([[
+      {7:  }{23:------------------}│{7:  }{22:foo              }|
+      {7:  }{23:------------------}│{7:  }{22:bar              }|
+      {7:  }{23:------------------}│{7:  }{22:baz              }|
+      {7:  }^                  │{1:~                  }|
+      {1:~                   }│{1:~                  }|*10
+      {3:<onal-diff-screen-1  }{2:<l-diff-screen-1.2 }|
+      :set diffopt=filler                     |
+    ]])
+
+    feed(':set diffopt+=internal<cr>')
+    screen:expect([[
+      {7:  }{23:------------------}│{7:  }{22:foo              }|
+      {7:  }{23:------------------}│{7:  }{22:bar              }|
+      {7:  }{23:------------------}│{7:  }{22:baz              }|
+      {7:  }^                  │{1:~                  }|
+      {1:~                   }│{1:~                  }|*10
+      {3:<onal-diff-screen-1  }{2:<l-diff-screen-1.2 }|
+      :set diffopt+=internal                  |
+    ]])
+  end)
+
   it('diffopt+=icase', function()
     write_file(fname, 'a\nb\ncd\n', false)
     write_file(fname_2, 'A\nb\ncDe\n', false)
@@ -1131,7 +1171,6 @@ end)
 it('win_update redraws lines properly', function()
   local screen
   screen = Screen.new(50, 10)
-  screen:attach()
   screen:set_default_attr_ids({
     [1] = { bold = true, foreground = Screen.colors.Blue1 },
     [2] = { foreground = Screen.colors.Grey100, background = Screen.colors.Red },
@@ -1209,8 +1248,8 @@ end)
 -- oldtest: Test_diff_rnu()
 it('diff updates line numbers below filler lines', function()
   local screen = Screen.new(40, 14)
-  screen:attach()
   exec([[
+    set diffopt=internal,filler,closeoff
     call setline(1, ['a', 'a', 'a', 'y', 'b', 'b', 'b', 'b', 'b'])
     vnew
     call setline(1, ['a', 'a', 'a', 'x', 'x', 'x', 'b', 'b', 'b', 'b', 'b'])
@@ -1269,7 +1308,6 @@ end)
 -- oldtest: Test_diff_with_scroll_and_change()
 it('Align the filler lines when changing text in diff mode', function()
   local screen = Screen.new(40, 20)
-  screen:attach()
   exec([[
     call setline(1, range(1, 15))
     vnew
@@ -1335,7 +1373,6 @@ end)
 
 it("diff mode doesn't restore invalid 'foldcolumn' value #21647", function()
   local screen = Screen.new(60, 6)
-  screen:attach()
   eq('0', api.nvim_get_option_value('foldcolumn', {}))
   command('diffsplit | bd')
   screen:expect([[
@@ -1348,7 +1385,7 @@ end)
 
 it("'relativenumber' doesn't draw beyond end of window in diff mode #29403", function()
   local screen = Screen.new(60, 12)
-  screen:attach()
+  command('set diffopt=internal,filler,closeoff')
   command('set relativenumber')
   feed('10aa<CR><Esc>gg')
   command('vnew')
@@ -1392,7 +1429,6 @@ it('diff mode works properly if file contains NUL bytes vim-patch:8.2.3925', fun
   screen:add_extra_attr_ids {
     [100] = { foreground = Screen.colors.Blue, bold = true, background = Screen.colors.Red },
   }
-  screen:attach()
   exec([[
     call setline(1, ['a', 'b', "c\n", 'd', 'e', 'f', 'g'])
     vnew
@@ -1469,7 +1505,6 @@ end)
 -- oldtest: Test_diff_breakindent_after_filler()
 it("diff mode draws 'breakindent' correctly after filler lines", function()
   local screen = Screen.new(45, 8)
-  screen:attach()
   exec([[
     set laststatus=0 diffopt+=followwrap breakindent breakindentopt=min:0
     call setline(1, ['a', '  ' .. repeat('c', 50)])
@@ -1487,3 +1522,735 @@ it("diff mode draws 'breakindent' correctly after filler lines", function()
                                                  |
   ]])
 end)
+
+-- oldtest: Test_diff_overlapped_diff_blocks_will_be_merged()
+it('diff mode overlapped diff blocks will be merged', function()
+  write_file('Xdifile1', '')
+  write_file('Xdifile2', '')
+  write_file('Xdifile3', '')
+
+  finally(function()
+    os.remove('Xdifile1')
+    os.remove('Xdifile2')
+    os.remove('Xdifile3')
+    os.remove('Xdiin1')
+    os.remove('Xdinew1')
+    os.remove('Xdiout1')
+    os.remove('Xdiin2')
+    os.remove('Xdinew2')
+    os.remove('Xdiout2')
+  end)
+
+  exec([[
+    func DiffExprStub()
+      let txt_in = readfile(v:fname_in)
+      let txt_new = readfile(v:fname_new)
+      if txt_in == ["line1"] && txt_new == ["line2"]
+        call writefile(["1c1"], v:fname_out)
+      elseif txt_in == readfile("Xdiin1") && txt_new == readfile("Xdinew1")
+        call writefile(readfile("Xdiout1"), v:fname_out)
+      elseif txt_in == readfile("Xdiin2") && txt_new == readfile("Xdinew2")
+        call writefile(readfile("Xdiout2"), v:fname_out)
+      endif
+    endfunc
+  ]])
+
+  local screen = Screen.new(35, 20)
+  command('set winwidth=10 diffopt=filler,internal')
+
+  command('args Xdifile1 Xdifile2 | vert all | windo diffthis')
+
+  WriteDiffFiles('a\nb', 'x\nx')
+  write_file('Xdiin1', 'a\nb')
+  write_file('Xdinew1', 'x\nx')
+  write_file('Xdiout1', '1c1\n2c2')
+  command('set diffexpr=DiffExprStub()')
+  screen:expect([[
+    {7:  }{27:a}{4:              }│{7:  }{27:^x}{4:              }|
+    {7:  }{27:b}{4:              }│{7:  }{27:x}{4:              }|
+    {1:~                }│{1:~                }|*16
+    {2:Xdifile1          }{3:Xdifile2         }|
+                                       |
+  ]])
+  command('set diffexpr&')
+
+  WriteDiffFiles('a\nb\nc', 'x\nc')
+  write_file('Xdiin1', 'a\nb\nc')
+  write_file('Xdinew1', 'x\nc')
+  write_file('Xdiout1', '1c1\n2c1')
+  command('set diffexpr=DiffExprStub()')
+  screen:expect([[
+    {7:  }{27:a}{4:              }│{7:  }{27:^x}{4:              }|
+    {7:  }{22:b              }│{7:  }{23:---------------}|
+    {7:  }c              │{7:  }c              |
+    {1:~                }│{1:~                }|*15
+    {2:Xdifile1          }{3:Xdifile2         }|
+                                       |
+  ]])
+  command('set diffexpr&')
+
+  WriteDiffFiles('a\nc', 'x\nx\nc')
+  write_file('Xdiin1', 'a\nc')
+  write_file('Xdinew1', 'x\nx\nc')
+  write_file('Xdiout1', '1c1\n1a2')
+  command('set diffexpr=DiffExprStub()')
+  screen:expect([[
+    {7:  }{27:a}{4:              }│{7:  }{27:^x}{4:              }|
+    {7:  }{23:---------------}│{7:  }{22:x              }|
+    {7:  }c              │{7:  }c              |
+    {1:~                }│{1:~                }|*15
+    {2:Xdifile1          }{3:Xdifile2         }|
+                                       |
+  ]])
+  command('set diffexpr&')
+
+  command('args Xdifile1 Xdifile2 Xdifile3 | vert all | windo diffthis')
+
+  WriteDiffFiles3('a\nb\nc', 'a\nx\nc', 'y\nb\nc')
+  screen:expect([[
+    {7:  }{27:a}{4:        }│{7:  }{27:a}{4:        }│{7:  }{27:^y}{4:        }|
+    {7:  }{27:b}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:b}{4:        }|
+    {7:  }c        │{7:  }c        │{7:  }c        |
+    {1:~          }│{1:~          }│{1:~          }|*15
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  WriteDiffFiles3('a\nb\nc', 'a\nx\nc', 'a\ny\nc')
+  screen:expect([[
+    {7:  }a        │{7:  }a        │{7:  }^a        |
+    {7:  }{27:b}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:y}{4:        }|
+    {7:  }c        │{7:  }c        │{7:  }c        |
+    {1:~          }│{1:~          }│{1:~          }|*15
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  WriteDiffFiles3('a\nb\nc', 'a\nx\nc', 'a\nb\ny')
+  screen:expect([[
+    {7:  }a        │{7:  }a        │{7:  }^a        |
+    {7:  }{27:b}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:b}{4:        }|
+    {7:  }{27:c}{4:        }│{7:  }{27:c}{4:        }│{7:  }{27:y}{4:        }|
+    {1:~          }│{1:~          }│{1:~          }|*15
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  WriteDiffFiles3('a\nb\nc', 'a\nx\nc', 'y\ny\nc')
+  screen:expect([[
+    {7:  }{27:a}{4:        }│{7:  }{27:a}{4:        }│{7:  }{27:^y}{4:        }|
+    {7:  }{27:b}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:y}{4:        }|
+    {7:  }c        │{7:  }c        │{7:  }c        |
+    {1:~          }│{1:~          }│{1:~          }|*15
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  WriteDiffFiles3('a\nb\nc', 'a\nx\nc', 'a\ny\ny')
+  screen:expect([[
+    {7:  }a        │{7:  }a        │{7:  }^a        |
+    {7:  }{27:b}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:y}{4:        }|
+    {7:  }{27:c}{4:        }│{7:  }{27:c}{4:        }│{7:  }{27:y}{4:        }|
+    {1:~          }│{1:~          }│{1:~          }|*15
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  WriteDiffFiles3('a\nb\nc', 'a\nx\nc', 'y\ny\ny')
+  screen:expect([[
+    {7:  }{27:a}{4:        }│{7:  }{27:a}{4:        }│{7:  }{27:^y}{4:        }|
+    {7:  }{27:b}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:y}{4:        }|
+    {7:  }{27:c}{4:        }│{7:  }{27:c}{4:        }│{7:  }{27:y}{4:        }|
+    {1:~          }│{1:~          }│{1:~          }|*15
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  WriteDiffFiles3('a\nb\nc', 'a\nx\nx', 'y\ny\nc')
+  screen:expect([[
+    {7:  }{27:a}{4:        }│{7:  }{27:a}{4:        }│{7:  }{27:^y}{4:        }|
+    {7:  }{27:b}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:y}{4:        }|
+    {7:  }{27:c}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:c}{4:        }|
+    {1:~          }│{1:~          }│{1:~          }|*15
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  WriteDiffFiles3('a\nb\nc', 'x\nx\nc', 'a\ny\ny')
+  screen:expect([[
+    {7:  }{27:a}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:^a}{4:        }|
+    {7:  }{27:b}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:y}{4:        }|
+    {7:  }{27:c}{4:        }│{7:  }{27:c}{4:        }│{7:  }{27:y}{4:        }|
+    {1:~          }│{1:~          }│{1:~          }|*15
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  WriteDiffFiles3('a\nb\nc\nd\ne', 'a\nx\nc\nx\ne', 'y\ny\ny\nd\ne')
+  screen:expect([[
+    {7:  }{27:a}{4:        }│{7:  }{27:a}{4:        }│{7:  }{27:^y}{4:        }|
+    {7:  }{27:b}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:y}{4:        }|
+    {7:  }{27:c}{4:        }│{7:  }{27:c}{4:        }│{7:  }{27:y}{4:        }|
+    {7:  }{27:d}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:d}{4:        }|
+    {7:  }e        │{7:  }e        │{7:  }e        |
+    {1:~          }│{1:~          }│{1:~          }|*13
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  WriteDiffFiles3('a\nb\nc\nd\ne', 'a\nx\nc\nx\ne', 'y\ny\ny\ny\ne')
+  screen:expect([[
+    {7:  }{27:a}{4:        }│{7:  }{27:a}{4:        }│{7:  }{27:^y}{4:        }|
+    {7:  }{27:b}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:y}{4:        }|
+    {7:  }{27:c}{4:        }│{7:  }{27:c}{4:        }│{7:  }{27:y}{4:        }|
+    {7:  }{27:d}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:y}{4:        }|
+    {7:  }e        │{7:  }e        │{7:  }e        |
+    {1:~          }│{1:~          }│{1:~          }|*13
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  WriteDiffFiles3('a\nb\nc\nd\ne', 'a\nx\nc\nx\ne', 'y\ny\ny\ny\ny')
+  screen:expect([[
+    {7:  }{27:a}{4:        }│{7:  }{27:a}{4:        }│{7:  }{27:^y}{4:        }|
+    {7:  }{27:b}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:y}{4:        }|
+    {7:  }{27:c}{4:        }│{7:  }{27:c}{4:        }│{7:  }{27:y}{4:        }|
+    {7:  }{27:d}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:y}{4:        }|
+    {7:  }{27:e}{4:        }│{7:  }{27:e}{4:        }│{7:  }{27:y}{4:        }|
+    {1:~          }│{1:~          }│{1:~          }|*13
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  WriteDiffFiles3('a\nb\nc\nd\ne', 'a\nx\nc\nx\ne', 'a\ny\ny\nd\ne')
+  screen:expect([[
+    {7:  }a        │{7:  }a        │{7:  }^a        |
+    {7:  }{27:b}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:y}{4:        }|
+    {7:  }{27:c}{4:        }│{7:  }{27:c}{4:        }│{7:  }{27:y}{4:        }|
+    {7:  }{27:d}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:d}{4:        }|
+    {7:  }e        │{7:  }e        │{7:  }e        |
+    {1:~          }│{1:~          }│{1:~          }|*13
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  WriteDiffFiles3('a\nb\nc\nd\ne', 'a\nx\nc\nx\ne', 'a\ny\ny\ny\ne')
+  screen:expect([[
+    {7:  }a        │{7:  }a        │{7:  }^a        |
+    {7:  }{27:b}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:y}{4:        }|
+    {7:  }{27:c}{4:        }│{7:  }{27:c}{4:        }│{7:  }{27:y}{4:        }|
+    {7:  }{27:d}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:y}{4:        }|
+    {7:  }e        │{7:  }e        │{7:  }e        |
+    {1:~          }│{1:~          }│{1:~          }|*13
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  WriteDiffFiles3('a\nb\nc\nd\ne', 'a\nx\nc\nx\ne', 'a\ny\ny\ny\ny')
+  screen:expect([[
+    {7:  }a        │{7:  }a        │{7:  }^a        |
+    {7:  }{27:b}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:y}{4:        }|
+    {7:  }{27:c}{4:        }│{7:  }{27:c}{4:        }│{7:  }{27:y}{4:        }|
+    {7:  }{27:d}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:y}{4:        }|
+    {7:  }{27:e}{4:        }│{7:  }{27:e}{4:        }│{7:  }{27:y}{4:        }|
+    {1:~          }│{1:~          }│{1:~          }|*13
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  WriteDiffFiles3('a\nb\nc\nd\ne', 'a\nx\nc\nx\ne', 'a\nb\ny\nd\ne')
+  screen:expect([[
+    {7:  }a        │{7:  }a        │{7:  }^a        |
+    {7:  }{27:b}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:b}{4:        }|
+    {7:  }{27:c}{4:        }│{7:  }{27:c}{4:        }│{7:  }{27:y}{4:        }|
+    {7:  }{27:d}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:d}{4:        }|
+    {7:  }e        │{7:  }e        │{7:  }e        |
+    {1:~          }│{1:~          }│{1:~          }|*13
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  WriteDiffFiles3('a\nb\nc\nd\ne', 'a\nx\nc\nx\ne', 'a\nb\ny\ny\ne')
+  screen:expect([[
+    {7:  }a        │{7:  }a        │{7:  }^a        |
+    {7:  }{27:b}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:b}{4:        }|
+    {7:  }{27:c}{4:        }│{7:  }{27:c}{4:        }│{7:  }{27:y}{4:        }|
+    {7:  }{27:d}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:y}{4:        }|
+    {7:  }e        │{7:  }e        │{7:  }e        |
+    {1:~          }│{1:~          }│{1:~          }|*13
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  WriteDiffFiles3('a\nb\nc\nd\ne', 'a\nx\nc\nx\ne', 'a\nb\ny\ny\ny')
+  screen:expect([[
+    {7:  }a        │{7:  }a        │{7:  }^a        |
+    {7:  }{27:b}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:b}{4:        }|
+    {7:  }{27:c}{4:        }│{7:  }{27:c}{4:        }│{7:  }{27:y}{4:        }|
+    {7:  }{27:d}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:y}{4:        }|
+    {7:  }{27:e}{4:        }│{7:  }{27:e}{4:        }│{7:  }{27:y}{4:        }|
+    {1:~          }│{1:~          }│{1:~          }|*13
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  WriteDiffFiles3('a\nb', 'x\nb', 'y\ny')
+  write_file('Xdiin1', 'a\nb')
+  write_file('Xdinew1', 'x\nb')
+  write_file('Xdiout1', '1c1')
+  write_file('Xdiin2', 'a\nb')
+  write_file('Xdinew2', 'y\ny')
+  write_file('Xdiout2', '1c1\n2c2')
+  command('set diffexpr=DiffExprStub()')
+  screen:expect([[
+    {7:  }{27:a}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:^y}{4:        }|
+    {7:  }{27:b}{4:        }│{7:  }{27:b}{4:        }│{7:  }{27:y}{4:        }|
+    {1:~          }│{1:~          }│{1:~          }|*16
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+  command('set diffexpr&')
+
+  WriteDiffFiles3('a\nb\nc\nd', 'x\nb\nx\nd', 'y\ny\nc\nd')
+  write_file('Xdiin1', 'a\nb\nc\nd')
+  write_file('Xdinew1', 'x\nb\nx\nd')
+  write_file('Xdiout1', '1c1\n3c3')
+  write_file('Xdiin2', 'a\nb\nc\nd')
+  write_file('Xdinew2', 'y\ny\nc\nd')
+  write_file('Xdiout2', '1c1\n2c2')
+  command('set diffexpr=DiffExprStub()')
+  screen:expect([[
+    {7:  }{27:a}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:^y}{4:        }|
+    {7:  }{27:b}{4:        }│{7:  }{27:b}{4:        }│{7:  }{27:y}{4:        }|
+    {7:  }{27:c}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:c}{4:        }|
+    {7:  }d        │{7:  }d        │{7:  }d        |
+    {1:~          }│{1:~          }│{1:~          }|*14
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+  command('set diffexpr&')
+
+  WriteDiffFiles3('a\nb\nc\nd', 'x\nb\nx\nd', 'y\ny\ny\nd')
+  write_file('Xdiin1', 'a\nb\nc\nd')
+  write_file('Xdinew1', 'x\nb\nx\nd')
+  write_file('Xdiout1', '1c1\n3c3')
+  write_file('Xdiin2', 'a\nb\nc\nd')
+  write_file('Xdinew2', 'y\ny\ny\nd')
+  write_file('Xdiout2', '1c1\n2,3c2,3')
+  command('set diffexpr=DiffExprStub()')
+  screen:expect([[
+    {7:  }{27:a}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:^y}{4:        }|
+    {7:  }{27:b}{4:        }│{7:  }{27:b}{4:        }│{7:  }{27:y}{4:        }|
+    {7:  }{27:c}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:y}{4:        }|
+    {7:  }d        │{7:  }d        │{7:  }d        |
+    {1:~          }│{1:~          }│{1:~          }|*14
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+  command('set diffexpr&')
+
+  WriteDiffFiles3('a\nb\nc\nd', 'x\nb\nx\nd', 'y\ny\ny\ny')
+  write_file('Xdiin1', 'a\nb\nc\nd')
+  write_file('Xdinew1', 'x\nb\nx\nd')
+  write_file('Xdiout1', '1c1\n3c3')
+  write_file('Xdiin2', 'a\nb\nc\nd')
+  write_file('Xdinew2', 'y\ny\ny\ny')
+  write_file('Xdiout2', '1c1\n2,4c2,4')
+  command('set diffexpr=DiffExprStub()')
+  screen:expect([[
+    {7:  }{27:a}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:^y}{4:        }|
+    {7:  }{27:b}{4:        }│{7:  }{27:b}{4:        }│{7:  }{27:y}{4:        }|
+    {7:  }{27:c}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:y}{4:        }|
+    {7:  }{27:d}{4:        }│{7:  }{27:d}{4:        }│{7:  }{27:y}{4:        }|
+    {1:~          }│{1:~          }│{1:~          }|*14
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+  command('set diffexpr&')
+
+  WriteDiffFiles3('a\nb\nc', 'a\nx\nc', 'b\nc')
+  screen:expect([[
+    {7:  }{27:a}{4:        }│{7:  }{27:a}{4:        }│{7:  }{27:^b}{4:        }|
+    {7:  }{27:b}{4:        }│{7:  }{27:x}{4:        }│{7:  }{23:---------}|
+    {7:  }c        │{7:  }c        │{7:  }c        |
+    {1:~          }│{1:~          }│{1:~          }|*15
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  WriteDiffFiles3('a\nb\nc', 'a\nx\nc', 'c')
+  screen:expect([[
+    {7:  }{4:a        }│{7:  }{4:a        }│{7:  }{23:---------}|
+    {7:  }{27:b}{4:        }│{7:  }{27:x}{4:        }│{7:  }{23:---------}|
+    {7:  }c        │{7:  }c        │{7:  }^c        |
+    {1:~          }│{1:~          }│{1:~          }|*15
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  WriteDiffFiles3('a\nb\nc', 'a\nx\nc', '')
+  screen:expect([[
+    {7:  }{4:a        }│{7:  }{4:a        }│{7:  }{23:---------}|
+    {7:  }{27:b}{4:        }│{7:  }{27:x}{4:        }│{7:  }{23:---------}|
+    {7:  }{4:c        }│{7:  }{4:c        }│{7:  }{23:---------}|
+    {1:~          }│{1:~          }│{7:  }^         |
+    {1:~          }│{1:~          }│{1:~          }|*14
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  WriteDiffFiles3('a\nb\nc', 'a\nx\nc', 'a\nc')
+  screen:expect([[
+    {7:  }a        │{7:  }a        │{7:  }^a        |
+    {7:  }{27:b}{4:        }│{7:  }{27:x}{4:        }│{7:  }{23:---------}|
+    {7:  }c        │{7:  }c        │{7:  }c        |
+    {1:~          }│{1:~          }│{1:~          }|*15
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  WriteDiffFiles3('a\nb\nc', 'a\nx\nc', 'a')
+  screen:expect([[
+    {7:  }a        │{7:  }a        │{7:  }^a        |
+    {7:  }{27:b}{4:        }│{7:  }{27:x}{4:        }│{7:  }{23:---------}|
+    {7:  }{4:c        }│{7:  }{4:c        }│{7:  }{23:---------}|
+    {1:~          }│{1:~          }│{1:~          }|*15
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  WriteDiffFiles3('a\nb\nc', 'a\nx\nc', 'b')
+  screen:expect([[
+    {7:  }{27:a}{4:        }│{7:  }{27:a}{4:        }│{7:  }{27:^b}{4:        }|
+    {7:  }{27:b}{4:        }│{7:  }{27:x}{4:        }│{7:  }{23:---------}|
+    {7:  }{4:c        }│{7:  }{4:c        }│{7:  }{23:---------}|
+    {1:~          }│{1:~          }│{1:~          }|*15
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  WriteDiffFiles3('a\nb\nc\nd\ne', 'a\nx\nc\nx\ne', 'd\ne')
+  screen:expect([[
+    {7:  }{27:a}{4:        }│{7:  }{27:a}{4:        }│{7:  }{27:^d}{4:        }|
+    {7:  }{27:b}{4:        }│{7:  }{27:x}{4:        }│{7:  }{23:---------}|
+    {7:  }{4:c        }│{7:  }{4:c        }│{7:  }{23:---------}|
+    {7:  }{27:d}{4:        }│{7:  }{27:x}{4:        }│{7:  }{23:---------}|
+    {7:  }e        │{7:  }e        │{7:  }e        |
+    {1:~          }│{1:~          }│{1:~          }|*13
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  WriteDiffFiles3('a\nb\nc\nd\ne', 'a\nx\nc\nx\ne', 'e')
+  screen:expect([[
+    {7:  }{4:a        }│{7:  }{4:a        }│{7:  }{23:---------}|
+    {7:  }{27:b}{4:        }│{7:  }{27:x}{4:        }│{7:  }{23:---------}|
+    {7:  }{4:c        }│{7:  }{4:c        }│{7:  }{23:---------}|
+    {7:  }{27:d}{4:        }│{7:  }{27:x}{4:        }│{7:  }{23:---------}|
+    {7:  }e        │{7:  }e        │{7:  }^e        |
+    {1:~          }│{1:~          }│{1:~          }|*13
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  WriteDiffFiles3('a\nb\nc\nd\ne', 'a\nx\nc\nx\ne', 'a\nd\ne')
+  screen:expect([[
+    {7:  }a        │{7:  }a        │{7:  }^a        |
+    {7:  }{27:b}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:d}{4:        }|
+    {7:  }{4:c        }│{7:  }{4:c        }│{7:  }{23:---------}|
+    {7:  }{27:d}{4:        }│{7:  }{27:x}{4:        }│{7:  }{23:---------}|
+    {7:  }e        │{7:  }e        │{7:  }e        |
+    {1:~          }│{1:~          }│{1:~          }|*13
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  WriteDiffFiles3('a\nb\nc\nd\ne', 'a\nx\nc\nx\ne', 'a\ne')
+  screen:expect([[
+    {7:  }a        │{7:  }a        │{7:  }^a        |
+    {7:  }{27:b}{4:        }│{7:  }{27:x}{4:        }│{7:  }{23:---------}|
+    {7:  }{4:c        }│{7:  }{4:c        }│{7:  }{23:---------}|
+    {7:  }{27:d}{4:        }│{7:  }{27:x}{4:        }│{7:  }{23:---------}|
+    {7:  }e        │{7:  }e        │{7:  }e        |
+    {1:~          }│{1:~          }│{1:~          }|*13
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  WriteDiffFiles3('a\nb\nc\nd\ne', 'a\nx\nc\nx\ne', 'a')
+  screen:expect([[
+    {7:  }a        │{7:  }a        │{7:  }^a        |
+    {7:  }{27:b}{4:        }│{7:  }{27:x}{4:        }│{7:  }{23:---------}|
+    {7:  }{4:c        }│{7:  }{4:c        }│{7:  }{23:---------}|
+    {7:  }{27:d}{4:        }│{7:  }{27:x}{4:        }│{7:  }{23:---------}|
+    {7:  }{4:e        }│{7:  }{4:e        }│{7:  }{23:---------}|
+    {1:~          }│{1:~          }│{1:~          }|*13
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  WriteDiffFiles3('a\nb\nc\nd\ne', 'a\nx\nc\nx\ne', 'a\nb\nd\ne')
+  screen:expect([[
+    {7:  }a        │{7:  }a        │{7:  }^a        |
+    {7:  }{27:b}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:b}{4:        }|
+    {7:  }{27:c}{4:        }│{7:  }{27:c}{4:        }│{7:  }{27:d}{4:        }|
+    {7:  }{27:d}{4:        }│{7:  }{27:x}{4:        }│{7:  }{23:---------}|
+    {7:  }e        │{7:  }e        │{7:  }e        |
+    {1:~          }│{1:~          }│{1:~          }|*13
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  WriteDiffFiles3('a\nb\nc\nd\ne', 'a\nx\nc\nx\ne', 'a\nb\ne')
+  screen:expect([[
+    {7:  }a        │{7:  }a        │{7:  }^a        |
+    {7:  }{27:b}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:b}{4:        }|
+    {7:  }{4:c        }│{7:  }{4:c        }│{7:  }{23:---------}|
+    {7:  }{27:d}{4:        }│{7:  }{27:x}{4:        }│{7:  }{23:---------}|
+    {7:  }e        │{7:  }e        │{7:  }e        |
+    {1:~          }│{1:~          }│{1:~          }|*13
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  WriteDiffFiles3('a\nb\nc\nd\ne', 'a\nx\nc\nx\ne', 'a\nb')
+  screen:expect([[
+    {7:  }a        │{7:  }a        │{7:  }^a        |
+    {7:  }{27:b}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:b}{4:        }|
+    {7:  }{4:c        }│{7:  }{4:c        }│{7:  }{23:---------}|
+    {7:  }{27:d}{4:        }│{7:  }{27:x}{4:        }│{7:  }{23:---------}|
+    {7:  }{4:e        }│{7:  }{4:e        }│{7:  }{23:---------}|
+    {1:~          }│{1:~          }│{1:~          }|*13
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  WriteDiffFiles3('a\nb\nc', 'a\nx\nc', 'a\ny\nb\nc')
+  screen:expect([[
+    {7:  }a        │{7:  }a        │{7:  }^a        |
+    {7:  }{27:b}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:y}{4:        }|
+    {7:  }{23:---------}│{7:  }{23:---------}│{7:  }{22:b        }|
+    {7:  }c        │{7:  }c        │{7:  }c        |
+    {1:~          }│{1:~          }│{1:~          }|*14
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  WriteDiffFiles3('a\nb\nc', 'a\nx\nc', 'a\nb\ny\nc')
+  screen:expect([[
+    {7:  }a        │{7:  }a        │{7:  }^a        |
+    {7:  }{27:b}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:b}{4:        }|
+    {7:  }{23:---------}│{7:  }{23:---------}│{7:  }{22:y        }|
+    {7:  }c        │{7:  }c        │{7:  }c        |
+    {1:~          }│{1:~          }│{1:~          }|*14
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  WriteDiffFiles3('a\nb\nc', 'd\ne', 'b\nf')
+  screen:expect([[
+    {7:  }{27:a}{4:        }│{7:  }{27:d}{4:        }│{7:  }{27:^b}{4:        }|
+    {7:  }{27:b}{4:        }│{7:  }{27:e}{4:        }│{7:  }{27:f}{4:        }|
+    {7:  }{22:c        }│{7:  }{23:---------}│{7:  }{23:---------}|
+    {1:~          }│{1:~          }│{1:~          }|*15
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  WriteDiffFiles3('a\nb\nc', 'd\ne', 'b')
+  screen:expect([[
+    {7:  }{27:a}{4:        }│{7:  }{27:d}{4:        }│{7:  }{27:^b}{4:        }|
+    {7:  }{27:b}{4:        }│{7:  }{27:e}{4:        }│{7:  }{23:---------}|
+    {7:  }{22:c        }│{7:  }{23:---------}│{7:  }{23:---------}|
+    {1:~          }│{1:~          }│{1:~          }|*15
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  -- File 3 overlaps twice, 2nd overlap completely within the existing block.
+  WriteDiffFiles3('foo\na\nb\nc\nbar', 'foo\nw\nx\ny\nz\nbar', 'foo\n1\na\nb\n2\nbar')
+  screen:expect([[
+    {7:  }foo      │{7:  }foo      │{7:  }^foo      |
+    {7:  }{27:a}{4:        }│{7:  }{27:w}{4:        }│{7:  }{27:1}{4:        }|
+    {7:  }{27:b}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:a}{4:        }|
+    {7:  }{27:c}{4:        }│{7:  }{27:y}{4:        }│{7:  }{27:b}{4:        }|
+    {7:  }{23:---------}│{7:  }{27:z}{4:        }│{7:  }{27:2}{4:        }|
+    {7:  }bar      │{7:  }bar      │{7:  }bar      |
+    {1:~          }│{1:~          }│{1:~          }|*12
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  -- File 3 overlaps twice, 2nd overlap extends beyond existing block on new
+  -- side. Make sure we don't over-extend the range and hit 'bar'.
+  WriteDiffFiles3('foo\na\nb\nc\nd\nbar', 'foo\nw\nx\ny\nz\nu\nbar', 'foo\n1\na\nb\n2\nd\nbar')
+  screen:expect([[
+    {7:  }foo      │{7:  }foo      │{7:  }^foo      |
+    {7:  }{27:a}{4:        }│{7:  }{27:w}{4:        }│{7:  }{27:1}{4:        }|
+    {7:  }{27:b}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:a}{4:        }|
+    {7:  }{27:c}{4:        }│{7:  }{27:y}{4:        }│{7:  }{27:b}{4:        }|
+    {7:  }{27:d}{4:        }│{7:  }{27:z}{4:        }│{7:  }{27:2}{4:        }|
+    {7:  }{23:---------}│{7:  }{27:u}{4:        }│{7:  }{27:d}{4:        }|
+    {7:  }bar      │{7:  }bar      │{7:  }bar      |
+    {1:~          }│{1:~          }│{1:~          }|*11
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  -- Chained overlaps. File 3's 2nd overlap spans two diff blocks and is longer
+  -- than the 2nd one.
+  WriteDiffFiles3(
+    'foo\na\nb\nc\nd\ne\nf\nbar',
+    'foo\nw\nx\ny\nz\ne\nu\nbar',
+    'foo\n1\nb\n2\n3\nd\n4\nf\nbar'
+  )
+  screen:expect([[
+    {7:  }foo      │{7:  }foo      │{7:  }^foo      |
+    {7:  }{27:a}{4:        }│{7:  }{27:w}{4:        }│{7:  }{27:1}{4:        }|
+    {7:  }{27:b}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:b}{4:        }|
+    {7:  }{27:c}{4:        }│{7:  }{27:y}{4:        }│{7:  }{27:2}{4:        }|
+    {7:  }{27:d}{4:        }│{7:  }{27:z}{4:        }│{7:  }{27:3}{4:        }|
+    {7:  }{27:e}{4:        }│{7:  }{27:e}{4:        }│{7:  }{27:d}{4:        }|
+    {7:  }{27:f}{4:        }│{7:  }{27:u}{4:        }│{7:  }{27:4}{4:        }|
+    {7:  }{23:---------}│{7:  }{23:---------}│{7:  }{22:f        }|
+    {7:  }bar      │{7:  }bar      │{7:  }bar      |
+    {1:~          }│{1:~          }│{1:~          }|*9
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  -- File 3 has 2 overlaps. An add and a delete. First overlap's expansion hits
+  -- the 2nd one. Make sure we adjust the diff block to have fewer lines.
+  WriteDiffFiles3('foo\na\nb\nbar', 'foo\nx\ny\nbar', 'foo\n1\na\nbar')
+  screen:expect([[
+    {7:  }foo      │{7:  }foo      │{7:  }^foo      |
+    {7:  }{27:a}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:1}{4:        }|
+    {7:  }{27:b}{4:        }│{7:  }{27:y}{4:        }│{7:  }{27:a}{4:        }|
+    {7:  }bar      │{7:  }bar      │{7:  }bar      |
+    {1:~          }│{1:~          }│{1:~          }|*14
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+
+  -- File 3 has 2 overlaps. An add and another add. First overlap's expansion hits
+  -- the 2nd one. Make sure we adjust the diff block to have more lines.
+  WriteDiffFiles3('foo\na\nb\nc\nd\nbar', 'foo\nw\nx\ny\nz\nu\nbar', 'foo\n1\na\nb\n3\n4\nd\nbar')
+  screen:expect([[
+    {7:  }foo      │{7:  }foo      │{7:  }^foo      |
+    {7:  }{27:a}{4:        }│{7:  }{27:w}{4:        }│{7:  }{27:1}{4:        }|
+    {7:  }{27:b}{4:        }│{7:  }{27:x}{4:        }│{7:  }{27:a}{4:        }|
+    {7:  }{27:c}{4:        }│{7:  }{27:y}{4:        }│{7:  }{27:b}{4:        }|
+    {7:  }{27:d}{4:        }│{7:  }{27:z}{4:        }│{7:  }{27:3}{4:        }|
+    {7:  }{23:---------}│{7:  }{27:u}{4:        }│{7:  }{27:4}{4:        }|
+    {7:  }{23:---------}│{7:  }{23:---------}│{7:  }{22:d        }|
+    {7:  }bar      │{7:  }bar      │{7:  }bar      |
+    {1:~          }│{1:~          }│{1:~          }|*10
+    {2:Xdifile1    Xdifile2    }{3:Xdifile3   }|
+                                       |
+  ]])
+end)
+
+-- oldtest: Test_diff_topline_noscroll()
+it('diff mode does not scroll with line("w0")', function()
+  local screen = Screen.new(45, 20)
+  exec([[
+    set scrolloff=5
+    call setline(1, range(1,60))
+    vnew
+    call setline(1, range(1,10) + range(50,60))
+    windo diffthis
+    norm! G
+    exe "norm! 30\<C-y>"
+  ]])
+  screen:expect([[
+    {7:  }9                   │{7:  }9                   |
+    {7:  }10                  │{7:  }10                  |
+    {7:  }{23:--------------------}│{7:  }{22:11                  }|
+    {7:  }{23:--------------------}│{7:  }{22:12                  }|
+    {7:  }{23:--------------------}│{7:  }{22:13                  }|
+    {7:  }{23:--------------------}│{7:  }{22:14                  }|
+    {7:  }{23:--------------------}│{7:  }{22:15                  }|
+    {7:  }{23:--------------------}│{7:  }{22:16                  }|
+    {7:  }{23:--------------------}│{7:  }{22:17                  }|
+    {7:  }{23:--------------------}│{7:  }{22:18                  }|
+    {7:  }{23:--------------------}│{7:  }{22:19                  }|
+    {7:  }{23:--------------------}│{7:  }{22:20                  }|
+    {7:  }{23:--------------------}│{7:  }{22:^21                  }|
+    {7:  }{23:--------------------}│{7:  }{22:22                  }|
+    {7:  }{23:--------------------}│{7:  }{22:23                  }|
+    {7:  }{23:--------------------}│{7:  }{22:24                  }|
+    {7:  }{23:--------------------}│{7:  }{22:25                  }|
+    {7:  }{23:--------------------}│{7:  }{22:26                  }|
+    {2:[No Name] [+]          }{3:[No Name] [+]         }|
+                                                 |
+  ]])
+  command([[echo line('w0', 1001)]])
+  screen:expect([[
+    {7:  }9                   │{7:  }9                   |
+    {7:  }10                  │{7:  }10                  |
+    {7:  }{23:--------------------}│{7:  }{22:11                  }|
+    {7:  }{23:--------------------}│{7:  }{22:12                  }|
+    {7:  }{23:--------------------}│{7:  }{22:13                  }|
+    {7:  }{23:--------------------}│{7:  }{22:14                  }|
+    {7:  }{23:--------------------}│{7:  }{22:15                  }|
+    {7:  }{23:--------------------}│{7:  }{22:16                  }|
+    {7:  }{23:--------------------}│{7:  }{22:17                  }|
+    {7:  }{23:--------------------}│{7:  }{22:18                  }|
+    {7:  }{23:--------------------}│{7:  }{22:19                  }|
+    {7:  }{23:--------------------}│{7:  }{22:20                  }|
+    {7:  }{23:--------------------}│{7:  }{22:^21                  }|
+    {7:  }{23:--------------------}│{7:  }{22:22                  }|
+    {7:  }{23:--------------------}│{7:  }{22:23                  }|
+    {7:  }{23:--------------------}│{7:  }{22:24                  }|
+    {7:  }{23:--------------------}│{7:  }{22:25                  }|
+    {7:  }{23:--------------------}│{7:  }{22:26                  }|
+    {2:[No Name] [+]          }{3:[No Name] [+]         }|
+    9                                            |
+  ]])
+  feed('<C-W>p')
+  screen:expect([[
+    {7:  }{23:--------------------}│{7:  }{22:39                  }|
+    {7:  }{23:--------------------}│{7:  }{22:40                  }|
+    {7:  }{23:--------------------}│{7:  }{22:41                  }|
+    {7:  }{23:--------------------}│{7:  }{22:42                  }|
+    {7:  }{23:--------------------}│{7:  }{22:43                  }|
+    {7:  }{23:--------------------}│{7:  }{22:44                  }|
+    {7:  }{23:--------------------}│{7:  }{22:45                  }|
+    {7:  }{23:--------------------}│{7:  }{22:46                  }|
+    {7:  }{23:--------------------}│{7:  }{22:47                  }|
+    {7:  }{23:--------------------}│{7:  }{22:48                  }|
+    {7:  }{23:--------------------}│{7:  }{22:49                  }|
+    {7:  }^50                  │{7:  }50                  |
+    {7:  }51                  │{7:  }51                  |
+    {7:  }52                  │{7:  }52                  |
+    {7:  }53                  │{7:  }53                  |
+    {7:  }54                  │{7:  }54                  |
+    {7:  }55                  │{7:  }55                  |
+    {7:+ }{13:+--  5 lines: 56····}│{7:+ }{13:+--  5 lines: 56····}|
+    {3:[No Name] [+]          }{2:[No Name] [+]         }|
+    9                                            |
+  ]])
+  feed('<C-W>p')
+  screen:expect([[
+    {7:  }{23:--------------------}│{7:  }{22:39                  }|
+    {7:  }{23:--------------------}│{7:  }{22:40                  }|
+    {7:  }{23:--------------------}│{7:  }{22:41                  }|
+    {7:  }{23:--------------------}│{7:  }{22:42                  }|
+    {7:  }{23:--------------------}│{7:  }{22:43                  }|
+    {7:  }{23:--------------------}│{7:  }{22:^44                  }|
+    {7:  }{23:--------------------}│{7:  }{22:45                  }|
+    {7:  }{23:--------------------}│{7:  }{22:46                  }|
+    {7:  }{23:--------------------}│{7:  }{22:47                  }|
+    {7:  }{23:--------------------}│{7:  }{22:48                  }|
+    {7:  }{23:--------------------}│{7:  }{22:49                  }|
+    {7:  }50                  │{7:  }50                  |
+    {7:  }51                  │{7:  }51                  |
+    {7:  }52                  │{7:  }52                  |
+    {7:  }53                  │{7:  }53                  |
+    {7:  }54                  │{7:  }54                  |
+    {7:  }55                  │{7:  }55                  |
+    {7:+ }{13:+--  5 lines: 56····}│{7:+ }{13:+--  5 lines: 56····}|
+    {2:[No Name] [+]          }{3:[No Name] [+]         }|
+    9                                            |
+  ]])
+end)
