@@ -32,7 +32,6 @@ describe('highlight: `:syntax manual`', function()
   before_each(function()
     clear()
     screen = Screen.new(20, 5)
-    screen:attach()
     -- syntax highlight for vimscript's "echo"
   end)
 
@@ -93,7 +92,6 @@ describe('highlight defaults', function()
       [100] = { foreground = Screen.colors.Red, background = Screen.colors.WebGreen },
       [101] = { italic = true },
     }
-    screen:attach()
   end)
 
   it('window status bar', function()
@@ -303,7 +301,6 @@ describe('highlight', function()
 
   it('Visual', function()
     local screen = Screen.new(45, 5)
-    screen:attach()
     insert([[
       line1 foo bar
     abcdefghijklmnopqrs
@@ -426,9 +423,89 @@ describe('highlight', function()
     ]])
   end)
 
+  it('blockwise Visual highlight with virtualedit=block #34235', function()
+    local screen = Screen.new(45, 5)
+    command('set virtualedit=block')
+    insert('foobar\nfoo')
+    feed('0<C-V>k$')
+    screen:expect([[
+      {17:foobar}^                                       |
+      {17:foo    }                                      |
+      {1:~                                            }|*2
+      {5:-- VISUAL BLOCK --}                           |
+    ]])
+    feed('10l')
+    screen:expect([[
+      {17:foobar          }^                             |
+      {17:foo              }                            |
+      {1:~                                            }|*2
+      {5:-- VISUAL BLOCK --}                           |
+    ]])
+  end)
+
+  it("is updated with H/L and 'scrolloff' #36059", function()
+    local screen = Screen.new(40, 10)
+    exec([[
+      call setline(1, map(range(1, 100), "'line ' .. v:val"))
+      set scrolloff=2 nostartofline
+      call cursor(50, 1)
+    ]])
+    feed('z<CR>V')
+    screen:expect([[
+      line 48                                 |
+      line 49                                 |
+      ^l{17:ine 50}                                 |
+      line 51                                 |
+      line 52                                 |
+      line 53                                 |
+      line 54                                 |
+      line 55                                 |
+      line 56                                 |
+      {5:-- VISUAL LINE --}                       |
+    ]])
+    feed('L')
+    screen:expect([[
+      line 48                                 |
+      line 49                                 |
+      {17:line 50}                                 |
+      {17:line 51}                                 |
+      {17:line 52}                                 |
+      {17:line 53}                                 |
+      ^l{17:ine 54}                                 |
+      line 55                                 |
+      line 56                                 |
+      {5:-- VISUAL LINE --}                       |
+    ]])
+    feed('<Esc>V')
+    screen:expect([[
+      line 48                                 |
+      line 49                                 |
+      line 50                                 |
+      line 51                                 |
+      line 52                                 |
+      line 53                                 |
+      ^l{17:ine 54}                                 |
+      line 55                                 |
+      line 56                                 |
+      {5:-- VISUAL LINE --}                       |
+    ]])
+    feed('H')
+    screen:expect([[
+      line 48                                 |
+      line 49                                 |
+      ^l{17:ine 50}                                 |
+      {17:line 51}                                 |
+      {17:line 52}                                 |
+      {17:line 53}                                 |
+      {17:line 54}                                 |
+      line 55                                 |
+      line 56                                 |
+      {5:-- VISUAL LINE --}                       |
+    ]])
+  end)
+
   it('cterm=standout gui=standout', function()
     local screen = Screen.new(20, 5)
-    screen:attach()
     screen:add_extra_attr_ids {
       [100] = {
         foreground = Screen.colors.Blue1,
@@ -454,7 +531,6 @@ describe('highlight', function()
 
   it('strikethrough', function()
     local screen = Screen.new(25, 6)
-    screen:attach()
     feed_command('syntax on')
     feed_command('syn keyword TmpKeyword foo')
     feed_command('hi! Awesome cterm=strikethrough gui=strikethrough')
@@ -490,7 +566,6 @@ describe('highlight', function()
         background = Screen.colors.Yellow,
       },
     }
-    screen:attach()
     feed_command('syntax on')
     feed_command('hi! Underlined cterm=underline gui=underline')
     feed_command('syn keyword Underlined foobar')
@@ -532,7 +607,6 @@ describe('highlight', function()
 
   it('guisp (special/undercurl)', function()
     local screen = Screen.new(25, 10)
-    screen:attach()
     feed_command('syntax on')
     feed_command('syn keyword TmpKeyword neovim')
     feed_command('syn keyword TmpKeyword1 special')
@@ -585,7 +659,6 @@ describe('highlight', function()
 
   it("'diff', syntax and extmark #23722", function()
     local screen = Screen.new(25, 10)
-    screen:attach()
     exec([[
       new
       call setline(1, ['', '01234 6789'])
@@ -596,13 +669,13 @@ describe('highlight', function()
     ]])
     screen:expect(
       [[
-      {1:  }^                       |
+      {1:  }{5:^                       }|
       {1:  }{2:01}{3:234 67}{2:89}{5:             }|
       {4:~                        }|*2
       {7:[No Name] [+]            }|
-      {1:  }                       |
       {1:  }{6:-----------------------}|
-      {4:~                        }|
+      {1:  }{6:-----------------------}|
+      {1:  }                       |
       {8:[No Name]                }|
                                |
     ]],
@@ -631,7 +704,6 @@ describe("'listchars' highlight", function()
   before_each(function()
     clear()
     screen = Screen.new(20, 5)
-    screen:attach()
   end)
 
   it("'cursorline' and 'cursorcolumn'", function()
@@ -873,7 +945,6 @@ describe('CursorLine and CursorLineNr highlights', function()
       [100] = { background = Screen.colors.LightRed },
       [101] = { foreground = Screen.colors.SlateBlue, background = Screen.colors.Grey90 },
     }
-    screen:attach()
 
     command('filetype on')
     command('syntax on')
@@ -906,7 +977,6 @@ describe('CursorLine and CursorLineNr highlights', function()
       [102] = { foreground = Screen.colors.Grey0, background = Screen.colors.Grey100 },
       [103] = { foreground = Screen.colors.Yellow1, background = Screen.colors.Grey100 },
     }
-    screen:attach()
 
     feed_command('set wrap cursorline')
     feed_command('set showbreak=>>>')
@@ -957,7 +1027,6 @@ describe('CursorLine and CursorLineNr highlights', function()
       [102] = { foreground = Screen.colors.Black, background = Screen.colors.Grey100 },
       [103] = { foreground = Screen.colors.WebGreen, background = Screen.colors.Red },
     }
-    screen:attach()
 
     command('set wrap cursorline cursorlineopt=screenline')
     command('set showbreak=>>>')
@@ -1057,31 +1126,11 @@ describe('CursorLine and CursorLineNr highlights', function()
       {101:  }{100:>>>}aaaaaaaaaaaa   |
                           |
     ]])
-
-    command('inoremap <F2> <Cmd>call cursor(1, 1)<CR>')
-    feed('A')
-    screen:expect([[
-      {103:0 }øøøøøøøøøøøøøøøøøø|
-      {101:  }{100:>>>}{102:øøøøøøøøøøøø^   }|
-      {101:1 }aaaaaaaaaaaaaaaaaa|
-      {101:  }{100:>>>}aaaaaaaaaaaa   |
-      {5:-- INSERT --}        |
-    ]])
-
-    feed('<F2>')
-    screen:expect([[
-      {103:0 }{102:^øøøøøøøøøøøøøøøøøø}|
-      {101:  }{100:>>>}øøøøøøøøøøøø   |
-      {101:1 }aaaaaaaaaaaaaaaaaa|
-      {101:  }{100:>>>}aaaaaaaaaaaa   |
-      {5:-- INSERT --}        |
-    ]])
   end)
 
   -- oldtest: Test_cursorline_screenline_resize()
   it("'cursorlineopt' screenline is updated on window resize", function()
     local screen = Screen.new(75, 8)
-    screen:attach()
     exec([[
       50vnew
       call setline(1, repeat('xyz ', 30))
@@ -1116,6 +1165,47 @@ describe('CursorLine and CursorLineNr highlights', function()
     ]])
   end)
 
+  -- oldtest: Test_cursorline_screenline_update()
+  it("'cursorlineopt' screenline is updated on various movements", function()
+    local screen = Screen.new(75, 8)
+    exec([[
+      func TestRetab()
+        let w = winwidth(0)
+        call cursor([1, w + 1, 0, w + 1])
+        call line('w0')
+        retab 8
+      endfunc
+
+      call setline(1, repeat('xyz ', 30))
+      set cursorline cursorlineopt=screenline tabstop=8
+      inoremap <F2> <Cmd>call cursor(1, 1)<CR>
+      inoremap <F3> <Cmd>call TestRetab()<CR>
+    ]])
+
+    feed('A')
+    screen:expect([[
+      xyz xyz xyz xyz xyz xyz xyz xyz xyz xyz xyz xyz xyz xyz xyz xyz xyz xyz xyz|
+      {21: xyz xyz xyz xyz xyz xyz xyz xyz xyz xyz xyz ^                              }|
+      {1:~                                                                          }|*5
+      {5:-- INSERT --}                                                               |
+    ]])
+    feed('<F2>')
+    screen:expect([[
+      {21:^xyz xyz xyz xyz xyz xyz xyz xyz xyz xyz xyz xyz xyz xyz xyz xyz xyz xyz xyz}|
+       xyz xyz xyz xyz xyz xyz xyz xyz xyz xyz xyz                               |
+      {1:~                                                                          }|*5
+      {5:-- INSERT --}                                                               |
+    ]])
+    feed('<F3>')
+    screen:expect([[
+      xyz xyz xyz xyz xyz xyz xyz xyz xyz xyz xyz xyz xyz xyz xyz xyz xyz xyz xyz|
+      {21:^ xyz xyz xyz xyz xyz xyz xyz xyz xyz xyz xyz                               }|
+      {1:~                                                                          }|*5
+      {5:-- INSERT --}                                                               |
+    ]])
+    feed('<Esc>')
+  end)
+
   -- oldtest: Test_cursorline_after_yank()
   it('always updated. vim-patch:8.1.0849', function()
     local screen = Screen.new(50, 5)
@@ -1123,7 +1213,6 @@ describe('CursorLine and CursorLineNr highlights', function()
       [100] = { background = Screen.colors.LightRed },
       [101] = { foreground = Screen.colors.SlateBlue, background = Screen.colors.Grey90 },
     }
-    screen:attach()
     command('set cursorline relativenumber')
     command('call setline(1, ["","1","2","3",""])')
     feed('Gy3k')
@@ -1151,7 +1240,6 @@ describe('CursorLine and CursorLineNr highlights', function()
       [100] = { foreground = Screen.colors.SlateBlue, background = Screen.colors.Grey90 },
       [101] = { background = Screen.colors.LightRed },
     }
-    screen:attach()
     command('set cursorline')
     command('call setline(1, repeat(["abc"], 50))')
     feed('V<C-f>zbkkjk')
@@ -1166,7 +1254,6 @@ describe('CursorLine and CursorLineNr highlights', function()
   -- oldtest: Test_cursorline_callback()
   it('is updated if cursor is moved up from timer vim-patch:8.2.4591', function()
     local screen = Screen.new(50, 8)
-    screen:attach()
     exec([[
       call setline(1, ['aaaaa', 'bbbbb', 'ccccc', 'ddddd'])
       set cursorline
@@ -1175,30 +1262,25 @@ describe('CursorLine and CursorLineNr highlights', function()
       func Func(timer)
         call cursor(2, 1)
       endfunc
-
-      call timer_start(300, 'Func')
     ]])
-    screen:expect({
-      grid = [[
+    screen:expect([[
       aaaaa                                             |
       bbbbb                                             |
       ccccc                                             |
       {21:^ddddd                                             }|
       {1:~                                                 }|*3
                                                         |
-    ]],
-      timeout = 100,
-    })
-    screen:expect({
-      grid = [[
+    ]])
+    command([[call timer_start(300, 'Func')]])
+    screen:expect_unchanged(false, 100)
+    screen:expect([[
       aaaaa                                             |
       {21:^bbbbb                                             }|
       ccccc                                             |
       ddddd                                             |
       {1:~                                                 }|*3
                                                         |
-    ]],
-    })
+    ]])
   end)
 
   it('with split windows in diff mode', function()
@@ -1207,7 +1289,6 @@ describe('CursorLine and CursorLineNr highlights', function()
       [100] = { background = Screen.colors.Plum1, underline = true },
       [101] = { background = Screen.colors.Red1, bold = true, underline = true },
     }
-    screen:attach()
 
     command('hi CursorLine ctermbg=red ctermfg=white guibg=red guifg=white')
     command('set cursorline')
@@ -1267,7 +1348,6 @@ describe('CursorLine and CursorLineNr highlights', function()
     screen:add_extra_attr_ids {
       [100] = { foreground = Screen.colors.Black, bold = true, background = Screen.colors.Grey100 },
     }
-    screen:attach()
 
     command('hi CursorLine guibg=red guifg=white')
     command('hi CursorLineNr guibg=white guifg=black gui=bold')
@@ -1308,7 +1388,6 @@ describe('CursorColumn highlight', function()
     screen:add_extra_attr_ids {
       [100] = { background = Screen.colors.Blue1 },
     }
-    screen:attach()
   end)
 
   it('is updated when pressing "i" on a TAB character', function()
@@ -1356,30 +1435,25 @@ describe('CursorColumn highlight', function()
       func Func(timer)
         call cursor(1, 1)
       endfunc
-
-      call timer_start(300, 'Func')
     ]])
-    screen:expect({
-      grid = [[
+    screen:expect([[
       aaaa{21:a}                                             |
       bbbb{21:b}                                             |
       cccc{21:c}                                             |
       dddd^d                                             |
       {1:~                                                 }|*3
                                                         |
-    ]],
-      timeout = 100,
-    })
-    screen:expect({
-      grid = [[
+    ]])
+    command([[call timer_start(300, 'Func')]])
+    screen:expect_unchanged(false, 100)
+    screen:expect([[
       ^aaaaa                                             |
       {21:b}bbbb                                             |
       {21:c}cccc                                             |
       {21:d}dddd                                             |
       {1:~                                                 }|*3
                                                         |
-    ]],
-    })
+    ]])
   end)
 
   it('is not shown on current line with virtualedit', function()
@@ -1435,7 +1509,6 @@ describe('ColorColumn highlight', function()
       [101] = { background = Screen.colors.LightRed },
       [102] = { foreground = Screen.colors.Blue1, bold = true, background = Screen.colors.LightRed },
     }
-    screen:attach()
   end)
 
   -- oldtest: Test_colorcolumn()
@@ -1538,7 +1611,6 @@ describe('MsgSeparator highlight and msgsep fillchar', function()
       [12] = { background = Screen.colors.Gray60, bold = true, foreground = tonumber('0x297d4e') },
       [13] = { background = tonumber('0xff4cff'), bold = true, foreground = tonumber('0xb200ff') },
     })
-    screen:attach()
   end)
 
   it('works', function()
@@ -1653,7 +1725,6 @@ describe("'winhighlight' highlight", function()
   before_each(function()
     clear()
     screen = Screen.new(20, 8)
-    screen:attach()
     screen:set_default_attr_ids {
       [0] = { bold = true, foreground = Screen.colors.Blue },
       [1] = { background = Screen.colors.DarkBlue },
@@ -1691,6 +1762,7 @@ describe("'winhighlight' highlight", function()
       [29] = { foreground = Screen.colors.Blue1, background = Screen.colors.Red, bold = true },
       [30] = { background = tonumber('0xff8800') },
       [31] = { background = tonumber('0xff8800'), bold = true, foreground = Screen.colors.Blue },
+      [32] = { bold = true, reverse = true, background = Screen.colors.DarkGreen },
     }
     command('hi Background1 guibg=DarkBlue')
     command('hi Background2 guibg=DarkGreen')
@@ -2253,10 +2325,10 @@ describe("'winhighlight' highlight", function()
       some text                               |
       more tex^t                               |
       {0:~                                       }|
-      {3:[No Name]                        }{1:2,9 All}|
+      {3:[No Name]                        }{11:2,9 All}|
       some text                               |
       more text                               |
-      {4:[No Name]                        }{1:1,1 All}|
+      {4:[No Name]                        }{14:1,1 All}|
                                               |
     ]],
     }
@@ -2267,10 +2339,10 @@ describe("'winhighlight' highlight", function()
       some text                               |
       more tex^t                               |
       {0:~                                       }|
-      {3:[No Name]                        }{5:2,9 All}|
+      {3:[No Name]                        }{32:2,9 All}|
       some text                               |
       more text                               |
-      {4:[No Name]                        }{1:1,1 All}|
+      {4:[No Name]                        }{14:1,1 All}|
                                               |
     ]],
     }
@@ -2281,10 +2353,10 @@ describe("'winhighlight' highlight", function()
       some tex^t                               |
       more text                               |
       {0:~                                       }|
-      {3:[No Name]                        }{5:1,9 All}|
+      {3:[No Name]                        }{32:1,9 All}|
       some text                               |
       more text                               |
-      {4:[No Name]                        }{1:1,1 All}|
+      {4:[No Name]                        }{14:1,1 All}|
                                               |
     ]],
     }
@@ -2316,7 +2388,6 @@ describe('highlight namespaces', function()
   before_each(function()
     clear()
     screen = Screen.new(25, 10)
-    screen:attach()
     screen:set_default_attr_ids {
       [1] = { foreground = Screen.colors.Blue, bold = true },
       [2] = { background = Screen.colors.DarkGrey },
@@ -2425,16 +2496,24 @@ describe('highlight namespaces', function()
   end)
 
   it('winhl does not accept invalid value #24586', function()
-    local res = exec_lua([[
-      local curwin = vim.api.nvim_get_current_win()
-      vim.api.nvim_command("set winhl=Normal:Visual")
-      local _, msg = pcall(vim.api.nvim_command,"set winhl='Normal:Wrong'")
-      return { msg, vim.wo[curwin].winhl }
-    ]])
-    eq({
-      'Vim(set):E5248: Invalid character in group name',
-      'Normal:Visual',
-    }, res)
+    command('set winhl=Normal:Visual')
+    for _, cmd in ipairs({
+      [[set winhl='Normal:Wrong']],
+      [[set winhl=Normal:Wrong']],
+      [[set winhl='Normal:Wrong]],
+    }) do
+      local res = exec_lua(
+        [[
+          local _, msg = pcall(vim.api.nvim_command, ...)
+          return { msg, vim.wo.winhl }
+        ]],
+        cmd
+      )
+      eq({
+        'Vim(set):E5248: Invalid character in group name',
+        'Normal:Visual',
+      }, res)
+    end
   end)
 
   it('Normal in set_hl #25474', function()
@@ -2458,10 +2537,8 @@ describe('highlight namespaces', function()
 end)
 
 describe('synIDattr()', function()
-  local screen
   before_each(function()
     clear()
-    screen = Screen.new(50, 7)
     command('highlight Normal ctermfg=252 guifg=#ff0000 guibg=Black')
     -- Salmon #fa8072 Maroon #800000
     command(
@@ -2486,7 +2563,7 @@ describe('synIDattr()', function()
   end)
 
   it('returns gui-color if RGB-capable UI is attached', function()
-    screen:attach({ rgb = true })
+    local _ = Screen.new(50, 7, { rgb = true })
     eq('#ff0000', eval('synIDattr(hlID("Normal"),  "fg")'))
     eq('Black', eval('synIDattr(hlID("Normal"),  "bg")'))
     eq('Salmon', eval('synIDattr(hlID("Keyword"), "fg")'))
@@ -2494,15 +2571,15 @@ describe('synIDattr()', function()
   end)
 
   it('returns #RRGGBB value for fg#/bg#/sp#', function()
-    screen:attach({ rgb = true })
+    local _ = Screen.new(50, 7, { rgb = true })
     eq('#ff0000', eval('synIDattr(hlID("Normal"),  "fg#")'))
     eq('#000000', eval('synIDattr(hlID("Normal"),  "bg#")'))
     eq('#fa8072', eval('synIDattr(hlID("Keyword"), "fg#")'))
     eq('#800000', eval('synIDattr(hlID("Keyword"), "sp#")'))
   end)
 
-  it('returns color number if non-GUI', function()
-    screen:attach({ rgb = false })
+  it('returns color number if non-RGB GUI', function()
+    local _ = Screen.new(50, 7, { rgb = false })
     eq('252', eval('synIDattr(hlID("Normal"), "fg")'))
     eq('79', eval('synIDattr(hlID("Keyword"), "fg")'))
   end)
@@ -2527,10 +2604,8 @@ describe('synIDattr()', function()
 end)
 
 describe('fg/bg special colors', function()
-  local screen
   before_each(function()
     clear()
-    screen = Screen.new(50, 7)
     command('highlight Normal ctermfg=145 ctermbg=16 guifg=#ff0000 guibg=Black')
     command('highlight Visual ctermfg=bg ctermbg=fg guifg=bg guibg=fg guisp=bg')
   end)
@@ -2549,7 +2624,7 @@ describe('fg/bg special colors', function()
   end)
 
   it('resolve to "Normal" values in RGB-capable UI', function()
-    screen:attach({ rgb = true })
+    local _ = Screen.new(50, 7, { rgb = true })
     eq('bg', eval('synIDattr(hlID("Visual"), "fg")'))
     eq(eval('synIDattr(hlID("Normal"), "bg#")'), eval('synIDattr(hlID("Visual"), "fg#")'))
     eq('fg', eval('synIDattr(hlID("Visual"), "bg")'))
@@ -2559,7 +2634,7 @@ describe('fg/bg special colors', function()
   end)
 
   it('resolve after the "Normal" group is modified', function()
-    screen:attach({ rgb = true })
+    local _ = Screen.new(50, 7, { rgb = true })
     local new_guibg = '#282c34'
     local new_guifg = '#abb2bf'
     command('highlight Normal guifg=' .. new_guifg .. ' guibg=' .. new_guibg)
