@@ -2,7 +2,7 @@ local t = require('test.testutil')
 local n = require('test.functional.testnvim')()
 local Screen = require('test.functional.ui.screen')
 
-local spawn, set_session, clear = n.spawn, n.set_session, n.clear
+local set_session, clear = n.set_session, n.clear
 local feed, command = n.feed, n.command
 local exec = n.exec
 local insert = n.insert
@@ -26,10 +26,9 @@ describe('screen', function()
   }
 
   before_each(function()
-    local screen_nvim = spawn(nvim_argv)
+    local screen_nvim = n.new_session(false, { args = nvim_argv, merge = false })
     set_session(screen_nvim)
     screen = Screen.new()
-    screen:attach()
   end)
 
   it('default initial screen', function()
@@ -47,8 +46,7 @@ local function screen_tests(linegrid)
 
   before_each(function()
     clear()
-    screen = Screen.new()
-    screen:attach({ rgb = true, ext_linegrid = linegrid })
+    screen = Screen.new(53, 14, { rgb = true, ext_linegrid = linegrid })
     screen:set_default_attr_ids({
       [0] = { bold = true, foreground = 255 },
       [1] = { bold = true, reverse = true },
@@ -647,6 +645,59 @@ local function screen_tests(linegrid)
                             |
       ]])
     end)
+
+    it('clamps &cmdheight for current tabpage', function()
+      command('set cmdheight=10 laststatus=2')
+      screen:expect([[
+        ^                                                     |
+        {0:~                                                    }|*2
+        {1:[No Name]                                            }|
+                                                             |*10
+      ]])
+      screen:try_resize(53, 8)
+      screen:expect([[
+        ^                                                     |
+        {1:[No Name]                                            }|
+                                                             |*6
+      ]])
+      eq(6, api.nvim_get_option_value('cmdheight', {}))
+    end)
+
+    it('clamps &cmdheight for another tabpage #31380', function()
+      command('tabnew')
+      command('set cmdheight=9 laststatus=2')
+      screen:expect([[
+        {4: [No Name] }{2: [No Name] }{3:                              }{4:X}|
+        ^                                                     |
+        {0:~                                                    }|*2
+        {1:[No Name]                                            }|
+                                                             |*9
+      ]])
+      command('tabprev')
+      screen:expect([[
+        {2: [No Name] }{4: [No Name] }{3:                              }{4:X}|
+        ^                                                     |
+        {0:~                                                    }|*10
+        {1:[No Name]                                            }|
+                                                             |
+      ]])
+      screen:try_resize(53, 8)
+      screen:expect([[
+        {2: [No Name] }{4: [No Name] }{3:                              }{4:X}|
+        ^                                                     |
+        {0:~                                                    }|*4
+        {1:[No Name]                                            }|
+                                                             |
+      ]])
+      command('tabnext')
+      screen:expect([[
+        {4: [No Name] }{2: [No Name] }{3:                              }{4:X}|
+        ^                                                     |
+        {1:[No Name]                                            }|
+                                                             |*5
+      ]])
+      eq(5, api.nvim_get_option_value('cmdheight', {}))
+    end)
   end)
 
   describe('press enter', function()
@@ -715,10 +766,9 @@ describe('Screen default colors', function()
       'colorscheme vim',
       '--embed',
     }
-    local screen_nvim = spawn(nvim_argv)
+    local screen_nvim = n.new_session(false, { args = nvim_argv, merge = false })
     set_session(screen_nvim)
-    screen = Screen.new()
-    screen:attach(termcolors and { rgb = true, ext_termcolors = true } or { rgb = true })
+    screen = Screen.new(53, 14, { rgb = true, ext_termcolors = termcolors or nil })
   end
 
   it('are dark per default', function()
@@ -777,7 +827,6 @@ end)
 it('CTRL-F or CTRL-B scrolls a page after UI attach/resize #20605', function()
   clear()
   local screen = Screen.new(100, 100)
-  screen:attach()
   eq(100, api.nvim_get_option_value('lines', {}))
   eq(99, api.nvim_get_option_value('window', {}))
   eq(99, api.nvim_win_get_height(0))
@@ -810,7 +859,6 @@ end)
 it("showcmd doesn't cause empty grid_line with redrawdebug=compositor #22593", function()
   clear()
   local screen = Screen.new(30, 2)
-  screen:attach()
   command('set showcmd redrawdebug=compositor')
   feed('d')
   screen:expect {
@@ -824,7 +872,6 @@ end)
 it("scrolling in narrow window doesn't draw over separator #29033", function()
   clear()
   local screen = Screen.new(60, 8)
-  screen:attach()
   feed('100Oa<Esc>gg')
   exec([[
     set number nowrap
