@@ -24,11 +24,9 @@ for _, model in ipairs(mousemodels) do
     before_each(function()
       clear()
       screen = Screen.new(40, 8)
-      screen:set_default_attr_ids({
-        [0] = { bold = true, foreground = Screen.colors.Blue }, -- NonText
-        [1] = { bold = true, reverse = true }, -- StatusLine
-      })
-      screen:attach()
+      screen:add_extra_attr_ids {
+        [100] = { bold = true, reverse = true, foreground = Screen.colors.Blue },
+      }
       command('set laststatus=2 mousemodel=' .. model)
       exec([=[
         function! MyClickFunc(minwid, clicks, button, mods)
@@ -63,6 +61,22 @@ for _, model in ipairs(mousemodels) do
       eq('0 3 r', eval('g:testvar'))
       api.nvim_input_mouse('right', 'press', '', 0, 6, 28)
       eq('0 4 r', eval('g:testvar'))
+      api.nvim_input_mouse('x1', 'press', '', 0, 6, 17)
+      eq('0 1 x1', eval('g:testvar'))
+      api.nvim_input_mouse('x1', 'press', '', 0, 6, 17)
+      eq('0 2 x1', eval('g:testvar'))
+      api.nvim_input_mouse('x1', 'press', '', 0, 6, 17)
+      eq('0 3 x1', eval('g:testvar'))
+      api.nvim_input_mouse('x1', 'press', '', 0, 6, 17)
+      eq('0 4 x1', eval('g:testvar'))
+      api.nvim_input_mouse('x2', 'press', '', 0, 6, 28)
+      eq('0 1 x2', eval('g:testvar'))
+      api.nvim_input_mouse('x2', 'press', '', 0, 6, 28)
+      eq('0 2 x2', eval('g:testvar'))
+      api.nvim_input_mouse('x2', 'press', '', 0, 6, 28)
+      eq('0 3 x2', eval('g:testvar'))
+      api.nvim_input_mouse('x2', 'press', '', 0, 6, 28)
+      eq('0 4 x2', eval('g:testvar'))
     end)
 
     it('works with control characters and highlight', function()
@@ -70,8 +84,8 @@ for _, model in ipairs(mousemodels) do
       screen:expect {
         grid = [[
         ^                                        |
-        {0:~                                       }|*5
-        {1:^I}{0:^A^I^A^I}{1:^A                            }|
+        {1:~                                       }|*5
+        {3:^I}{100:^A^I^A^I}{3:^A                            }|
                                                 |
       ]],
       }
@@ -194,8 +208,8 @@ for _, model in ipairs(mousemodels) do
       screen:expect {
         grid = [[
         ^                    │                   |
-        {0:~                   }│{0:~                  }|*5
-        {1:Clicky stuff                Clicky stuff}|
+        {1:~                   }│{1:~                  }|*5
+        {3:Clicky stuff                Clicky stuff}|
                                                 |
       ]],
       }
@@ -239,14 +253,9 @@ describe('global statusline', function()
   before_each(function()
     clear()
     screen = Screen.new(60, 16)
-    screen:attach()
-    screen:set_default_attr_ids({
-      [1] = { bold = true, foreground = Screen.colors.Blue },
-      [2] = { bold = true, reverse = true },
-      [3] = { bold = true },
-      [4] = { reverse = true },
-      [5] = { bold = true, foreground = Screen.colors.Fuchsia },
-    })
+    screen:add_extra_attr_ids {
+      [100] = { foreground = Screen.colors.Magenta1, bold = true },
+    }
     command('set laststatus=3')
     command('set ruler')
   end)
@@ -255,7 +264,7 @@ describe('global statusline', function()
     screen:expect([[
       ^                                                            |
       {1:~                                                           }|*13
-      {2:[No Name]                                 0,0-1          All}|
+      {3:[No Name]                                 0,0-1          All}|
                                                                   |
     ]])
 
@@ -264,8 +273,8 @@ describe('global statusline', function()
                                                                   |*2
       ^                                                            |
       {1:~                                                           }|*11
-      {2:[No Name] [+]                             3,1            All}|
-      {3:-- INSERT --}                                                |
+      {3:[No Name] [+]                             3,1            All}|
+      {5:-- INSERT --}                                                |
     ]])
   end)
 
@@ -282,7 +291,7 @@ describe('global statusline', function()
       ────────────────────┴────────────────┴─┤{1:~                   }|
                                              │{1:~                   }|
       {1:~                                      }│{1:~                   }|*3
-      {2:[No Name]                                 0,0-1          All}|
+      {3:[No Name]                                 0,0-1          All}|
                                                                   |
     ]])
   end)
@@ -299,7 +308,7 @@ describe('global statusline', function()
     screen:expect([[
       ^                                                            |
       {1:~                                                           }|*13
-      {2:[No Name]                                 0,0-1          All}|
+      {3:[No Name]                                 0,0-1          All}|
                                                                   |
     ]])
 
@@ -308,15 +317,15 @@ describe('global statusline', function()
     screen:expect([[
                           │                │ │^                    |
       {1:~                   }│{1:~               }│{1:~}│{1:~                   }|*3
-      {1:~                   }│{4:< Name] 0,0-1   }│{1:~}│{1:~                   }|
+      {1:~                   }│{2:< Name] 0,0-1   }│{1:~}│{1:~                   }|
       {1:~                   }│                │{1:~}│{1:~                   }|
       {1:~                   }│{1:~               }│{1:~}│{1:~                   }|
-      {1:~                   }│{1:~               }│{1:~}│{2:<No Name] 0,0-1  All}|
+      {1:~                   }│{1:~               }│{1:~}│{3:<No Name] 0,0-1  All}|
       {1:~                   }│{1:~               }│{1:~}│                    |
-      {4:<No Name] 0,0-1  All < Name] 0,0-1    <}│{1:~                   }|
+      {2:<No Name] 0,0-1  All < Name] 0,0-1    <}│{1:~                   }|
                                              │{1:~                   }|
       {1:~                                      }│{1:~                   }|*3
-      {4:[No Name]            0,0-1          All <No Name] 0,0-1  All}|
+      {2:[No Name]            0,0-1          All <No Name] 0,0-1  All}|
                                                                   |
     ]])
 
@@ -332,7 +341,7 @@ describe('global statusline', function()
       ────────────────────┴────────────────┴─┤{1:~                   }|
                                              │{1:~                   }|
       {1:~                                      }│{1:~                   }|*3
-      {2:[No Name]                                 0,0-1          All}|
+      {3:[No Name]                                 0,0-1          All}|
                                                                   |
     ]])
 
@@ -340,12 +349,12 @@ describe('global statusline', function()
     screen:expect([[
                           │                │ │^                    |
       {1:~                   }│{1:~               }│{1:~}│{1:~                   }|*3
-      {1:~                   }│{4:< Name] 0,0-1   }│{1:~}│{1:~                   }|
+      {1:~                   }│{2:< Name] 0,0-1   }│{1:~}│{1:~                   }|
       {1:~                   }│                │{1:~}│{1:~                   }|
       {1:~                   }│{1:~               }│{1:~}│{1:~                   }|
-      {1:~                   }│{1:~               }│{1:~}│{2:<No Name] 0,0-1  All}|
+      {1:~                   }│{1:~               }│{1:~}│{3:<No Name] 0,0-1  All}|
       {1:~                   }│{1:~               }│{1:~}│                    |
-      {4:<No Name] 0,0-1  All < Name] 0,0-1    <}│{1:~                   }|
+      {2:<No Name] 0,0-1  All < Name] 0,0-1    <}│{1:~                   }|
                                              │{1:~                   }|
       {1:~                                      }│{1:~                   }|*4
                                                 0,0-1         All |
@@ -363,7 +372,7 @@ describe('global statusline', function()
       ────────────────────┴────────────────┴─┤{1:~                   }|
                                              │{1:~                   }|
       {1:~                                      }│{1:~                   }|*3
-      {2:[No Name]                                 0,0-1          All}|
+      {3:[No Name]                                 0,0-1          All}|
                                                                   |
     ]])
   end)
@@ -413,7 +422,7 @@ describe('global statusline', function()
       0004;<control>;Cc;0;BN;;;;;N;END OF TRANSMISSION;;;;        |
       0005;<control>;Cc;0;BN;;;;;N;ENQUIRY;;;;                    |
       ^0006;<control>;Cc;0;BN;;;;;N;ACKNOWLEDGE;;;;                |
-      {2:test/functional/fixtures/bigfile.txt      7,1            Top}|
+      {3:test/functional/fixtures/bigfile.txt      7,1            Top}|
                                                                   |
     ]])
     feed('j')
@@ -428,12 +437,12 @@ describe('global statusline', function()
       0005;<control>;Cc;0;BN;;;;;N;ENQUIRY;;;;                    |
       0006;<control>;Cc;0;BN;;;;;N;ACKNOWLEDGE;;;;                |
       ^0007;<control>;Cc;0;BN;;;;;N;BELL;;;;                       |
-      {2:test/functional/fixtures/bigfile.txt      8,1             0%}|
+      {3:test/functional/fixtures/bigfile.txt      8,1             0%}|
                                                                   |
     ]])
     api.nvim_set_option_value('showtabline', 2, {})
     screen:expect([[
-      {3: }{5:2}{3: t/f/f/bigfile.txt }{4:                                       }|
+      {5: }{100:2}{5: t/f/f/bigfile.txt }{2:                                       }|
                                                                   |
       {1:~                                                           }|*5
       ────────────────────────────────────────────────────────────|
@@ -443,12 +452,12 @@ describe('global statusline', function()
       0005;<control>;Cc;0;BN;;;;;N;ENQUIRY;;;;                    |
       0006;<control>;Cc;0;BN;;;;;N;ACKNOWLEDGE;;;;                |
       ^0007;<control>;Cc;0;BN;;;;;N;BELL;;;;                       |
-      {2:test/functional/fixtures/bigfile.txt      8,1             0%}|
+      {3:test/functional/fixtures/bigfile.txt      8,1             0%}|
                                                                   |
     ]])
     api.nvim_set_option_value('cmdheight', 0, {})
     screen:expect([[
-      {3: }{5:2}{3: t/f/f/bigfile.txt }{4:                                       }|
+      {5: }{100:2}{5: t/f/f/bigfile.txt }{2:                                       }|
                                                                   |
       {1:~                                                           }|*5
       ────────────────────────────────────────────────────────────|
@@ -459,11 +468,11 @@ describe('global statusline', function()
       0005;<control>;Cc;0;BN;;;;;N;ENQUIRY;;;;                    |
       0006;<control>;Cc;0;BN;;;;;N;ACKNOWLEDGE;;;;                |
       ^0007;<control>;Cc;0;BN;;;;;N;BELL;;;;                       |
-      {2:test/functional/fixtures/bigfile.txt      8,1             0%}|
+      {3:test/functional/fixtures/bigfile.txt      8,1             0%}|
     ]])
     api.nvim_set_option_value('cmdheight', 1, {})
     screen:expect([[
-      {3: }{5:2}{3: t/f/f/bigfile.txt }{4:                                       }|
+      {5: }{100:2}{5: t/f/f/bigfile.txt }{2:                                       }|
                                                                   |
       {1:~                                                           }|*5
       ────────────────────────────────────────────────────────────|
@@ -473,7 +482,7 @@ describe('global statusline', function()
       0005;<control>;Cc;0;BN;;;;;N;ENQUIRY;;;;                    |
       0006;<control>;Cc;0;BN;;;;;N;ACKNOWLEDGE;;;;                |
       ^0007;<control>;Cc;0;BN;;;;;N;BELL;;;;                       |
-      {2:test/functional/fixtures/bigfile.txt      8,1             0%}|
+      {3:test/functional/fixtures/bigfile.txt      8,1             0%}|
                                                                   |
     ]])
   end)
@@ -492,310 +501,305 @@ describe('global statusline', function()
       ────────────────────────────────────────────────────────────|
       ^                                                            |
       {1:~                                                           }|*6
+      {3:[No Name]                                 0,0-1          All}|
+                                                                  |
+    ]])
+
+    -- Shouldn't gain a hsep if the global statusline is turned off.
+    command('set laststatus=2')
+    eq('Vim(wincmd):E36: Not enough room', pcall_err(command, 'wincmd L'))
+    command('mode')
+    screen:expect([[
+                                                                  |
+      {1:~                                                           }|*5
       {2:[No Name]                                 0,0-1          All}|
+      ^                                                            |
+      {1:~                                                           }|*6
+      {3:[No Name]                                 0,0-1          All}|
                                                                   |
     ]])
   end)
 end)
 
-it('statusline does not crash if it has Arabic characters #19447', function()
-  clear()
-  api.nvim_set_option_value('statusline', 'غً', {})
-  api.nvim_set_option_value('laststatus', 2, {})
-  command('redraw!')
-  assert_alive()
-end)
+describe('statusline', function()
+  local screen
+  before_each(function()
+    clear()
+    screen = Screen.new(40, 8)
+    screen:add_extra_attr_ids {
+      [100] = { bold = true, reverse = true, foreground = Screen.colors.Blue },
+      [101] = { reverse = true, bold = true, foreground = Screen.colors.SlateBlue },
+    }
+  end)
 
-it('statusline is redrawn with :resize from <Cmd> mapping #19629', function()
-  clear()
-  local screen = Screen.new(40, 8)
-  screen:set_default_attr_ids({
-    [0] = { bold = true, foreground = Screen.colors.Blue }, -- NonText
-    [1] = { bold = true, reverse = true }, -- StatusLine
-  })
-  screen:attach()
-  exec([[
-    set laststatus=2
-    nnoremap <Up> <cmd>resize -1<CR>
-    nnoremap <Down> <cmd>resize +1<CR>
-  ]])
-  feed('<Up>')
-  screen:expect([[
-    ^                                        |
-    {0:~                                       }|*4
-    {1:[No Name]                               }|
-                                            |*2
-  ]])
-  feed('<Down>')
-  screen:expect([[
-    ^                                        |
-    {0:~                                       }|*5
-    {1:[No Name]                               }|
-                                            |
-  ]])
-end)
+  it('does not crash if it has Arabic characters #19447', function()
+    api.nvim_set_option_value('statusline', 'غً', {})
+    api.nvim_set_option_value('laststatus', 2, {})
+    command('redraw!')
+    assert_alive()
+  end)
 
-it('showcmdloc=statusline does not show if statusline is too narrow', function()
-  clear()
-  local screen = Screen.new(40, 8)
-  screen:set_default_attr_ids({
-    [0] = { bold = true, foreground = Screen.colors.Blue }, -- NonText
-    [1] = { bold = true, reverse = true }, -- StatusLine
-    [2] = { reverse = true }, -- StatusLineNC
-  })
-  screen:attach()
-  command('set showcmd')
-  command('set showcmdloc=statusline')
-  command('1vsplit')
-  screen:expect([[
-    ^ │                                      |
-    {0:~}│{0:~                                     }|*5
-    {1:< }{2:[No Name]                             }|
-                                            |
-  ]])
-  feed('1234')
-  screen:expect_unchanged()
-end)
+  it('is redrawn with :resize from <Cmd> mapping #19629', function()
+    exec([[
+      set laststatus=2
+      nnoremap <Up> <cmd>resize -1<CR>
+      nnoremap <Down> <cmd>resize +1<CR>
+    ]])
+    feed('<Up>')
+    screen:expect([[
+      ^                                        |
+      {1:~                                       }|*4
+      {3:[No Name]                               }|
+                                              |*2
+    ]])
+    feed('<Down>')
+    screen:expect([[
+      ^                                        |
+      {1:~                                       }|*5
+      {3:[No Name]                               }|
+                                              |
+    ]])
+  end)
 
-it('K_EVENT does not trigger a statusline redraw unnecessarily', function()
-  clear()
-  local screen = Screen.new(40, 8)
-  screen:attach()
-  -- does not redraw on vim.schedule (#17937)
-  command([[
-    set laststatus=2
-    let g:counter = 0
-    func Status()
-      let g:counter += 1
-      lua vim.schedule(function() end)
-      return g:counter
-    endfunc
-    set statusline=%!Status()
-  ]])
-  sleep(50)
-  eq(1, eval('g:counter < 50'), 'g:counter=' .. eval('g:counter'))
-  -- also in insert mode
-  feed('i')
-  sleep(50)
-  eq(1, eval('g:counter < 50'), 'g:counter=' .. eval('g:counter'))
-  -- does not redraw on timer call (#14303)
-  command([[
-    let g:counter = 0
-    func Timer(timer)
-    endfunc
-    call timer_start(1, 'Timer', {'repeat': 100})
-  ]])
-  sleep(50)
-  eq(1, eval('g:counter < 50'), 'g:counter=' .. eval('g:counter'))
-end)
+  it('does not contain showmcd with showcmdloc=statusline when too narrow', function()
+    command('set showcmd')
+    command('set showcmdloc=statusline')
+    command('1vsplit')
+    screen:expect([[
+      ^ │                                      |
+      {1:~}│{1:~                                     }|*5
+      {3:< }{2:[No Name]                             }|
+                                              |
+    ]])
+    feed('1234')
+    screen:expect_unchanged()
+  end)
 
-it('statusline is redrawn on various state changes', function()
-  clear()
-  local screen = Screen.new(40, 4)
-  screen:attach()
+  it('does not redraw unnecessarily after K_EVENT', function()
+    -- does not redraw on vim.schedule (#17937)
+    command([[
+      set laststatus=2
+      let g:counter = 0
+      func Status()
+        let g:counter += 1
+        lua vim.schedule(function() end)
+        return g:counter
+      endfunc
+      set statusline=%!Status()
+    ]])
+    sleep(50)
+    eq(1, eval('g:counter < 50'), 'g:counter=' .. eval('g:counter'))
+    -- also in insert mode
+    feed('i')
+    sleep(50)
+    eq(1, eval('g:counter < 50'), 'g:counter=' .. eval('g:counter'))
+    -- does not redraw on timer call (#14303)
+    command([[
+      let g:counter = 0
+      func Timer(timer)
+      endfunc
+      call timer_start(1, 'Timer', {'repeat': 100})
+    ]])
+    sleep(50)
+    eq(1, eval('g:counter < 50'), 'g:counter=' .. eval('g:counter'))
+  end)
 
-  -- recording state change #22683
-  command('set ls=2 stl=%{repeat(reg_recording(),5)}')
-  screen:expect([[
-    ^                                        |
-    {1:~                                       }|
-    {3:                                        }|
-                                            |
-  ]])
-  feed('qQ')
-  screen:expect([[
-    ^                                        |
-    {1:~                                       }|
-    {3:QQQQQ                                   }|
-    {5:recording @Q}                            |
-  ]])
-  feed('q')
-  screen:expect([[
-    ^                                        |
-    {1:~                                       }|
-    {3:                                        }|
-                                            |
-  ]])
+  it('is redrawn on various state changes', function()
+    -- recording state change #22683
+    command('set ls=2 stl=%{repeat(reg_recording(),5)}')
+    local s1 = [[
+      ^                                        |
+      {1:~                                       }|*5
+      {3:                                        }|
+                                              |
+    ]]
+    screen:expect(s1)
+    feed('qQ')
+    screen:expect([[
+      ^                                        |
+      {1:~                                       }|*5
+      {3:QQQQQ                                   }|
+      {5:recording @Q}                            |
+    ]])
+    feed('q')
+    screen:expect(s1)
 
-  -- Visual mode change #23932
-  command('set ls=2 stl=%{mode(1)}')
-  screen:expect([[
-    ^                                        |
-    {1:~                                       }|
-    {3:n                                       }|
-                                            |
-  ]])
-  feed('v')
-  screen:expect([[
-    ^                                        |
-    {1:~                                       }|
-    {3:v                                       }|
-    {5:-- VISUAL --}                            |
-  ]])
-  feed('V')
-  screen:expect([[
-    ^                                        |
-    {1:~                                       }|
-    {3:V                                       }|
-    {5:-- VISUAL LINE --}                       |
-  ]])
-  feed('<C-V>')
-  screen:expect([[
-    ^                                        |
-    {1:~                                       }|
-    {3:^V                                      }|
-    {5:-- VISUAL BLOCK --}                      |
-  ]])
-  feed('<Esc>')
-  screen:expect([[
-    ^                                        |
-    {1:~                                       }|
-    {3:n                                       }|
-                                            |
-  ]])
-end)
+    -- Visual mode change #23932
+    command('set ls=2 stl=%{mode(1)}')
+    local s2 = [[
+      ^                                        |
+      {1:~                                       }|*5
+      {3:n                                       }|
+                                              |
+    ]]
+    screen:expect(s2)
+    feed('v')
+    screen:expect([[
+      ^                                        |
+      {1:~                                       }|*5
+      {3:v                                       }|
+      {5:-- VISUAL --}                            |
+    ]])
+    feed('V')
+    screen:expect([[
+      ^                                        |
+      {1:~                                       }|*5
+      {3:V                                       }|
+      {5:-- VISUAL LINE --}                       |
+    ]])
+    feed('<C-V>')
+    screen:expect([[
+      ^                                        |
+      {1:~                                       }|*5
+      {3:^V                                      }|
+      {5:-- VISUAL BLOCK --}                      |
+    ]])
+    feed('<Esc>')
+    screen:expect(s2)
 
-it('ruler is redrawn in cmdline with redrawstatus #22804', function()
-  clear()
-  local screen = Screen.new(40, 2)
-  screen:attach()
-  command([[
-    let g:n = 'initial value'
-    set ls=1 ru ruf=%{g:n}
-    redraw
-    let g:n = 'other value'
-    redrawstatus
-  ]])
-  screen:expect([[
-    ^                                        |
-                          other value       |
-  ]])
-end)
+    -- Visual selection other end change #36280
+    exec([[
+      function! DebugVisualSelection()
+        return printf("v %s %s", col("v"), col("."))
+      endfunction
+      set statusline=%!DebugVisualSelection()
+    ]])
+    feed('iabc<Esc>v')
+    screen:expect([[
+      ab^c                                     |
+      {1:~                                       }|*5
+      {3:v 3 3                                   }|
+      {5:-- VISUAL --}                            |
+    ]])
+    feed('iw')
+    screen:expect([[
+      {17:ab}^c                                     |
+      {1:~                                       }|*5
+      {3:v 1 3                                   }|
+      {5:-- VISUAL --}                            |
+    ]])
+  end)
 
-it('shows correct ruler in cmdline with no statusline', function()
-  clear()
-  local screen = Screen.new(30, 8)
-  screen:set_default_attr_ids {
-    [1] = { bold = true, foreground = Screen.colors.Blue }, -- NonText
-    [2] = { bold = true, reverse = true }, -- StatusLine
-    [3] = { reverse = true }, -- StatusLineNC
-  }
-  screen:attach()
-  -- Use long ruler to check 'ruler' with 'rulerformat' set has correct width.
-  command [[
-    set ruler rulerformat=%{winnr()}longlonglong ls=0 winwidth=10
-    split
-    wincmd b
-    vsplit
-    wincmd t
-    wincmd |
-    mode
-  ]]
-  -- Window 1 is current. It has a statusline, so cmdline should show the
-  -- last window's ruler, which has no statusline.
-  command '1wincmd w'
-  screen:expect [[
-    ^                              |
-    {1:~                             }|*2
-    {2:[No Name]      1longlonglong  }|
-                   │              |
-    {1:~              }│{1:~             }|*2
-                   3longlonglong  |
-  ]]
-  -- Window 2 is current. It has no statusline, so cmdline should show its
-  -- ruler instead.
-  command '2wincmd w'
-  screen:expect [[
-                                  |
-    {1:~                             }|*2
-    {3:[No Name]      1longlonglong  }|
-    ^               │              |
-    {1:~              }│{1:~             }|*2
-                   2longlonglong  |
-  ]]
-  -- Window 3 is current. Cmdline should again show its ruler.
-  command '3wincmd w'
-  screen:expect [[
-                                  |
-    {1:~                             }|*2
-    {3:[No Name]      1longlonglong  }|
-                   │^              |
-    {1:~              }│{1:~             }|*2
-                   3longlonglong  |
-  ]]
-end)
+  it('ruler is redrawn in cmdline with redrawstatus #22804', function()
+    command([[
+      let g:n = 'initial value'
+      set ls=1 ru ruf=%{g:n}
+      redraw
+      let g:n = 'other value'
+      redrawstatus
+    ]])
+    screen:expect([[
+      ^                                        |
+      {1:~                                       }|*6
+                            other value       |
+    ]])
+  end)
 
-it('uses "stl" and "stlnc" fillchars even if they are the same #19803', function()
-  clear()
-  local screen = Screen.new(53, 4)
-  screen:attach()
-  screen:set_default_attr_ids({
-    [1] = { bold = true, foreground = Screen.colors.Blue }, -- NonText
-  })
-  command('hi clear StatusLine')
-  command('hi clear StatusLineNC')
-  command('vsplit')
-  screen:expect {
-    grid = [[
-    ^                          │                          |
-    {1:~                         }│{1:~                         }|
-    [No Name]                  [No Name]                 |
-                                                         |
-  ]],
-  }
-end)
+  it('hidden moves ruler to cmdline', function()
+    -- Use long ruler to check 'ruler' with 'rulerformat' set has correct width.
+    command [[
+      set ruler rulerformat=%{winnr()}longlonglong ls=0 winwidth=10
+      split
+      wincmd b
+      vsplit
+      wincmd t
+      wincmd |
+      mode
+    ]]
+    -- Window 1 is current. It has a statusline, so cmdline should show the
+    -- last window's ruler, which has no statusline.
+    command '1wincmd w'
+    screen:expect([[
+      ^                                        |
+      {1:~                                       }|*2
+      {3:[No Name]             1longlonglong     }|
+                          │                   |
+      {1:~                   }│{1:~                  }|*2
+                            3longlonglong     |
+    ]])
+    -- Window 2 is current. It has no statusline, so cmdline should show its
+    -- ruler instead.
+    command '2wincmd w'
+    screen:expect([[
+                                              |
+      {1:~                                       }|*2
+      {2:[No Name]             1longlonglong     }|
+      ^                    │                   |
+      {1:~                   }│{1:~                  }|*2
+                            2longlonglong     |
+    ]])
+    -- Window 3 is current. Cmdline should again show its ruler.
+    command '3wincmd w'
+    screen:expect([[
+                                              |
+      {1:~                                       }|*2
+      {2:[No Name]             1longlonglong     }|
+                          │^                   |
+      {1:~                   }│{1:~                  }|*2
+                            3longlonglong     |
+    ]])
+  end)
 
-it('showcmdloc=statusline works with vertical splits', function()
-  clear()
-  local screen = Screen.new(53, 4)
-  screen:set_default_attr_ids {
-    [1] = { bold = true, foreground = Screen.colors.Blue }, -- NonText
-    [2] = { bold = true, reverse = true }, -- StatusLine
-    [3] = { reverse = true }, -- StatusLineNC
-  }
-  screen:attach()
-  command('rightbelow vsplit')
-  command('set showcmd showcmdloc=statusline')
-  feed('1234')
-  screen:expect([[
-                              │^                          |
-    {1:~                         }│{1:~                         }|
-    {3:[No Name]                  }{2:[No Name]      1234       }|
-                                                         |
-  ]])
-  feed('<Esc>')
-  command('set laststatus=3')
-  feed('1234')
-  screen:expect([[
-                              │^                          |
-    {1:~                         }│{1:~                         }|
-    {2:[No Name]                                 1234       }|
-                                                         |
-  ]])
-end)
+  it('uses "stl" and "stlnc" fillchars even if they are the same #19803', function()
+    command('hi clear StatusLine')
+    command('hi clear StatusLineNC')
+    command('vsplit')
+    screen:expect([[
+      ^                    │                   |
+      {1:~                   }│{1:~                  }|*5
+      [No Name]            [No Name]          |
+                                              |
+    ]])
+  end)
 
-it('keymap is shown with vertical splits #27269', function()
-  clear()
-  local screen = Screen.new(53, 4)
-  screen:set_default_attr_ids {
-    [1] = { bold = true, foreground = Screen.colors.Blue }, -- NonText
-    [2] = { bold = true, reverse = true }, -- StatusLine
-    [3] = { reverse = true }, -- StatusLineNC
-  }
-  screen:attach()
-  command('setlocal keymap=dvorak')
-  command('rightbelow vsplit')
-  screen:expect([[
-                              │^                          |
-    {1:~                         }│{1:~                         }|
-    {3:[No Name]         <en-dv>  }{2:[No Name]         <en-dv> }|
-                                                         |
-  ]])
-  command('set laststatus=3')
-  screen:expect([[
-                              │^                          |
-    {1:~                         }│{1:~                         }|
-    {2:[No Name]                                    <en-dv> }|
-                                                         |
-  ]])
+  it('showcmdloc=statusline works with vertical splits', function()
+    command('rightbelow vsplit')
+    command('set showcmd showcmdloc=statusline')
+    feed('1234')
+    screen:expect([[
+                         │^                    |
+      {1:~                  }│{1:~                   }|*5
+      {2:[No Name]           }{3:[No Name] 1234      }|
+                                              |
+    ]])
+    feed('<Esc>')
+    command('set laststatus=3')
+    feed('1234')
+    screen:expect([[
+                         │^                    |
+      {1:~                  }│{1:~                   }|*5
+      {3:[No Name]                    1234       }|
+                                              |
+    ]])
+  end)
+
+  it('keymap is shown with vertical splits #27269', function()
+    command('setlocal keymap=dvorak')
+    command('rightbelow vsplit')
+    screen:expect([[
+                         │^                    |
+      {1:~                  }│{1:~                   }|*5
+      {2:[No Name]  <en-dv>  }{3:[No Name]   <en-dv> }|
+                                              |
+    ]])
+
+    command('set laststatus=3')
+    screen:expect([[
+                         │^                    |
+      {1:~                  }│{1:~                   }|*5
+      {3:[No Name]                       <en-dv> }|
+                                              |
+    ]])
+  end)
+
+  it("nested call from nvim_eval_statusline() doesn't overwrite items #32259", function()
+    exec_lua('vim.o.laststatus = 2')
+    exec_lua([[vim.o.statusline = '%#Special#B:%{nvim_eval_statusline("%f", []).str}']])
+    screen:expect([[
+      ^                                        |
+      {1:~                                       }|*5
+      {101:B:[No Name]                             }|
+                                              |
+    ]])
+  end)
 end)
