@@ -18,7 +18,6 @@ describe('winbar', function()
   before_each(function()
     clear()
     screen = Screen.new(60, 13)
-    screen:attach()
     screen:set_default_attr_ids({
       [1] = { bold = true },
       [2] = { reverse = true },
@@ -40,6 +39,16 @@ describe('winbar', function()
         bold = true,
         foreground = Screen.colors.Magenta,
       },
+      [12] = {
+        underline = true,
+        background = Screen.colors.Red,
+      },
+      [13] = {
+        underline = true,
+        bold = true,
+        foreground = Screen.colors.Blue,
+        background = Screen.colors.Red,
+      },
     })
     api.nvim_set_option_value('winbar', 'Set Up The Bars', {})
   end)
@@ -182,6 +191,18 @@ describe('winbar', function()
     ]])
   end)
 
+  it('works with combined highlight attributes', function()
+    command('hi Winbar guibg=red gui=underline')
+    command('hi Identifier guifg=blue gui=bold')
+    command('set winbar=Lookatmy%#Identifier#highlights')
+    screen:expect([[
+      {12:Lookatmy}{13:highlights                                          }|
+      ^                                                            |
+      {3:~                                                           }|*10
+                                                                  |
+    ]])
+  end)
+
   it('can be ruler', function()
     insert [[
       just some
@@ -519,6 +540,15 @@ describe('winbar', function()
     ]])
     eq('Vim(set):E36: Not enough room', pcall_err(command, 'set winbar=test'))
   end)
+
+  it('does not crash due to negative grid height #33176', function()
+    screen:try_resize(screen._width, 20)
+    command('botright split | belowright vsplit | 2wincmd w')
+    api.nvim_set_option_value('winfixheight', true, { scope = 'local', win = 0 })
+    api.nvim_win_set_height(0, 8)
+    feed('q:')
+    n.assert_alive()
+  end)
 end)
 
 describe('local winbar with tabs', function()
@@ -526,7 +556,6 @@ describe('local winbar with tabs', function()
   before_each(function()
     clear()
     screen = Screen.new(60, 10)
-    screen:attach()
     api.nvim_set_option_value('winbar', 'foo', { scope = 'local', win = 0 })
   end)
 
@@ -604,7 +633,6 @@ it('winbar works properly when redrawing is postponed #23534', function()
     },
   })
   local screen = Screen.new(60, 6)
-  screen:attach()
   screen:expect([[
     {5:(winbar)                                                    }|
     ^                                                            |
