@@ -16,19 +16,12 @@ local function shell_quote(str)
   return str
 end
 
---- This module uses functions from the context of the test runner.
+--- Functions executing in the context of the test runner (not the current nvim test session).
 --- @class test.testutil
 local M = {
   paths = Paths,
 }
 
---- @param p string
---- @return string
-local function relpath(p)
-  p = vim.fs.normalize(p)
-  return (p:gsub('^' .. uv.cwd, ''))
-end
-
 --- @param path string
 --- @return boolean
 function M.isdir(path)
@@ -42,6 +35,30 @@ function M.isdir(path)
   return stat.type == 'directory'
 end
 
+--- (Only on Windows) Replaces yucky "\\" slashes with delicious "/" slashes in a string, or all
+--- string values in a table (recursively).
+---
+--- @generic T: string|table
+--- @param obj T
+--- @return T|nil
+function M.fix_slashes(obj)
+  if not M.is_os('win') then
+    return obj
+  end
+  if type(obj) == 'string' then
+    local ret = string.gsub(obj, '\\', '/')
+    return ret
+  elseif type(obj) == 'table' then
+    --- @cast obj table<any,any>
+    local ret = {} --- @type table<any,any>
+    for k, v in pairs(obj) do
+      ret[k] = M.fix_slashes(v)
+    end
+    return ret
+  end
+  assert(false, 'expected string or table of strings, got ' .. type(obj))
+end
+
 --- @param ... string|string[]
 --- @return string
 function M.argss_to_cmd(...)
@@ -131,6 +148,7 @@ end
 --- @param actual string
 --- @return boolean
 function M.matches(pat, actual)
+  assert(pat and pat ~= '', 'pat must be a non-empty string')
   if nil ~= string.match(actual, pat) then
     return true
   end
@@ -143,7 +161,7 @@ end
 ---
 ---@param pat (string) Lua pattern to match lines in the log file
 ---@param logfile? (string) Full path to log file (default=$NVIM_LOG_FILE)
----@param nrlines? (number) Search up to this many log lines
+---@param nrlines? (number) Search up to this many log lines (default 10)
 ---@param inverse? (boolean) Assert that the pattern does NOT match.
 function M.assert_log(pat, logfile, nrlines, inverse)
   logfile = logfile or os.getenv('NVIM_LOG_FILE') or '.nvimlog'
@@ -369,51 +387,55 @@ function M.check_logs()
   )
 end
 
-local function sysname()
-  return uv.os_uname().sysname:lower()
-end
+local sysname = uv.os_uname().sysname:lower()
 
---- @param s 'win'|'mac'|'freebsd'|'openbsd'|'bsd'
+--- @param s 'win'|'mac'|'linux'|'freebsd'|'openbsd'|'bsd'
 --- @return boolean
 function M.is_os(s)
-  if not (s == 'win' or s == 'mac' or s == 'freebsd' or s == 'openbsd' or s == 'bsd') then
+  if
+    not (s == 'win' or s == 'mac' or s == 'linux' or s == 'freebsd' or s == 'openbsd' or s == 'bsd')
+  then
     error('unknown platform: ' .. tostring(s))
   end
   return not not (
-    (s == 'win' and (sysname():find('windows') or sysname():find('mingw')))
-    or (s == 'mac' and sysname() == 'darwin')
-    or (s == 'freebsd' and sysname() == 'freebsd')
-    or (s == 'openbsd' and sysname() == 'openbsd')
-    or (s == 'bsd' and sysname():find('bsd'))
+    (s == 'win' and (sysname:find('windows') or sysname:find('mingw')))
+    or (s == 'mac' and sysname == 'darwin')
+    or (s == 'linux' and sysname == 'linux')
+    or (s == 'freebsd' and sysname == 'freebsd')
+    or (s == 'openbsd' and sysname == 'openbsd')
+    or (s == 'bsd' and sysname:find('bsd'))
   )
 end
 
-local function tmpdir_get()
-  return os.getenv('TMPDIR') and os.getenv('TMPDIR') or os.getenv('TEMP')
+local architecture = uv.os_uname().machine
+
+--- @param s 'x86_64'|'arm64'
+--- @return boolean
+function M.is_arch(s)
+  if not (s == 'x86_64' or s == 'arm64') then
+    error('unknown architecture: ' .. tostring(s))
+  end
+  return s == architecture
 end
 
---- Is temp directory `dir` defined local to the project workspace?
---- @param dir string?
---- @return boolean
-local function tmpdir_is_local(dir)
-  return not not (dir and dir:find('Xtest'))
+function M.is_asan()
+  return M.paths.is_asan
 end
 
 local tmpname_id = 0
-local tmpdir = tmpdir_get()
+local tmpdir = os.getenv('TMPDIR') or os.getenv('TEMP')
+local tmpdir_is_local = not not (tmpdir and tmpdir:find('Xtest'))
 
---- Creates a new temporary file for use by tests.
-function M.tmpname()
-  if tmpdir_is_local(tmpdir) then
+local function get_tmpname()
+  if tmpdir_is_local then
     -- Cannot control os.tmpname() dir, so hack our own tmpname() impl.
     tmpname_id = tmpname_id + 1
     -- "…/Xtest_tmpdir/T42.7"
-    local fname = ('%s/%s.%d'):format(tmpdir, (_G._nvim_test_id or 'nvim-test'), tmpname_id)
-    io.open(fname, 'w'):close()
-    return fname
+    return ('%s/%s.%d'):format(tmpdir, (_G._nvim_test_id or 'nvim-test'), tmpname_id)
   end
 
   local fname = os.tmpname()
+
   if M.is_os('win') and fname:sub(1, 2) == '\\s' then
     -- In Windows tmpname() returns a filename starting with
     -- special sequence \s, prepend $TEMP path
@@ -422,7 +444,20 @@ function M.tmpname()
     -- In OS X /tmp links to /private/tmp
     return '/private' .. fname
   end
+  return fname
+end
 
+--- Generates a unique filepath for use by tests, in a test-specific "…/Xtest_tmpdir/T42.7"
+--- directory (which is cleaned up by the test runner).
+---
+--- @param create? boolean (default true) Create the file.
+--- @return string
+function M.tmpname(create)
+  local fname = get_tmpname()
+  os.remove(fname)
+  if create ~= false then
+    assert(io.open(fname, 'w')):close()
+  end
   return fname
 end
 
@@ -447,11 +482,12 @@ function M.check_cores(app, force) -- luacheck: ignore
   local random_skip = false
   -- Workspace-local $TMPDIR, scrubbed and pattern-escaped.
   -- "./Xtest-tmpdir/" => "Xtest%-tmpdir"
-  local local_tmpdir = (
-    tmpdir_is_local(tmpdir_get())
-      and relpath(tmpdir_get()):gsub('^[ ./]+', ''):gsub('%/+$', ''):gsub('([^%w])', '%%%1')
-    or nil
-  )
+  local local_tmpdir = nil
+  if tmpdir_is_local and tmpdir then
+    local_tmpdir =
+      vim.pesc(vim.fs.relpath(assert(vim.uv.cwd()), tmpdir):gsub('^[ ./]+', ''):gsub('%/+$', ''))
+  end
+
   local db_cmd --- @type string
   local test_glob_dir = os.getenv('NVIM_TEST_CORE_GLOB_DIRECTORY')
   if test_glob_dir and test_glob_dir ~= '' then
@@ -610,28 +646,9 @@ end
 --- @param leave_indent? integer
 --- @return string
 function M.dedent(str, leave_indent)
-  -- find minimum common indent across lines
-  local indent --- @type string?
-  for line in str:gmatch('[^\n]+') do
-    local line_indent = line:match('^%s+') or ''
-    if indent == nil or #line_indent < #indent then
-      indent = line_indent
-    end
-  end
-
-  if not indent or #indent == 0 then
-    -- no minimum common indent
-    return str
-  end
-
-  local left_indent = (' '):rep(leave_indent or 0)
-  -- create a pattern for the indent
-  indent = indent:gsub('%s', '[ \t]')
-  -- strip it from the first line
-  str = str:gsub('^' .. indent, left_indent)
-  -- strip it from the remaining lines
-  str = str:gsub('[\n]' .. indent, '\n' .. left_indent)
-  return str
+  -- Last blank line often has non-matching indent, so remove it.
+  str = str:gsub('\n[ ]+$', '\n')
+  return (vim.text.indent(leave_indent or 0, str))
 end
 
 function M.intchar2lua(ch)
@@ -827,4 +844,8 @@ function M.skip_fragile(pending_fn, cond)
   return false
 end
 
+function M.translations_enabled()
+  return M.paths.translations_enabled
+end
+
 return M
