@@ -2,5 +2,5 @@
 -- Busted started doing this to help provide more isolation.  See issue #62
 -- for more information about this.
 local ffi = require('ffi')
-local helpers = require('test.unit.helpers')(nil)
+local t = require('test.unit.testutil')
 local preprocess = require('test.unit.preprocess')
