@@ -141,6 +141,13 @@ local function filter_complex_blocks(body)
         or string.find(line, '_Float')
         or string.find(line, '__s128')
         or string.find(line, '__u128')
+        or string.find(line, '__SVFloat32_t')
+        or string.find(line, '__SVFloat64_t')
+        or string.find(line, '__SVBool_t')
+        or string.find(line, '__f32x4_t')
+        or string.find(line, '__f64x2_t')
+        or string.find(line, '__sv_f32_t')
+        or string.find(line, '__sv_f64_t')
         or string.find(line, 'msgpack_zone_push_finalizer')
         or string.find(line, 'msgpack_unpacker_reserve_buffer')
         or string.find(line, 'value_init_')
@@ -340,6 +347,9 @@ local function alloc_log_new()
         end
       end
     end
+    -- JIT-compiled FFI calls cannot call back into Lua, so disable JIT.
+    -- Ref: https://luajit.org/ext_ffi_semantics.html#callback
+    jit.off()
   end
 
   log.set_mocks = child_call(log.set_mocks)
